# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import string


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    from unicodedata import normalize, category

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


class ImportaDados():
    def __init__(self):
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.dir_arquivo = raw_input('Diretorio completo do arquivo: ')
        self.conn, self.uid = self.new_connection()

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(3)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def create_pessoa(self, pessoa_dados=[]):
        """
        O metodo cria uma nova pessoa.
        :return: id da tabela adm_pessoa
        :rtype: int
        """
        pessoa_id = False
        vals = dict()
        vals_endereco = dict()
        vals_telefone = dict()
        contador = 1
        endereco_tipo_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.endereco.tipo', 'search', [], 0, 1,
            'id')[0]
        telefone_tipo_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.telefone.tipo', 'search', [], 0, 1,
            'id')[0]
        for dado in pessoa_dados:
            if contador == 1:
                print dado[1]
                # Codigo [0] / Nome [1] / Nome fantasia [2] / Inscricao CGC [6]
                cpf_cnpj = filter(lambda cc: cc.isdigit(), dado[6])
                name = remove_acentuacao(dado[1], 'ASCII')
                if len(name.split(' ')) == 1:
                    name += ' ' + dado[0]
                if self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa', 'search',
                                     [('cpf_cnpj', '=', cpf_cnpj), ('name', '=', name)], 0, 1):
                    break
                vals.update({
                    'name': name, 'fantasia_apelido': remove_acentuacao(dado[2], 'ASCII'),
                    'cpf_cnpj': cpf_cnpj, 'pessoa_tipo': 'F' if len(cpf_cnpj) == 11 else 'J'})
            elif contador == 2:
                # Endereco [0] / IE [3]
                endereco = dado[0].split(' ')
                numero = filter(lambda n: n.isdigit(), endereco) or ''
                if numero:
                    numero = numero[0]
                logradouro_id = False
                descricao = ' '.join(endereco[1:])
                logradouro_ids = self.conn.execute(
                    self.database, self.uid, self.password, 'adm.logradouro', 'search',
                    [('titulo', '=', endereco[0]),
                     ('descricao', '=', remove_acentuacao(descricao, 'ASCII'))])
                if logradouro_ids:
                    logradouro_id = logradouro_ids[0]
                descricao = remove_acentuacao(' '.join(endereco[1:]), 'ASCII')
                logradouro_id = logradouro_id or self.conn.execute(
                    self.database, self.uid, self.password, 'adm.logradouro', 'create',
                    {'titulo': endereco[0], 'descricao': descricao})
                if endereco[0] and descricao:
                    vals_endereco.update({
                        'logradouro_id': logradouro_id, 'numero': numero
                    })
                vals.update({'admpes_jur_ie': dado[3].replace('\r\n', '')})
            elif contador == 3:
                # Bairro [1] / Fax [2]
                bairro = remove_acentuacao(dado[1], 'ASCII')
                if bairro:
                    vals_endereco.update({
                        'bairro_id': self.conn.execute(
                            self.database, self.uid, self.password, 'adm.bairro', 'create',
                            {'name': bairro})
                    })
            else:
                # CEP [1] / Telefone [4] / Cidade-Estado [3]
                cidade_estado = dado[3].split(' ')
                uf_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.uf',
                                           'search', [('sigla', '=', cidade_estado[-1].upper())],
                                           0, 1)
                cidade = remove_acentuacao(' '.join(cidade_estado[:-1]), 'ASCII')
                cidade_ids = self.conn.execute(
                    self.database, self.uid, self.password, 'adm.cidade', 'search',
                    [('uf_id', '=', uf_ids[0] if uf_ids else False),
                     ('name', 'ilike', cidade)], 0, 1)
                cidade_id = False
                if uf_ids and not cidade_ids:
                    cidade_id = self.conn.execute(
                        self.database, self.uid, self.password, 'adm.cidade', 'create',
                        {'uf_id': uf_ids[0], 'name': cidade})
                elif uf_ids and cidade_ids:
                    cidade_id = cidade_ids[0]
                if cidade_id:
                    pais_id = self.conn.execute(
                        self.database, self.uid, self.password, 'adm.uf', 'read',
                        uf_ids[0], ['pais_id']).get('pais_id')[0]
                    vals_endereco.update({'cidade_id': cidade_id, 'uf_id': uf_ids[0],
                                          'pais_id': pais_id})
                if dado[1] and endereco_tipo_id:
                    vals_endereco.update({
                        'cep': dado[1],
                        'endereco_tipo_id': endereco_tipo_id
                    })
                if dado[4] and telefone_tipo_id:
                    vals_telefone.update({'telefone_tipo_id': telefone_tipo_id, 'numero': dado[4]})
            contador += 1
        if vals:
            pessoa_id = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                          'create', vals)
            if pessoa_id and vals_telefone:
                vals_telefone.update({'pessoa_id': pessoa_id})
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                  'create', vals_telefone)
            if pessoa_id and vals_endereco:
                vals_endereco.update({'pessoa_id': pessoa_id})
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.endereco',
                                  'create', vals_endereco)
        return pessoa_id

    def create_financeiro_fornecedor(self, pessoa_id):
        """
        O metodo cria um novo fornecedor.
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :return: id da tabela financeiro_fornecedor
        :rtype: int
        """
        cliente_id = self.conn.execute(self.database, self.uid, self.password,
                                       'financeiro.fornecedor', 'create', {'pessoa_id': pessoa_id})
        return True


if __name__ == "__main__":
    i_id = ImportaDados()
    arquivo_dados = open(i_id.dir_arquivo, 'r').readlines()
    pessoa_dados = list()
    dados_cabecalho = ['MAX - Empresarial', 'RELAO DE FORNECEDORES EM ORDEM ALFABTICA',
                       '\r\n', '"', '""\r\n', 'Cdigo', 'Endereo', 'Bairro', 'CEP/Cidade/Estado']
    char_validos = string.letters + string.digits + ' ' + '-' + '/'
    for dado in arquivo_dados[8:]:
        list_dado = dado.split(',')
        if len(list_dado) == 1:
            list_dado.append('')
        descricao = ''.join([char if char in char_validos else '' for char in (
                list_dado[0].replace('\r\n', '').replace('"', '') or
                list_dado[1].replace('\r\n', '').replace('"', ''))])
        if descricao in dados_cabecalho:
            continue
        if dado in dados_cabecalho:
            continue
        if dado == '"' + '-' * 132 + '"\r\n':
            pessoa_id = i_id.create_pessoa(pessoa_dados)
            if pessoa_id:
                i_id.create_financeiro_fornecedor(pessoa_id)
            pessoa_dados = list()
        else:
            dado = dado.replace('"', '')
            pessoa_dados.append(dado.split(','))
    print 'Fornecedores importados com sucesso!'
