# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import string


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    from unicodedata import normalize, category

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


def remove_caracter(caracteres, letras=True, numeros=True, espaco=True, outros=None,
                    substituto=''):
    """
    @Parametros: caracteres - Eh a palavra ou frase que deseja remover caracters
                              especiais - Tipo: string
                 letras     - Se True filtra letras da sequencia informada - Tipo boolean
                 numeros    - Se True filtra os digitos da sequencia de caracteres
                              informados - Tipo: boolean
                 espaco     - Se True filtra os espacos em branco - Tipo: boolean
                 outros     - uma lista de outros caracteres que deseja filtrar - Tipo: lista
                 substituto - caracter que substituira o caracter especial removido
    @Funcao: Remover caracters especiais de uma sequencia de caracteres
    @Retorno: Retorna uma string tratada de acordo com as informacoes passadas no
              parametro - Tipo: string
    """
    if not caracteres:
        return ''
    if not isinstance(outros, list):
        outros = []
    char_validos = ''
    if letras:
        char_validos += string.letters
    if numeros:
        char_validos += string.digits
    if espaco:
        char_validos += ' '
    for i in outros:
        char_validos += i
    new_caracters = ''.join([char if char in char_validos else substituto for char in
                             caracteres])
    return new_caracters


class ImportaClienteDados():
    def __init__(self):
        self.database = 'teste'
        self.host = 'localhost'
        self.user = 'admin'
        self.password = 'admin'
        self.port = '8069'
        self.dir_arquivo = '/tmp/CLIENTES.TXT'
        self.arquivo_problema_cep = open('/tmp/cliente_problema_cep.txt', 'a')
        self.arquivo_problema_cpf_cnpj = open('/tmp/cliente_problema_cpf_cnpj.txt', 'a')
        # self.database = str(raw_input('Nome do banco de dados: '))
        # self.host = str(raw_input('Host/IP OpenERP: '))
        # self.user = str(raw_input('Usuario OpenERP: '))
        # self.password = str(raw_input('Senha OpenERP: '))
        # self.port = raw_input('Porta do OpenERP: ')
        # self.dir_arquivo = raw_input('Diretorio completo do arquivo: ')
        self.conn, self.uid = self.new_connection()
        self.endereco_tipo_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.endereco.tipo', 'search',
            [('name', '=', 'Normal')], 0, 1, 'id')[0]
        self.endereco_tipo_cobranca_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.endereco.tipo', 'search',
            [('name', '=', 'Cobranca')], 0, 1, 'id')[0]
        self.telefone_tipo_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.telefone.tipo', 'search', [], 0, 1,
            'id')[0]
        self.telefone_tipo_fax_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.telefone.tipo', 'search',
            [('name', '=', 'Fax')], 0, 1)[0]

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(3)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def create_pessoa(self, pessoa_dados=[]):
        """
        O metodo cria uma nova pessoa.
        :return: id da tabela adm_pessoa
        :rtype: int
        """

        # A ordem abaixo se refere a posicao na lista python:
        # CPF_CNPJ [0] / Codigo [1] / Fantasia [2] / R.Social [3] / Endereco [4] / Bairro [5] /
        # Cod. IBGE [7] / Cidade [8] / CEP [9] / Estado [10] / Telefone [11] [12] [13] / Fax [14] /
        # Tipo Pessoa [16] / IE [17] / Data nascimento [21] / End.cob [25] / Bairro cob. [26] /
        # Cod. cob. IBGE [27] / Cidade cob. [28] / CEP cob. [29] / Estado cob. [30] /
        # Limite cred. [33] / RG [61] / Nome Pai [66] / Nome Mae [67] / Estado civil [75] /
        # Sexo [76]
        pessoa_id = False
        vals_endereco = dict()
        vals_endereco_cobranca = dict()
        vals_telefone1 = dict()
        vals_telefone2 = dict()
        vals_telefone3 = dict()
        vals_telefone4 = dict()

        print pessoa_dados[3]
        cpf_cnpj = filter(lambda cc: cc.isdigit(), pessoa_dados[0])
        name = remove_acentuacao(pessoa_dados[3], 'ASCII')
        name = remove_caracter(name, numeros=False, outros=['/'])
        limite_credito = float(pessoa_dados[33] or 0)
        if len(name.split(' ')) == 1:
            name += ' ' + pessoa_dados[1]
        if self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa', 'search',
                             [('cpf_cnpj', '=', cpf_cnpj or False), ('name', '=', name)], 0, 1):
            return pessoa_id, limite_credito
        if cpf_cnpj and self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                          'search', [('cpf_cnpj', '=', cpf_cnpj or False)], 0, 1):
            return pessoa_id, limite_credito
        codigos = ['468', '677']

        # ENDERECO COMUM
        endereco = pessoa_dados[4].split(' ')
        descricao = remove_acentuacao(' '.join(endereco[1:]), 'ASCII')
        # numero = filter(lambda n: n.isdigit(), endereco) or ''
        # if numero:
        #     numero = numero[0]
        #     descricao = remove_acentuacao(' '.join(endereco[1:-1]), 'ASCII')
        # else:
        #     descricao = remove_acentuacao(' '.join(endereco[1:]), 'ASCII')
        list_num = ['No.', 'NUMERO', 'APTO']
        numero = ''
        pop = False
        if pessoa_dados[5].replace(' ', '').replace('/', '').isdigit():
            numero = pessoa_dados[5].replace(' ', '')
            pessoa_dados.pop(5)
            pop = True
        if not numero:
            numero = filter(lambda num: num in list_num, pessoa_dados[5].split(' '))
            if numero and not pop:
                pessoa_dados.pop(5)
                pop = True
        if (len(pessoa_dados) > 96
                and not any([pessoa_dados[6], pessoa_dados[7], pessoa_dados[8]])
                and not pop) or pessoa_dados[1] in codigos:
            pessoa_dados.pop(6)
        logradouro_id = False
        logradouro_ids = self.conn.execute(
            self.database, self.uid, self.password, 'adm.logradouro', 'search',
            [('titulo', '=', endereco[0]),
             ('descricao', '=', descricao)])
        if logradouro_ids:
            logradouro_id = logradouro_ids[0]
        logradouro_id = logradouro_id or self.conn.execute(
            self.database, self.uid, self.password, 'adm.logradouro', 'create',
            {'titulo': endereco[0], 'descricao': descricao})
        if endereco[0] and descricao:
            vals_endereco.update({
                'logradouro_id': logradouro_id,  # 'numero': numero
            })
        bairro = remove_acentuacao(pessoa_dados[5], 'ASCII')
        if bairro:
            bairro_ids = self.conn.execute(
                self.database, self.uid, self.password, 'adm.bairro', 'search',
                [('name', '=', bairro)], 0, 1)
            vals_endereco.update({
                'bairro_id': bairro_ids[0] if bairro_ids else self.conn.execute(
                    self.database, self.uid, self.password, 'adm.bairro', 'create',
                    {'name': bairro})
            })
        uf_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.uf',
                                   'search', [('sigla', '=', pessoa_dados[10].upper())],
                                   0, 1)
        cidade = remove_acentuacao(pessoa_dados[8], 'ASCII')
        args_cidade = [('uf_id', '=', uf_ids[0] if uf_ids else False), ('name', 'ilike', cidade)]
        if pessoa_dados[7] and pessoa_dados[7].isdigit():
            args_cidade.append(('codigo_ibge', '=', int(pessoa_dados[7])))
        cidade_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.cidade',
                                       'search', args_cidade, 0, 1)
        cidade_id = False
        if uf_ids and not cidade_ids:
            vals_cidade = {'uf_id': uf_ids[0], 'name': cidade}
            if pessoa_dados[7]:
                vals_cidade.update({'codigo_ibge': pessoa_dados[7]})
            cidade_id = self.conn.execute(
                self.database, self.uid, self.password, 'adm.cidade', 'create', vals_cidade)
        elif uf_ids and cidade_ids:
            cidade_id = cidade_ids[0]
        if cidade_id:
            pais_id = self.conn.execute(
                self.database, self.uid, self.password, 'adm.uf', 'read',
                uf_ids[0], ['pais_id']).get('pais_id')[0]
            vals_endereco.update({'cidade_id': cidade_id, 'uf_id': uf_ids[0],
                                  'pais_id': pais_id})
        if pessoa_dados[9] and self.endereco_tipo_id:
            vals_endereco.update({
                'cep': pessoa_dados[9],
                'endereco_tipo_id': self.endereco_tipo_id
            })

        # ENDERECO COBRANCA
        endereco = pessoa_dados[25].split(' ')
        descricao = remove_acentuacao(' '.join(endereco[1:]), 'ASCII')
        # numero = filter(lambda n: n.isdigit(), endereco) or ''
        # if numero:
        #     numero = numero[0]
        #     descricao = remove_acentuacao(' '.join(endereco[1:-1]), 'ASCII')
        # else:
        #     descricao = remove_acentuacao(' '.join(endereco[1:]), 'ASCII')
        numero = ''
        pop = False
        if pessoa_dados[26].replace(' ', '').replace('/', '').isdigit():
            numero = pessoa_dados[26].replace(' ', '')
            pessoa_dados.pop(26)
            pop = True
        if not numero:
            numero = filter(lambda num: num in list_num, pessoa_dados[26].split(' '))
            if numero and not pop:
                pessoa_dados.pop(26)
                pop = True
        if (len(pessoa_dados) > 96 and not pessoa_dados[27] and not pop) or \
                pessoa_dados[1] == '468':
            pessoa_dados.pop(27)
        logradouro_id = False
        logradouro_ids = self.conn.execute(
            self.database, self.uid, self.password, 'adm.logradouro', 'search',
            [('titulo', '=', endereco[0]),
             ('descricao', '=', descricao)])
        if logradouro_ids:
            logradouro_id = logradouro_ids[0]
        logradouro_id = logradouro_id or self.conn.execute(
            self.database, self.uid, self.password, 'adm.logradouro', 'create',
            {'titulo': endereco[0], 'descricao': descricao})
        if endereco[0] and descricao:
            vals_endereco_cobranca.update({
                'logradouro_id': logradouro_id,  # 'numero': numero
            })
        bairro = remove_acentuacao(pessoa_dados[26], 'ASCII')
        if bairro:
            bairro_ids = self.conn.execute(
                self.database, self.uid, self.password, 'adm.bairro', 'search',
                [('name', '=', bairro)], 0, 1)
            vals_endereco_cobranca.update({
                'bairro_id': bairro_ids[0] if bairro_ids else self.conn.execute(
                    self.database, self.uid, self.password, 'adm.bairro', 'create',
                    {'name': bairro})
            })
        uf_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.uf',
                                   'search', [('sigla', '=', pessoa_dados[30].upper())],
                                   0, 1)
        cidade = remove_acentuacao(pessoa_dados[28], 'ASCII')
        args_cidade = [('uf_id', '=', uf_ids[0] if uf_ids else False), ('name', 'ilike', cidade)]
        if pessoa_dados[27] and pessoa_dados[27].isdigit():
            args_cidade.append(('codigo_ibge', '=', int(pessoa_dados[27])))
        cidade_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.cidade',
                                       'search', args_cidade, 0, 1)
        cidade_id = False
        if uf_ids and not cidade_ids:
            vals_cidade = {'uf_id': uf_ids[0], 'name': cidade}
            if pessoa_dados[27]:
                vals_cidade.update({'codigo_ibge': pessoa_dados[27]})
            cidade_id = self.conn.execute(
                self.database, self.uid, self.password, 'adm.cidade', 'create', vals_cidade)
        elif uf_ids and cidade_ids:
            cidade_id = cidade_ids[0]
        if cidade_id:
            pais_id = self.conn.execute(
                self.database, self.uid, self.password, 'adm.uf', 'read',
                uf_ids[0], ['pais_id']).get('pais_id')[0]
            vals_endereco_cobranca.update({'cidade_id': cidade_id, 'uf_id': uf_ids[0],
                                           'pais_id': pais_id})
        if pessoa_dados[29] and self.endereco_tipo_cobranca_id:
            vals_endereco_cobranca.update({
                'cep': pessoa_dados[29],
                'endereco_tipo_id': self.endereco_tipo_cobranca_id
            })

        # TELEFONES
        if pessoa_dados[11] and self.telefone_tipo_id:
            vals_telefone1.update({'telefone_tipo_id': self.telefone_tipo_id,
                                   'numero': pessoa_dados[11]})
        if pessoa_dados[12] and self.telefone_tipo_id:
            vals_telefone2.update({'telefone_tipo_id': self.telefone_tipo_id,
                                   'numero': pessoa_dados[12]})
        if pessoa_dados[13] and self.telefone_tipo_id:
            vals_telefone3.update({'telefone_tipo_id': self.telefone_tipo_id,
                                   'numero': pessoa_dados[13]})
        if pessoa_dados[14] and self.telefone_tipo_fax_id:
            vals_telefone4.update({'telefone_tipo_id': self.telefone_tipo_fax_id,
                                   'numero': pessoa_dados[14]})

        vals = {
            'name': name, 'fantasia_apelido': remove_acentuacao(pessoa_dados[2], 'ASCII'),
            'cpf_cnpj': cpf_cnpj or False, 'pessoa_tipo': pessoa_dados[16],
            'admpes_jur_ie': pessoa_dados[17].replace('\r\n', ''),
            'admpes_fis_data_nascimento': pessoa_dados[21] or False,
            'admpes_fis_rg': pessoa_dados[61], 'admpes_fis_estado_civil': pessoa_dados[76],
            'admpes_fis_sexo': pessoa_dados[77]
        }

        try:
            pessoa_id = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                          'create', vals)
        except Exception as e:
            if 'CPF/CNPJ Invalido' in str(e):
                self.arquivo_problema_cpf_cnpj.write(
                    str(pessoa_dados[1]) + ' - ' + str(cpf_cnpj) + ' - ' + name + '\r\n')
                return pessoa_id, limite_credito
            else:
                raise e

        if pessoa_id and vals_telefone1:
            vals_telefone1.update({'pessoa_id': pessoa_id})
            self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                              'create', vals_telefone1)
        if pessoa_id and vals_telefone2:
            vals_telefone2.update({'pessoa_id': pessoa_id})
            self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                              'create', vals_telefone2)
        if pessoa_id and vals_telefone3:
            vals_telefone3.update({'pessoa_id': pessoa_id})
            self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                              'create', vals_telefone3)
        if pessoa_id and vals_telefone4:
            vals_telefone4.update({'pessoa_id': pessoa_id})
            self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                              'create', vals_telefone4)
        codigo = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                   'read', pessoa_id, ['codigo']).get('codigo')
        if pessoa_id and vals_endereco and vals_endereco.get('pais_id'):
            vals_endereco.update({'pessoa_id': pessoa_id})
            try:
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.endereco',
                                  'create', vals_endereco)
            except Exception as e:
                if 'CEP invalido' in str(e):
                    self.arquivo_problema_cep.write(str(codigo) + ' - ' + name + '\r\n')
                else:
                    raise e
        if pessoa_id and vals_endereco_cobranca and vals_endereco_cobranca.get('pais_id'):
            vals_endereco_cobranca.update({'pessoa_id': pessoa_id})
            try:
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.endereco',
                                  'create', vals_endereco_cobranca)
            except Exception as e:
                if 'CEP invalido' in str(e):
                    self.arquivo_problema_cep.write(str(codigo) + ' - ' + name + '  (Cobranca)\r\n')
                else:
                    raise e
        return pessoa_id, limite_credito

    def create_financeiro_cliente(self, pessoa_id, limite_credito):
        """
        O metodo cria um novo cliente.
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :return: id da tabela financeiro_cliente
        :rtype: int
        """
        cliente_id = self.conn.execute(self.database, self.uid, self.password,
                                       'financeiro.cliente', 'create',
                                       {'pessoa_id': pessoa_id, 'limite_credito': limite_credito})
        return True


if __name__ == "__main__":
    i_cli = ImportaClienteDados()
    arquivo = open(i_cli.dir_arquivo, 'r')
    arquivo_dados = arquivo.readlines()
    for dado in arquivo_dados:
        dado = dado.replace('"', '')
        pessoa_id, limite_credito = i_cli.create_pessoa(dado.split(','))
        if not pessoa_id:
            continue
        i_cli.create_financeiro_cliente(pessoa_id, limite_credito)
    arquivo.close()
    i_cli.arquivo_problema_cep.close()
    i_cli.arquivo_problema_cpf_cnpj.close()
    print 'Clientes importados com sucesso!'
