# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import xlrd
import csv
from unicodedata import normalize, category
from datetime import datetime


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


arquivo_csv = '/tmp/equilibrio_cadastro.csv'


class ImportaDadosProduto():
    def __init__(self):
        # self.database = 'teste'
        # self.host = 'localhost'
        # self.user = 'admin'
        # self.password = 'admin'
        # self.port = '8069'
        # self.dir_arquivo = '/tmp/equilibrio cadastro.xlsx'
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.dir_arquivo = raw_input('Diretorio completo do arquivo: ')
        self.conn, self.uid = self.new_connection()

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(5)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def converts_xlsx_to_csv(self):
        res = False
        wb = xlrd.open_workbook(self.dir_arquivo)
        sh = wb.sheet_by_index(0)
        open_file = open(arquivo_csv, 'wb')
        wr = csv.writer(open_file, quoting=csv.QUOTE_ALL, delimiter='?')
        for rownum in range(4, sh.nrows):
            dados = sh.row_values(rownum)
            try:
                wr.writerow(dados)
            except:
                for dado in dados:
                    if isinstance(dado, (int, float)) or not dado:
                        continue
                    dados[dados.index(dado)] = remove_acentuacao(dado)
                wr.writerow(dados)
                res = True
        open_file.close()
        return res

    def create_estoque_produto(self, produto_dados):
        """
        O metodo cria um novo produto.
        :param produto_dados: lista com os dados para cadastro do produto.
        :type produto_dados: list
        :return: id da tabela estoque_produto
        :rtype: int
        """
        # Name [1] / Codigo barras [8] / Preco custo [11] / Preco venda [13] / Unidade [15]
        name = produto_dados[1]
        codigo_barras = produto_dados[7][:-2]
        preco_custo = float(produto_dados[9] or 0.0)
        preco_venda = float(produto_dados[10] or 0.0)
        unidade = produto_dados[11]
        if self.conn.execute(self.database, self.uid, self.password, 'estoque.produto', 'search',
                             [('name', '=', name)], 0, 1):
            return False
        ncm_ids = self.conn.execute(self.database, self.uid, self.password, 'fiscal.ncm',
                                    'search', [('codigo', '=', '35022000')])
        vals_produto = {
            'name': name, 'tipo_item': '00', 'ncm_id': ncm_ids[0] if ncm_ids else False
        }
        produto_id = self.conn.execute(self.database, self.uid, self.password,
                                       'estoque.produto', 'create', vals_produto)
        grade_id = False
        if produto_id:
            grade_ids = self.conn.execute(
                self.database, self.uid, self.password, 'estoque.produto.grade', 'search',
                [('produto_id', '=', produto_id)])
            if grade_ids:
                grade_id = grade_ids[0]
            self.create_fiscal_tributacao_encargos(produto_id)
        if produto_id and preco_custo:
            self.create_estoque_preco_custo(produto_id, preco_custo)
        if produto_id and preco_venda:
            self.create_estoque_preco_venda(produto_id, preco_venda)
        produto_embalagem_id = False
        if produto_id and unidade:
            produto_embalagem_id = self.create_estoque_produto_embalagem(produto_id, unidade)
        if produto_id and codigo_barras and produto_embalagem_id and grade_id:
            self.create_automacao_codigo_barra(produto_id, grade_id, produto_embalagem_id,
                                               codigo_barras)
        local_ids = self.conn.execute(self.database, self.uid, self.password, 'estoque.local',
                                      'search', [('name', '=', 'Loja')])
        if produto_id and local_ids and produto_embalagem_id and grade_id:
            self.create_estoque_local_locacao(produto_id, grade_id, produto_embalagem_id,
                                              local_ids[0])
        return produto_id

    def create_estoque_produto_embalagem(self, produto_id, unidade):
        """
        O metodo cria uma embalagem para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param unidade: sigla de representacao da embalagem.
        :type unidade: str
        :return: id da tabela estoque_produto_embalagem
        :rtype: int
        """
        embalagem_ids = self.conn.execute(self.database, self.uid, self.password,
                                          'estoque.embalagem', 'search', [('sigla', '=', 'UND')])
        vals = {'produto_id': produto_id, 'embalagem_id': embalagem_ids[0]}
        produto_embalagem_id = self.conn.execute(self.database, self.uid, self.password,
                                                 'estoque.produto.embalagem', 'create', vals)
        return produto_embalagem_id

    def create_automacao_codigo_barra(self, produto_id, grade_id, produto_embalagem_id,
                                      codigo_barra):
        """
        O metodo cria um registro de codigo de barras.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem.
        :type produto_embalagem_id: int
        :param codigo_barra: numeros que compoem o codigo de barras.
        :type codigo_barra: str
        :return: id da tabela automacao_codigo_barra
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'grade_id': grade_id,
                'produto_embalagem_id': produto_embalagem_id, 'codigo_barra': codigo_barra}
        codigo_barra_id = self.conn.execute(self.database, self.uid, self.password,
                                            'automacao.codigo.barra', 'create', vals)
        return codigo_barra_id

    def create_estoque_local_locacao(self, produto_id, grade_id, produto_embalagem_id,
                                     local_id):
        """
        O metodo cria um registro de codigo de barras.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem.
        :type produto_embalagem_id: int
        :param local_id: id da tabela estoque_local.
        :type local_id: int
        :return: id da tabela estoque_local_locacao
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'grade_id': grade_id,
                'estllc_produto_embalagem_id': produto_embalagem_id, 'local_id': local_id}
        locacao_id = self.conn.execute(self.database, self.uid, self.password,
                                       'estoque.local.locacao', 'create', vals)
        return locacao_id

    def create_estoque_preco_custo(self, produto_id, preco_custo):
        """
        O metodo cria um registro de preco de custo para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param preco_custo: valor do custo.
        :type preco_custo: float
        :return: id da tabela estoque_preco_custo
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'estprc_custo_reposicao': preco_custo,
                'estprc_custo_corrigido': preco_custo, 'estprc_custo_medio_contabil': preco_custo,
                'estprc_ultimo_custo': preco_custo, 'estprc_data': str(datetime.now())}
        custo_id = self.conn.execute(self.database, self.uid, self.password, 'estoque.preco.custo',
                                     'create', vals)
        return custo_id

    def create_estoque_preco_venda(self, produto_id, preco_venda):
        """
        O metodo cria um registro de preco de venda para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param preco_venda: valor para venda.
        :type preco_venda: float
        :return: id da tabela estoque_preco_venda
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'preco_venda': preco_venda}
        preco_venda_id = self.conn.execute(self.database, self.uid, self.password,
                                           'estoque.preco.venda', 'create', vals,
                                           {'cadastro_produto': True})
        return preco_venda_id

    def create_fiscal_tributacao_encargos(self, produto_id):
        """
        O metodo cria os registros da tributacao para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :return: Verdadeiro
        :rtype: bool
        """
        icms_encargo_id = self.conn.execute(self.database, self.uid, self.password,
                                            'fiscal.encargo', 'search', [('sigla', '=', 'ICMS')])[0]
        pis_encargo_id = self.conn.execute(self.database, self.uid, self.password,
                                           'fiscal.encargo', 'search', [('sigla', '=', 'PIS')])[0]
        cofins_encargo_id = self.conn.execute(self.database, self.uid, self.password,
                                              'fiscal.encargo', 'search',
                                              [('sigla', '=', 'COFINS')])[0]
        cst_50_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', '50'), ('tipo', '=', 'T'), ('fiscst_tipo_movimento', '=', 'E')])
        cst_01_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', '01'), ('tipo', '=', 'T'), ('fiscst_tipo_movimento', '=', 'S')])
        cst_60_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', '60'), ('tipo', '=', 'U'), ('fiscst_tipo_movimento', '=', 'A')])

        # ICMS
        icms_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', icms_encargo_id), ('cst_id', 'in', cst_60_ids)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': icms_encargo_id, 'cst_id': cst_60_ids[0],
                'tipo_tributacao': 'T', 'tributacao_id': icms_tributacao_id}
        self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                          'create', vals)

        # PIS ENTRADA
        pis_entrada_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', pis_encargo_id), ('cst_id', 'in', cst_50_ids)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': pis_encargo_id, 'cst_id': cst_50_ids[0],
                'tipo_tributacao': 'E', 'tributacao_id': pis_entrada_tributacao_id}
        self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                          'create', vals)

        # PIS SAIDA
        pis_saida_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', pis_encargo_id), ('cst_id', 'in', cst_01_ids)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': pis_encargo_id, 'cst_id': cst_01_ids[0],
                'tipo_tributacao': 'T', 'tributacao_id': pis_saida_tributacao_id}
        self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                          'create', vals)

        # COFINS ENTRADA
        cofins_entrada_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', cofins_encargo_id), ('cst_id', 'in', cst_50_ids)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': cofins_encargo_id, 'cst_id': cst_50_ids[0],
                'tipo_tributacao': 'E', 'tributacao_id': cofins_entrada_tributacao_id}
        self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                          'create', vals)

        # COFINS SAIDA
        cofins_saida_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', cofins_encargo_id), ('cst_id', 'in', cst_01_ids)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': cofins_encargo_id, 'cst_id': cst_01_ids[0],
                'tipo_tributacao': 'T', 'tributacao_id': cofins_saida_tributacao_id}
        self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                          'create', vals)
        return True


if __name__ == "__main__":
    i_cli = ImportaDadosProduto()
    if not i_cli.converts_xlsx_to_csv():
        raise Exception('Nao foi possivel converter os dados da planilha!')
    arquivo = open(arquivo_csv, 'r')
    arquivo_dados = arquivo.readlines()
    for dado in arquivo_dados:
        if not dado:
            continue
        dado = dado.replace('"', '').replace('\r\n', '')
        dado = dado.split('?')
        print dado[1]
        i_cli.create_estoque_produto(dado)
    arquivo.close()
    print ''
    print 'PRODUTOS IMPORTADOS COM SUCESSO!'
