# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import xlrd
import csv
from unicodedata import normalize, category
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
import pytz


def converte_tz(data, tz_origem, tz_destino):
    """
    Função para converter data de um timezone para outro
    @param data: data no formato datetime para ser convertida
    @param tz_origem: Timezone em que a data se encontra. Se None, vai buscar timezone do usuário
    @param tz_destino: Timezne em que a data será convertida. Se None, vai buscar timezone do usuário
    @return: data no formato datetime convertida para o novo timezone
    """
    obj_origem = pytz.timezone(tz_origem)
    obj_destino = pytz.timezone(tz_destino)
    x = obj_origem.localize(data, is_dst=False)
    return x.astimezone(obj_destino)


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


arquivo_csv = '/tmp/planilha_tesla.csv'
date = parse('1989-03-29 12:00:00')
date_tz = converte_tz(date, 'America/Sao_Paulo', 'UTC')
diff_hour = str(parse(date_tz.strftime('%Y-%m-%d %H:%M:%S')) - date)
if len(diff_hour) == 7:
    diff_hour = '0' + diff_hour
base_date = parse('1899-12-30T%s' % diff_hour)


class ImportaDadosOS():
    def __init__(self):
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.local = raw_input('Numero do local para cadastro de servicos: ')
        self.dir_arquivo = raw_input('Diretorio completo do arquivo: ')
        self.conn, self.uid = self.new_connection()
        self.local_id = self.check_estoque_local()
        self.embalagem_id = self.check_estoque_embalagem()

    def check_estoque_local(self):
        msg = 'ERRO: Local nao cadastrado.'
        try:
            local_id = self.conn.execute(self.database, self.uid, self.password, 'estoque.local',
                                         'search', [('numero', '=', int(self.local))], 0, 1)
        except:
            print msg
            exec exit()
        if not local_id:
            print msg
            exec exit()
        return local_id[0]

    def check_estoque_embalagem(self):
        embalagem_id = self.conn.execute(self.database, self.uid, self.password, 'adm.utils',
                                         'admutl_get_parametros', 'estoque',
                                         ['estpar_embalagem_servico_id']) or {}
        if not embalagem_id:
             print 'ERRO: Embalagem para servicos nao cadastrada nos parametros do estoque.'
             exec exit()
        return embalagem_id.get('estpar_embalagem_servico_id')

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(5)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def converts_xlsx_to_csv(self):
        res = False
        wb = xlrd.open_workbook(self.dir_arquivo)
        sh = wb.sheet_by_name('Planilha')
        open_file = open(arquivo_csv, 'wb')
        wr = csv.writer(open_file, quoting=csv.QUOTE_ALL, delimiter='?')
        for rownum in range(2, sh.nrows):
            dados = sh.row_values(rownum)
            try:
                wr.writerow(dados)
            except:
                for dado in dados:
                    if isinstance(dado, (int, float)) or not dado:
                        continue
                    dados[dados.index(dado)] = remove_acentuacao(dado)
                wr.writerow(dados)
                res = True
        open_file.close()
        return res

    def check_estoque_produto(self, name, dados):
        """
        O metodo verifica a existencia do servico. Se nao houver registro, cria um novo.
        :param name: descricao do servico.
        :type name: str
        :param dados: dados a serem importados da planilha.
        :type dados: list
        :return: id da tabela estoque_produto
        :rtype: int
        """
        produto_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.produto', 'search',
            [('name', '=', name[:60]), ('tipo', '=', 'S'), ('tipo_item', '=', '09')], 0, 1)
        if produto_ids:
            return produto_ids[0]
        categoria_id = oservico.check_estoque_categoria(dados[0])
        vals_produto = {'name': name[:60], 'tipo_item': '09', 'tipo': 'S',
                        'categoria_id': categoria_id}
        produto_id = self.conn.execute(self.database, self.uid, self.password,
                                       'estoque.produto', 'create', vals_produto,
                                       {'estoque_servico': True})
        grade_id = False
        if produto_id:
            grade_ids = self.conn.execute(
                self.database, self.uid, self.password, 'estoque.produto.grade', 'search',
                [('produto_id', '=', produto_id)])
            if grade_ids:
                grade_id = grade_ids[0]
        produto_embalagem_id = False
        if produto_id:
            produto_embalagem_id = self.create_estoque_produto_embalagem(produto_id)
        if produto_id and self.local_id and produto_embalagem_id and grade_id:
            self.create_estoque_local_locacao(produto_id, grade_id, produto_embalagem_id,
                                              self.local_id)
        return produto_id

    def create_estoque_produto_embalagem(self, produto_id):
        """
        O metodo cria uma embalagem para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :return: id da tabela estoque_produto_embalagem
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'embalagem_id': self.embalagem_id}
        produto_embalagem_id = self.conn.execute(self.database, self.uid, self.password,
                                                 'estoque.produto.embalagem', 'create', vals, {})
        return produto_embalagem_id

    def create_estoque_local_locacao(self, produto_id, grade_id, produto_embalagem_id,
                                     local_id):
        """
        O metodo cria um registro de codigo de barras.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem.
        :type produto_embalagem_id: int
        :param local_id: id da tabela estoque_local.
        :type local_id: int
        :return: id da tabela estoque_local_locacao
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'grade_id': grade_id,
                'estllc_produto_embalagem_id': produto_embalagem_id, 'local_id': local_id}
        locacao_id = self.conn.execute(self.database, self.uid, self.password,
                                       'estoque.local.locacao', 'create', vals, {})
        return locacao_id

    def check_adm_pessoa(self, pessoa):
        """
        O metodo verifica a existencia de registro em adm_pessoa com a descricao em 'pessoa'. Caso
        nao exista, cria um novo.
        :param pessoa: descricao do nome da pessoa.
        :type pessoa: str
        :return: registro da tabela adm_pessoa.
        :rtype: int
        """
        pessoa_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                       'search', [('nome', '=', pessoa)], 0, 1)
        if pessoa_ids:
            return pessoa_ids[0]
        pessoa_id = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                      'create', {'name': pessoa}, {})
        return pessoa_id

    def check_oservico_setor(self, setor):
        """
        O metodo verifica a existencia de registro em oservico_setor com a descricao em 'setor'.
        Caso nao exista, cria um novo.
        :param setor: descricao do nome do setor.
        :type setor: str
        :return: registro da tabela oservico_setor.
        :rtype: int
        """
        setor_ids = self.conn.execute(self.database, self.uid, self.password, 'oservico.setor',
                                      'search', [('name', '=', setor)], 0, 1)
        if setor_ids:
            return setor_ids[0]
        setor_id = self.conn.execute(self.database, self.uid, self.password, 'oservico.setor',
                                     'create', {'name': setor}, {})
        return setor_id

    def check_oservico_categoria(self, categoria):
        """
        O metodo verifica a existencia de registro em oservico_categoria com a descricao em
        'categoria'. Caso nao exista, cria um novo.
        :param categoria: descricao do nome da categoria.
        :type categoria: str
        :return: registro da tabela oservico_categoria.
        :rtype: int
        """
        categoria_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.categoria', 'search',
            [('name', 'ilike', categoria)], 0, 1)
        if categoria_ids:
            return categoria_ids[0]
        categoria_id = self.conn.execute(self.database, self.uid, self.password,
                                         'oservico.categoria', 'create', {'name': categoria}, {})
        return categoria_id

    def check_oservico_centro_custo(self, crsp, setor_id):
        """
        O metodo verifica a existencia de registro em oservico_centro_custo com a descricao em
        'crsp'. Caso nao exista, cria um novo.
        :param crsp: descricao do nome do centro de custos.
        :type crsp: str
        :param setor_id: registro da tabela oservico_setor.
        :type setor_id: int
        :return: registro da tabela oservico_centro_custo.
        :rtype: int
        """
        centro_custo_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.centro.custo', 'search',
            [('name', '=', crsp), ('osvcco_setor_id', '=', setor_id)], 0, 1)
        if centro_custo_ids:
            return centro_custo_ids[0]
        centro_custo_id = self.conn.execute(self.database, self.uid, self.password,
                                            'oservico.centro.custo', 'create',
                                            {'name': crsp, 'osvcco_setor_id': setor_id}, {})
        return centro_custo_id

    def create_oservico_os(self, dados):
        """
        O metodo cria um registro de OS, tabela oservico_os.
        :param dados: dados a serem importados da planilha.
        :type dados: list
        :return: registro da tabela oservico_os.
        :rtype: int
        """
        oservico_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.os', 'search',
            [('osvos_observacoes', 'ilike', '%' + dados[1] + '%')], 0, 1)
        if oservico_ids:
            return oservico_ids[0]
        osvos_data_abertura = (base_date + relativedelta(days=int(float(dados[9])))).strftime(
            '%Y-%m-%d %H:%M:%S')
        solicitante = dados[5]
        if len(solicitante.split()) == 1:
            solicitante += ' IMPORTADO'
        osvos_defeito_relatado = dados[4]
        state = dados[8].upper()
        if state == 'FINALIZADO':
            str_state = 'F'
        elif state in ('EM EXECUÇÃO', 'ABERTO'):
            str_state = 'A'
        elif state == 'CANCELADO':
            str_state = 'C'
        else:
            str_state = 'E'
        osvos_data_previsao = ''
        if dados[13]:
            try:
                osvos_data_previsao = (base_date + relativedelta(days=int(float(dados[13])))).\
                    strftime('%Y-%m-%d')
            except:
                pass
        osvos_data_entrega = ''
        if dados[16]:
            osvos_data_entrega = (base_date + relativedelta(days=int(float(dados[16])))).strftime(
                '%Y-%m-%d %H:%M:%S')
        osvos_observacoes = dados[20] + '\n' + dados[1]
        setor = dados[6]
        osvos_setor_id = self.check_oservico_setor(setor)
        motivacao = dados[2]
        osvos_categoria_id = self.check_oservico_categoria(motivacao)
        crsp = dados[3]
        osvos_centro_custo_id = self.check_oservico_centro_custo(crsp, osvos_setor_id)
        vals_os = {
            'osvos_data_abertura': osvos_data_abertura,
            'osvos_pessoa_id': self.check_adm_pessoa(solicitante),
            'osvos_execucao': 'E',
            'osvos_defeito_relatado': osvos_defeito_relatado,
            'state': str_state,
            'osvos_observacoes': osvos_observacoes,
            'osvos_setor_id': osvos_setor_id,
            'osvos_categoria_id': osvos_categoria_id,
            'osvos_centro_custo_id': osvos_centro_custo_id
        }
        if osvos_data_previsao:
            vals_os.update({'osvos_data_previsao': osvos_data_previsao})
        if osvos_data_entrega:
            vals_os.update({'osvos_data_entrega': osvos_data_entrega[:10],
                            'osvos_data_fechamento': osvos_data_entrega})
        return self.conn.execute(self.database, self.uid, self.password, 'oservico.os', 'create',
                                 vals_os, {})

    def create_oservico_servicos(self, dados, oservico_id):
        """
        O metodo cria um registro na tabela oservico_servicos para a ordem de servico (oservico_id)
        recebida.
        :param dados: dados a serem importados da planilha.
        :type dados: list
        :param oservico_id: id da tabela oservico_os.
        :type oservico_id: int
        :return: registro da tabela oservico_servicos.
        :rtype: int
        """
        servico = dados[4]
        osvsrv_servico_id = self.check_estoque_produto(servico, dados)
        servico_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.servicos', 'search',
            [('osvsrv_os_id', '=', oservico_id), ('osvsrv_servico_id', '=', osvsrv_servico_id)], 0,
            1)
        if servico_ids:
            return servico_ids[0]
        osvsrv_duracao_prevista = dados[14] or 0
        state = dados[8]
        osvsrv_data_inicio = False
        if dados[15]:
            osvsrv_data_inicio = (base_date + relativedelta(days=int(float(dados[15])))).strftime(
                '%Y-%m-%d %H:%M:%S')
        vals_servico = {
            'osvsrv_os_id': oservico_id,
            'osvsrv_servico_id': osvsrv_servico_id,
            'osvsrv_duracao_prevista': float(osvsrv_duracao_prevista),
            'osvsrv_quantidade': 1.0,
            'osvsrv_concluido': True if state == 'FINALIZADO' else False,
        }
        if osvsrv_data_inicio:
            vals_servico.update({'osvsrv_data_inicio': osvsrv_data_inicio})
        return self.conn.execute(self.database, self.uid, self.password, 'oservico.servicos',
                                 'create', vals_servico, {'importacao_planilha': True})

    def check_oservico_mecanico(self, mecanico):
        """
        O metodo verifica a existencia de registro do mecanico. Caso nao seja encontrado, e criado.
        :param mecanico: descricao do mecanico.
        :type mecanico: str
        :return: id da tabela oservico_mecanico.
        :rtype: int
        """
        mecanico = ' '.join(mecanico[:2])
        pessoa_id = self.check_adm_pessoa(mecanico)
        funcionario_ids = self.conn.execute(self.database, self.uid, self.password,
                                            'rh.funcionario', 'search', [('pessoa', '=', pessoa_id)],
                                            0, 1)
        if funcionario_ids:
            funcionario_id = funcionario_ids[0]
        else:
            funcionario_id = self.conn.execute(self.database, self.uid, self.password,
                                               'rh.funcionario', 'create', {'pessoa': pessoa_id},
                                               {})
        mecanico_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.mecanico', 'search',
            [('osvmec_funcionario_id', '=', funcionario_id)], 0, 1)
        if mecanico_ids:
            return mecanico_ids[0]
        mecanico_id = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.mecanico', 'create',
            {'osvmec_funcionario_id': funcionario_id}, {})
        return mecanico_id

    def create_oservico_apontamento(self, dados, oservico_id, oservico_servicos_id):
        """
        O metodo cria um novo registro na tabela oservico_apontamento.
        :param oservico_id: id da tabela oservico_os, identificador da OS.
        :type oservico_id: int
        :param oservico_servicos_id: id da tabela oservico_servicos, identificador do servico da OS.
        :type oservico_servicos_id: int
        :return: inteiro
        :rtype: int
        """
        if not dados[15]:
            return False
        apontamento_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.apontamento', 'search',
            [('osvapo_os_id', '=', oservico_id), ('osvapo_servico_id', '=', oservico_servicos_id)],
            0, 1)
        if apontamento_ids:
            return apontamento_ids[0]
        osvapo_inicio = (base_date + relativedelta(days=int(float(dados[15])))).strftime(
            '%Y-%m-%d %H:%M:%S')
        osvapo_fim = False
        if dados[21]:
            osvapo_fim = (base_date + relativedelta(days=int(float(dados[15]))) + relativedelta(
                hours=float(dados[21]))).strftime('%Y-%m-%d %H:%M:%S')
        elif dados[16]:
            osvapo_fim = (base_date + relativedelta(days=int(float(dados[16])))).strftime(
                '%Y-%m-%d %H:%M:%S')
        state = dados[8]
        vals_apontamento = {
            'osvapo_os_id': oservico_id,
            'osvapo_servico_id': oservico_servicos_id,
            'osvapo_inicio': osvapo_inicio,
            'state': 'F' if state == 'FINALIZADO' else 'A',
            'osvapo_mecanico_id': self.check_oservico_mecanico(dados[1])
        }
        if osvapo_fim:
            vals_apontamento.update({'osvapo_fim': osvapo_fim})
        return self.conn.execute(self.database, self.uid, self.password, 'oservico.apontamento',
                                 'create', vals_apontamento, {})

    def check_estoque_categoria(self, categoria):
        """
        O metodo verifica a existencia de registro em estoque_categoria com a descricao em
        'categoria'. Caso nao exista, cria um novo.
        :param categoria: descricao do nome da categoria.
        :type categoria: str
        :return: registro da tabela estoque_categoria.
        :rtype: int
        """
        categoria_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.categoria', 'search',
            [('name', 'ilike', categoria)], 0, 1)
        if categoria_ids:
            return categoria_ids[0]
        categoria_id = self.conn.execute(self.database, self.uid, self.password,
                                         'estoque.categoria', 'create', {'name': categoria}, {})
        return categoria_id


if __name__ == "__main__":
    oservico = ImportaDadosOS()
    if not oservico.converts_xlsx_to_csv():
        print 'ERRO: Nao foi possivel converter os dados da planilha!'
        exec exit()
    arquivo = open(arquivo_csv, 'r')
    arquivo_dados = arquivo.readlines()
    os_nao_importada = []
    dado_inicial = ''
    for dado in arquivo_dados:
        if not dado:
            continue
        if '\n' in dado[-2:] and dado[-2:] != '\r\n':
            dado_inicial = dado
            dado_inicial = dado_inicial.replace(' \n', ' ')
            continue
        if dado_inicial:
            dado = dado_inicial + dado
            dado_inicial = ''
        dado = dado.replace('"', '').replace('\r\n', '')
        dado = dado.split('?')
        try:
            print dado[1]
        except:
            continue
        if not dado[1]:
            continue
        try:
            print dado[8]
        except:
            os_nao_importada.append(dado[1])
            continue
        oservico_id = oservico.create_oservico_os(dado)
        oservico_servicos_id = oservico.create_oservico_servicos(dado, oservico_id)
        oservico.create_oservico_apontamento(dado, oservico_id, oservico_servicos_id)
    arquivo.close()
    print 'ORDENS DE SERVICO IMPORTADAS COM SUCESSO!'
    if os_nao_importada:
        print 'As ordens de servico de numero a seguir nao foram importadas devido a caracter de ' \
              'quebra de linha na coluna "Local": \n' + ' \n'.join(os_nao_importada)
