# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import xlrd
import csv
from unicodedata import normalize, category
from dateutil.parser import parse
import pytz


def converte_tz(data, tz_origem, tz_destino):
    """
    Função para converter data de um timezone para outro
    @param data: data no formato datetime para ser convertida
    @param tz_origem: Timezone em que a data se encontra. Se None, vai buscar timezone do usuário
    @param tz_destino: Timezne em que a data será convertida. Se None, vai buscar timezone do usuário
    @return: data no formato datetime convertida para o novo timezone
    """
    obj_origem = pytz.timezone(tz_origem)
    obj_destino = pytz.timezone(tz_destino)
    x = obj_origem.localize(data, is_dst=False)
    return x.astimezone(obj_destino)


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


arquivo_csv = '/tmp/planilha_tesla.csv'
date = parse('1989-03-29 12:00:00')
date_tz = converte_tz(date, 'America/Sao_Paulo', 'UTC')
diff_hour = str(parse(date_tz.strftime('%Y-%m-%d %H:%M:%S')) - date)
if len(diff_hour) == 7:
    diff_hour = '0' + diff_hour
base_date = parse('1899-12-30T%s' % diff_hour)


class ImportaDadosOS():
    def __init__(self):
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.dir_arquivo = raw_input('Diretorio completo do arquivo: ')
        self.conn, self.uid = self.new_connection()

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(5)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def converts_xlsx_to_csv(self):
        res = False
        wb = xlrd.open_workbook(self.dir_arquivo)
        sh = wb.sheet_by_name('Planilha')
        open_file = open(arquivo_csv, 'wb')
        wr = csv.writer(open_file, quoting=csv.QUOTE_ALL, delimiter='?')
        for rownum in range(2, sh.nrows):
            dados = sh.row_values(rownum)
            try:
                wr.writerow(dados)
            except:
                for dado in dados:
                    if isinstance(dado, (int, float)) or not dado:
                        continue
                    dados[dados.index(dado)] = remove_acentuacao(dado)
                wr.writerow(dados)
                res = True
        open_file.close()
        return res

    def check_estoque_produto(self, name, categoria_id):
        """
        O metodo verifica a existencia do servico. Se houver registro, o atualiza setando a
        'categoria_id'.
        retorna Falso.
        :param name: descricao do servico.
        :type name: str
        :return: Verdadeiro, caso o registro seja atualizado, senao, Falso.
        :rtype: bool
        """
        produto_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.produto', 'search',
            [('name', '=', name[:60]), ('tipo', '=', 'S'), ('tipo_item', '=', '09')], 0, 1)
        if not produto_ids:
            return False
        produto_categoria_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.produto', 'search',
            [('id', 'in', produto_ids), ('categoria_id', '=', categoria_id)], 0, 1)
        if produto_categoria_ids:
            return True
        res = self.conn.execute(self.database, self.uid, self.password, 'estoque.produto', 'write',
                                produto_ids, {'categoria_id': categoria_id}, {})
        return res

    def check_estoque_categoria(self, categoria):
        """
        O metodo verifica a existencia de registro em estoque_categoria com a descricao em
        'categoria'. Caso nao exista, cria um novo.
        :param categoria: descricao do nome da categoria.
        :type categoria: str
        :return: registro da tabela estoque_categoria.
        :rtype: int
        """
        categoria_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.categoria', 'search',
            [('name', 'ilike', categoria)], 0, 1)
        if categoria_ids:
            return categoria_ids[0]
        categoria_id = self.conn.execute(self.database, self.uid, self.password,
                                         'estoque.categoria', 'create', {'name': categoria}, {})
        return categoria_id


if __name__ == "__main__":
    oservico = ImportaDadosOS()
    if not oservico.converts_xlsx_to_csv():
        print 'ERRO: Nao foi possivel converter os dados da planilha!'
        exec exit()
    arquivo = open(arquivo_csv, 'r')
    arquivo_dados = arquivo.readlines()
    os_nao_importada = []
    dado_inicial = ''
    for dado in arquivo_dados:
        if not dado:
            continue
        if '\n' in dado[-2:] and dado[-2:] != '\r\n':
            dado_inicial = dado
            dado_inicial = dado_inicial.replace(' \n', ' ')
            continue
        if dado_inicial:
            dado = dado_inicial + dado
            dado_inicial = ''
        dado = dado.replace('"', '').replace('\r\n', '')
        dado = dado.split('?')
        try:
            print dado[0]
        except:
            continue
        if not dado[0]:
            continue
        try:
            print dado[4]
        except:
            os_nao_importada.append(dado[0])
            continue
        categoria_id = oservico.check_estoque_categoria(dado[0])
        oservico.check_estoque_produto(dado[4], categoria_id)
    arquivo.close()
    print 'CATEGORIAS IMPORTADAS E/OU RELACIONADAS COM SUCESSO!'
    if os_nao_importada:
        print 'As categorias a seguir nao foram importadas devido a falta de ' \
              'conteudo na coluna "Atividade": \n' + ' \n'.join(os_nao_importada)
