# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import xlrd
import csv
from unicodedata import normalize, category
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
import pytz


def converte_tz(data, tz_origem, tz_destino):
    """
    Função para converter data de um timezone para outro
    @param data: data no formato datetime para ser convertida
    @param tz_origem: Timezone em que a data se encontra. Se None, vai buscar timezone do usuário
    @param tz_destino: Timezne em que a data será convertida. Se None, vai buscar timezone do usuário
    @return: data no formato datetime convertida para o novo timezone
    """
    obj_origem = pytz.timezone(tz_origem)
    obj_destino = pytz.timezone(tz_destino)
    x = obj_origem.localize(data, is_dst=False)
    return x.astimezone(obj_destino)


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


arquivo_csv = '/tmp/planilha_tesla.csv'
date = parse('1989-03-29 12:00:00')
date_tz = converte_tz(date, 'America/Sao_Paulo', 'UTC')
diff_hour = str(parse(date_tz.strftime('%Y-%m-%d %H:%M:%S')) - date)
if len(diff_hour) == 7:
    diff_hour = '0' + diff_hour
base_date = parse('1899-12-30T%s' % diff_hour)


class ImportaDadosOS():
    def __init__(self):
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.dir_arquivo = raw_input('Diretorio completo do arquivo: ')
        self.conn, self.uid = self.new_connection()

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(5)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def converts_xlsx_to_csv(self):
        res = False
        wb = xlrd.open_workbook(self.dir_arquivo)
        sh = wb.sheet_by_name('Planilha')
        open_file = open(arquivo_csv, 'wb')
        wr = csv.writer(open_file, quoting=csv.QUOTE_ALL, delimiter='?')
        for rownum in range(2, sh.nrows):
            dados = sh.row_values(rownum)
            try:
                wr.writerow(dados)
            except:
                for dado in dados:
                    if isinstance(dado, (int, float)) or not dado:
                        continue
                    dados[dados.index(dado)] = remove_acentuacao(dado)
                wr.writerow(dados)
                res = True
        open_file.close()
        return res

    def get_oservico_os(self, dados):
        """
        O metodo encontra um registro de OS, tabela oservico_os.
        :param dados: dados a serem importados da planilha.
        :type dados: list
        :return: registro da tabela oservico_os, caso encontre. Senao, Falso.
        :rtype: int
        """
        oservico_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.os', 'search',
            [('osvos_observacoes', 'ilike', '%' + dados[1] + '%')], 0, 1)
        if oservico_ids:
            return oservico_ids[0]
        return False

    def update_oservico_apontamento(self, dados, oservico_id):
        """
        O metodo atualiza um registro na tabela oservico_apontamento.
        :param dados: dados a serem importados da planilha.
        :type dados: list
        :param oservico_id: id da tabela oservico_os, identificador da OS.
        :type oservico_id: int
        :return: Verdadeiro se conseguir atualizar, senao, Falso
        :rtype: bool
        """
        if not dados[15]:
            return False
        apontamento_ids = self.conn.execute(
            self.database, self.uid, self.password, 'oservico.apontamento', 'search',
            [('osvapo_os_id', '=', oservico_id)],
            0, 1)
        if not apontamento_ids:
            return False
        apontamento_id = apontamento_ids[0]
        osvapo_inicio = (base_date + relativedelta(days=int(float(dados[15])))).strftime(
            '%Y-%m-%d %H:%M:%S')
        osvapo_fim = False
        if dados[21]:
            osvapo_fim = (base_date + relativedelta(days=int(float(dados[15]))) + relativedelta(
                hours=float(dados[21]))).strftime('%Y-%m-%d %H:%M:%S')
        elif dados[16]:
            osvapo_fim = (base_date + relativedelta(days=int(float(dados[16])))).strftime(
                '%Y-%m-%d %H:%M:%S')
        if not osvapo_fim:
            return False
        res = self.conn.execute(self.database, self.uid, self.password, 'oservico.apontamento',
                                'write', [apontamento_id], {'osvapo_fim': osvapo_fim}, {})
        return res


if __name__ == "__main__":
    oservico = ImportaDadosOS()
    if not oservico.converts_xlsx_to_csv():
        print 'ERRO: Nao foi possivel converter os dados da planilha!'
        exec exit()
    arquivo = open(arquivo_csv, 'r')
    arquivo_dados = arquivo.readlines()
    os_nao_importada = []
    dado_inicial = ''
    for dado in arquivo_dados:
        if not dado:
            continue
        if '\n' in dado[-2:] and dado[-2:] != '\r\n':
            dado_inicial = dado
            dado_inicial = dado_inicial.replace(' \n', ' ')
            continue
        if dado_inicial:
            dado = dado_inicial + dado
            dado_inicial = ''
        dado = dado.replace('"', '').replace('\r\n', '')
        dado = dado.split('?')
        try:
            print dado[1]
        except:
            continue
        if not dado[1]:
            continue

        oservico_id = oservico.get_oservico_os(dado)
        if oservico_id:
            oservico.update_oservico_apontamento(dado, oservico_id)
    arquivo.close()
    print 'APONTAMENTOS ATUALIZADOS COM SUCESSO!'
