# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import xlrd
import csv
from datetime import datetime
from unicodedata import normalize, category


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


def unicode_to_ascii(obj):
    """
    Converte texto unicode em ASCII, retirando acentos e caracteres especiais.
    """
    import unicodedata
    # o método só funciona para unicode, então, se for passado uma string, convertemos para unicode.
    if type(obj) == unicode:
        s = obj
    elif type(obj) == str:
        s = obj.decode('utf-8')
    else:
        s = unicode(obj)

    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))


def formataString(valorString, qtdBytes, caracter=' ', alinhamento="D", usaTrim="S", forcado='N'):
    """
    sintaxe formataString(ValorDaString, QuantidadeDeBytes, CaracterDePreenchimento, Alinhamemnto,
    usarTrim, forcado)

    Esta funcao, formata um string fornecida (ValorDaString), acrescentando o caracter
    (CaracterDePreenchimento), ateh que a string tenha o tamanho (QuantidadeDeBytes).
    O preenchimento acontece conforme o (alinhamento), onde por default, serÃ¡ a direita se nada
    for especifidado ou se 'alinhamento=D' Ã  esquerda se 'alinhamento=E' ou nas estremidades se
    'alinhamento=C'.
    Se usaTrim='S', espacos a esquerda e a direita sao retirados para depois fazer o alinhamento.
    Se forcado='S', caso a string seja maior do que o tamanho a alinhar,
       trunca a direta,
    se forcado='N', caso a string seja maior do que o tamanho a alinhar,
       retorna for -1
    """
    try:
        caracter = caracter[0]
    except:
        caracter = ' '
    valorString = unicode_to_ascii(valorString)

    if str(type(valorString)) == "<type 'unicode'>":
        sStringAFormatar = valorString
    else:
        sStringAFormatar = str(valorString)

    # retira espaco inicial / Final
    if usaTrim == "S":
        sStringAFormatar = sStringAFormatar.strip()

    if forcado == 'S':
        if len(sStringAFormatar) > qtdBytes:
            sStringAFormatar = sStringAFormatar[0:qtdBytes]

    iTamString = len(sStringAFormatar)
    if iTamString > qtdBytes:
        return "-1"
    if iTamString == qtdBytes:
        return sStringAFormatar
    if alinhamento == 'D':
        sStringAFormatar = sStringAFormatar.ljust(qtdBytes, caracter)
    if alinhamento == 'E':
        sStringAFormatar = sStringAFormatar.rjust(qtdBytes, caracter)
    if alinhamento == "C":
        sStringAFormatar = sStringAFormatar.center(qtdBytes, caracter)

    return sStringAFormatar


def ean(Codigo):
    """Funcao para calculo do digito de controle de codigos ean
      Recebe um codigo, e retorna o digito de controle"""
    try:
        Codigo = str(Codigo)
        if len(Codigo) > 13:
            Codigo = Codigo[0:13]
        else:
            Codigo = formataString(Codigo, 13, "0", alinhamento="E", usaTrim="S", forcado='N')
        t2 = int(Codigo[0:1]) + int(Codigo[2:3]) + int(Codigo[4:5]) + int(Codigo[6:7]) + int(
            Codigo[8:9]) + int(Codigo[10:11]) + int(Codigo[12:13])
        t1 = int(Codigo[1:2]) + int(Codigo[3:4]) + int(Codigo[5:6]) + int(Codigo[7:8]) + int(
            Codigo[9:10]) + int(Codigo[11:12])
        t2 = (t2 * 3) + t1
        t3 = (((t2 / 10) + 1) * 10) - t2
        if t3 == 10:
            t3 = 0
        return t3
    except:
        return '-1'


arquivo_csv = '/tmp/cadastro_produto.csv'


class ImportaDadosProduto():
    def __init__(self):
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.local = raw_input('Numero do local para cadastro dos produtos: ')
        self.dir_arquivo = raw_input('Diretorio completo da planilha: ')
        self.conn, self.uid = self.new_connection()
        self.local_id = self.check_estoque_local()

    def check_estoque_local(self):
        msg = 'ERRO: Local nao cadastrado.'
        local_id = False
        try:
            local_id = self.conn.execute(self.database, self.uid, self.password, 'estoque.local',
                                         'search', [('numero', '=', int(self.local))], 0, 1)
        except:
            print msg
            exec exit()
        if not local_id:
            print msg
            exec exit()
        return local_id[0]

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(5)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def converts_xlsx_to_csv(self):
        res = False
        wb = xlrd.open_workbook(self.dir_arquivo)
        sh = wb.sheet_by_index(0)
        open_file = open(arquivo_csv, 'wb')
        wr = csv.writer(open_file, quoting=csv.QUOTE_ALL, delimiter='?')
        for rownum in range(1, sh.nrows):
            dados = sh.row_values(rownum)
            try:
                wr.writerow(dados)
                res = True
            except:
                for dado in dados:
                    if isinstance(dado, (int, float)) or not dado:
                        continue
                    dados[dados.index(dado)] = remove_acentuacao(dado)
                wr.writerow(dados)
                res = True
        open_file.close()
        return res

    def create_estoque_produto(self, produto_dados):
        """
        O metodo cria um novo produto como tambem os demais registros relacionados.
        :param produto_dados: lista com os dados para cadastro do produto.
        :type produto_dados: list
        :return: id da tabela estoque_produto
        :rtype: int
        """
        # Codigo barra [1] /  Name [2] / Preco custo [3] / Preco venda [4] / Unidade [5] /
        # Embalagem [6] / NCM [14] / CEST [15] / Nat Rec PIS COFINS [18]
        name = produto_dados[2].replace('.', ' ').replace('  ', '')
        if not name:
            return False
        preco_custo = float(produto_dados[3].replace('R$', '').replace(' ', '').replace(',', '.')
                            or 0.0)
        preco_venda = float(produto_dados[4].replace('R$', '').replace(' ', '').replace(',', '.')
                            or 0.0)
        cest = produto_dados[15]
        unidade = produto_dados[5]
        embalagem = produto_dados[6]
        ncm = produto_dados[14]
        nat_rec_pis_cofins = produto_dados[18].replace(' ', '').split('/')
        nat_rec_pis = nat_rec_pis_cofins[0]
        nat_rec_cofins = nat_rec_pis_cofins[1]
        produto_ids = self.conn.execute(self.database, self.uid, self.password, 'estoque.produto',
                                        'search', [('name', '=', name)], 0, 1)
        produto_id = produto_ids and produto_ids[0]
        if not produto_id:
            ncm_ids = self.conn.execute(self.database, self.uid, self.password, 'fiscal.ncm',
                                        'search', [('codigo', '=', ncm)])
            cest_ids = self.conn.execute(self.database, self.uid, self.password, 'fiscal.cest',
                                         'search', [('fiscest_codigo', '=', cest)])
            vals_produto = {
                'name': name, 'tipo_item': '00', 'ncm_id': ncm_ids[0] if ncm_ids else False,
                'estpro_cest_id': cest_ids[0] if cest_ids else False
            }
            if nat_rec_pis:
                vals_produto.update({'estpro_pis_nat_rec': nat_rec_pis})
            if nat_rec_cofins:
                vals_produto.update({'estpro_cofins_nat_rec': nat_rec_cofins})
            produto_id = self.conn.execute(self.database, self.uid, self.password,
                                           'estoque.produto', 'create', vals_produto)
        # codigo_barras = (self.conn.execute(
        #     self.database, self.uid, self.password, 'estoque.produto', 'read', produto_id,
        #     ['codigo']).get('codigo')).zfill(13)
        codigo_barras = produto_dados[1][:-2].zfill(14) if produto_dados[1] else ''
        grade_id = False
        if produto_id:
            grade_ids = self.conn.execute(
                self.database, self.uid, self.password, 'estoque.produto.grade', 'search',
                [('produto_id', '=', produto_id)])
            if grade_ids:
                grade_id = grade_ids[0]
            self.create_fiscal_tributacao_encargos(produto_id, produto_dados)
        if produto_id and preco_custo:
            self.create_estoque_preco_custo(produto_id, preco_custo)
        if produto_id and preco_venda:
            self.create_estoque_preco_venda(produto_id, preco_venda)
        produto_embalagem_id = False
        if produto_id and unidade and embalagem:
            produto_embalagem_id = self.create_estoque_produto_embalagem(produto_id, unidade,
                                                                         embalagem)
        if produto_id and codigo_barras and produto_embalagem_id and grade_id:
            self.create_automacao_codigo_barra(produto_id, grade_id, produto_embalagem_id,
                                               codigo_barras, name)
        if produto_id and self.local_id and produto_embalagem_id and grade_id:
            self.create_estoque_local_locacao(produto_id, grade_id, produto_embalagem_id,
                                              self.local_id)
        return produto_id

    def create_estoque_produto_embalagem(self, produto_id, unidade, embalagem):
        """
        O metodo cria uma embalagem para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param unidade: sigla de representacao da unidade da embalagem.
        :type unidade: str
        :param embalagem: descricao e quantidade da embalagem.
        :type embalagem: str
        :return: id da tabela estoque_produto_embalagem
        :rtype: int
        """
        desc_embalagem = embalagem.replace(' ', '')
        embalagem = embalagem.replace(' ', '').split('/')
        unidade_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.embalagem.unidade', 'search',
            [('unidade', '=', unidade.upper())])
        unidade_id = unidade_ids and unidade_ids[0]
        print 'Unidade:', unidade.upper()
        if not unidade_ids:
            unidade_id = self.conn.execute(
                self.database, self.uid, self.password, 'estoque.embalagem.unidade', 'create',
                {'unidade': unidade, 'name': desc_embalagem})
        embalagem_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.embalagem', 'search',
            [('sigla', '=', embalagem[0].upper())])
        # , ('unidade_id', '=', unidade_id)]),
        # ('quantidade_na_embalagem', '=', float(embalagem[1]))])
        embalagem_id = embalagem_ids and embalagem_ids[0]
        if not embalagem_ids:
            embalagem_id = self.conn.execute(
                self.database, self.uid, self.password, 'estoque.embalagem', 'create',
                {'sigla': embalagem[0].upper(), 'unidade_id': unidade_id,
                 'quantidade_na_embalagem': float(embalagem[1]), 'name': desc_embalagem})
        produto_embalagem_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.produto.embalagem', 'search',
            [('produto_id', '=', produto_id), ('embalagem_id', '=', embalagem_id)])
        produto_embalagem_id = produto_embalagem_ids and produto_embalagem_ids[0]
        if not produto_embalagem_id:
            vals = {'produto_id': produto_id, 'embalagem_id': embalagem_id}
            produto_embalagem_id = self.conn.execute(self.database, self.uid, self.password,
                                                     'estoque.produto.embalagem', 'create', vals)
        return produto_embalagem_id

    def create_automacao_codigo_barra(self, produto_id, grade_id, produto_embalagem_id,
                                      codigo_barra, name):
        """
        O metodo cria um registro de codigo de barras.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem.
        :type produto_embalagem_id: int
        :param codigo_barra: numeros que compoem o codigo de barras.
        :type codigo_barra: str
        :return: id da tabela automacao_codigo_barra
        :rtype: int
        """
        # codigo_barra += str(ean(codigo_barra))
        codigo_barra_ids = self.conn.execute(
            self.database, self.uid, self.password, 'automacao.codigo.barra', 'search',
            [('grade_id', '=', grade_id), ('produto_embalagem_id', '=', produto_embalagem_id),
             ('codigo_barra', '=', codigo_barra)])
        codigo_barra_id = codigo_barra_ids and codigo_barra_ids[0]
        if not codigo_barra_id:
            usa_digito = True
            if '***' in name:
                # quando tiver '***' indica que o codigo de barras do produto nao esta no padrao,
                # ele sera corrigido depois pelo usuario
                usa_digito = False
            vals = {'produto_id': produto_id, 'grade_id': grade_id,
                    'produto_embalagem_id': produto_embalagem_id, 'codigo_barra': codigo_barra,
                    'autcba_usa_digito': usa_digito}
            codigo_barra_id = self.conn.execute(self.database, self.uid, self.password,
                                                'automacao.codigo.barra', 'create', vals)
        return codigo_barra_id

    def create_estoque_local_locacao(self, produto_id, grade_id, produto_embalagem_id,
                                     local_id):
        """
        O metodo cria um registro de codigo de barras.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem.
        :type produto_embalagem_id: int
        :param local_id: id da tabela estoque_local.
        :type local_id: int
        :return: id da tabela estoque_local_locacao
        :rtype: int
        """
        locacao_ids = self.conn.execute(
            self.database, self.uid, self.password, 'estoque.local.locacao', 'search',
            [('grade_id', '=', grade_id),
             ('estllc_produto_embalagem_id', '=', produto_embalagem_id),
             ('local_id', '=', local_id)])
        locacao_id = locacao_ids and locacao_ids[0]
        if not locacao_id:
            vals = {'produto_id': produto_id, 'grade_id': grade_id,
                    'estllc_produto_embalagem_id': produto_embalagem_id, 'local_id': local_id}
            locacao_id = self.conn.execute(self.database, self.uid, self.password,
                                           'estoque.local.locacao', 'create', vals)
        return locacao_id

    def create_estoque_preco_custo(self, produto_id, preco_custo):
        """
        O metodo cria um registro de preco de custo para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param preco_custo: valor do custo.
        :type preco_custo: float
        :return: id da tabela estoque_preco_custo
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'estprc_custo_reposicao': preco_custo,
                'estprc_custo_corrigido': preco_custo, 'estprc_custo_medio_contabil': preco_custo,
                'estprc_ultimo_custo': preco_custo, 'estprc_data': str(datetime.now())[:19] }
        custo_id = self.conn.execute(self.database, self.uid, self.password, 'estoque.preco.custo',
                                     'create', vals)
        return custo_id

    def create_estoque_preco_venda(self, produto_id, preco_venda):
        """
        O metodo cria um registro de preco de venda para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param preco_venda: valor para venda.
        :type preco_venda: float
        :return: id da tabela estoque_preco_venda
        :rtype: int
        """
        vals = {'produto_id': produto_id, 'preco_venda': preco_venda}
        preco_venda_id = self.conn.execute(self.database, self.uid, self.password,
                                           'estoque.preco.venda', 'create', vals,
                                           {'cadastro_produto': True})
        return preco_venda_id

    def create_fiscal_tributacao_encargos(self, produto_id, produto_dados):
        """
        O metodo cria os registros da tributacao para o produto.
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param produto_dados: lista com os dados para cadastro do produto.
        :type produto_dados: list
        :return: Verdadeiro
        :rtype: bool
        """
        # CST_ICMS [6] / BASE_ICMS [7] / ALIQUOTA_ICMS [8] / OUTRAS_ICMS [9] / ISENTOS_ICMS [10]
        # ALIQUOTA_PIS [15] / ALIQUOTA_COFINS [16] / CST_PIS_COFINS [13]
        cst_icms = str(int(float(produto_dados[9]))).zfill(2)
        try:
            base_icms = float(produto_dados[10])
        except:
            base_icms = 0
        try:
            aliquota_icms = float(produto_dados[11])
        except:
            aliquota_icms = 0
        try:
            outras_icms = float(produto_dados[12])
        except:
            outras_icms = 0
        try:
            isentos_icms = float(produto_dados[13])
        except:
            isentos_icms = 0
        try:
            aliquota_pis = float(produto_dados[19])
        except:
            aliquota_pis = 0
        try:
            aliquota_cofins = float(produto_dados[20])
        except:
            aliquota_cofins = 0

        cst_pis_cofins = produto_dados[17].split('/')
        cst_pis = cst_pis_cofins[0].split('-')
        cst_cofins = cst_pis_cofins[1].split('-')

        icms_encargo_id = self.conn.execute(self.database, self.uid, self.password,
                                            'fiscal.encargo', 'search', [('sigla', '=', 'ICMS')])[0]
        pis_encargo_id = self.conn.execute(self.database, self.uid, self.password,
                                           'fiscal.encargo', 'search', [('sigla', '=', 'PIS')])[0]
        cofins_encargo_id = self.conn.execute(self.database, self.uid, self.password,
                                              'fiscal.encargo', 'search',
                                              [('sigla', '=', 'COFINS')])[0]
        pis_entrada_cst_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', cst_pis[0]), ('fiscst_tipo_movimento', '=', 'E')])
        cofins_entrada_cst_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', cst_cofins[0]), ('fiscst_tipo_movimento', '=', 'E')])

        pis_saida_cst_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', cst_pis[1]), ('fiscst_tipo_movimento', '=', 'S')])
        cofins_saida_cst_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', cst_cofins[1]), ('fiscst_tipo_movimento', '=', 'S')])

        icms_tipo_cst = 'T'
        if cst_icms == '60':
            icms_tipo_cst = 'U'
        elif outras_icms:
            icms_tipo_cst = 'O'
        elif isentos_icms:
            icms_tipo_cst = 'I'
        cst_ids = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.cst', 'search',
            [('codigo', '=', cst_icms), ('tipo', '=', icms_tipo_cst),
             ('fiscst_tipo_movimento', '=', 'A')])
        print "----\nCodigo ICMS:", cst_icms, '  Tipo CST:', icms_tipo_cst

        # ICMS
        args_icms = [('encargo_id', '=', icms_encargo_id), ('cst_id', 'in', cst_ids),
                     ('aliquota', '=', aliquota_icms)]
        if icms_tipo_cst == 'T':
            args_icms.append(('base', '=', base_icms))
        print 'Argumentos para consulta de tributacao:', args_icms
        icms_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search', args_icms)[0]
        vals = {'produto_id': produto_id, 'encargo_id': icms_encargo_id, 'cst_id': cst_ids[0],
                'tipo_tributacao': 'T', 'tributacao_id': icms_tributacao_id}
        if not self.conn.execute(
                self.database, self.uid, self.password, 'fiscal.tributacao.encargos', 'search',
                [('produto_id', '=', produto_id), ('encargo_id', '=', icms_encargo_id),
                 ('tributacao_id', '=', icms_tributacao_id)]):
            self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                              'create', vals)

        # PIS ENTRADA
        pis_entrada_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', pis_encargo_id), ('cst_id', 'in', pis_entrada_cst_ids),
             ('aliquota', '=', aliquota_pis)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': pis_encargo_id,
                'cst_id': pis_entrada_cst_ids[0],
                'tipo_tributacao': 'E', 'tributacao_id': pis_entrada_tributacao_id}
        if not self.conn.execute(
                self.database, self.uid, self.password, 'fiscal.tributacao.encargos', 'search',
                [('produto_id', '=', produto_id), ('encargo_id', '=', pis_encargo_id),
                 ('tributacao_id', '=', pis_entrada_tributacao_id)]):
            self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                              'create', vals)

        # PIS SAIDA
        pis_saida_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', pis_encargo_id), ('cst_id', 'in', pis_saida_cst_ids),
             ('aliquota', '=', aliquota_pis)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': pis_encargo_id,
                'cst_id': pis_saida_cst_ids[0],
                'tipo_tributacao': 'T', 'tributacao_id': pis_saida_tributacao_id}
        if not self.conn.execute(
                self.database, self.uid, self.password, 'fiscal.tributacao.encargos', 'search',
                [('produto_id', '=', produto_id), ('encargo_id', '=', pis_encargo_id),
                 ('tributacao_id', '=', pis_saida_tributacao_id)]):
            self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                              'create', vals)

        # COFINS ENTRADA
        cofins_entrada_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', cofins_encargo_id), ('cst_id', 'in', cofins_entrada_cst_ids),
             ('aliquota', '=', aliquota_cofins)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': cofins_encargo_id,
                'cst_id': cofins_entrada_cst_ids[0],
                'tipo_tributacao': 'E', 'tributacao_id': cofins_entrada_tributacao_id}
        if not self.conn.execute(
                self.database, self.uid, self.password, 'fiscal.tributacao.encargos', 'search',
                [('produto_id', '=', produto_id), ('encargo_id', '=', cofins_encargo_id),
                 ('tributacao_id', '=', cofins_entrada_tributacao_id)]):
            self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                              'create', vals)

        # COFINS SAIDA
        cofins_saida_tributacao_id = self.conn.execute(
            self.database, self.uid, self.password, 'fiscal.tributacao', 'search',
            [('encargo_id', '=', cofins_encargo_id), ('cst_id', 'in', cofins_saida_cst_ids),
             ('aliquota', '=', aliquota_cofins)])[0]
        vals = {'produto_id': produto_id, 'encargo_id': cofins_encargo_id,
                'cst_id': cofins_saida_cst_ids[0],
                'tipo_tributacao': 'T', 'tributacao_id': cofins_saida_tributacao_id}
        if not self.conn.execute(
                self.database, self.uid, self.password, 'fiscal.tributacao.encargos', 'search',
                [('produto_id', '=', produto_id), ('encargo_id', '=', cofins_encargo_id),
                 ('tributacao_id', '=', cofins_saida_tributacao_id)]):
            self.conn.execute(self.database, self.uid, self.password, 'fiscal.tributacao.encargos',
                              'create', vals)
        return True


if __name__ == "__main__":
    imp_pro = ImportaDadosProduto()
    if not imp_pro.converts_xlsx_to_csv():
        raise Exception('Nao foi possivel converter os dados da planilha!')
    arquivo = open(arquivo_csv, 'r')
    arquivo_dados = arquivo.readlines()
    for dado in arquivo_dados:
        if not dado:
            continue
        dado = dado.replace('"', '').replace('\r\n', '')
        dado = dado.split('?')
        if not filter(lambda d: d, dado):
            continue
        print dado[2]
        imp_pro.create_estoque_produto(dado)
    arquivo.close()
    print ''
    print 'PRODUTOS IMPORTADOS COM SUCESSO!'
