# -*- coding: utf-8 -*-

import xmlrpclib
import socket
import string
import xlrd
import csv


def remove_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    from unicodedata import normalize, category

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')

    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod, 'ignore'))))


def remove_caracter(caracteres, letras=True, numeros=True, espaco=True, outros=None,
                    substituto=''):
    """
    @Parametros: caracteres - Eh a palavra ou frase que deseja remover caracters
                              especiais - Tipo: string
                 letras     - Se True filtra letras da sequencia informada - Tipo boolean
                 numeros    - Se True filtra os digitos da sequencia de caracteres
                              informados - Tipo: boolean
                 espaco     - Se True filtra os espacos em branco - Tipo: boolean
                 outros     - uma lista de outros caracteres que deseja filtrar - Tipo: lista
                 substituto - caracter que substituira o caracter especial removido
    @Funcao: Remover caracters especiais de uma sequencia de caracteres
    @Retorno: Retorna uma string tratada de acordo com as informacoes passadas no
              parametro - Tipo: string
    """
    if not caracteres:
        return ''
    if not isinstance(outros, list):
        outros = []
    char_validos = ''
    if letras:
        char_validos += string.letters
    if numeros:
        char_validos += string.digits
    if espaco:
        char_validos += ' '
    for i in outros:
        char_validos += i
    new_caracters = ''.join([char if char in char_validos else substituto for char in
                             caracteres])
    return new_caracters


arquivo_csv = '/tmp/cadastro_cliente.csv'


class ImportaClienteDados():
    def __init__(self):
        # self.arquivo_problema_cep = open('/tmp/cliente_problema_cep.txt', 'a')
        # self.arquivo_problema_cpf_cnpj = open('/tmp/cliente_problema_cpf_cnpj.txt', 'a')
        self.database = str(raw_input('Nome do banco de dados: '))
        self.host = str(raw_input('Host/IP OpenERP: '))
        self.user = str(raw_input('Usuario OpenERP: '))
        self.password = str(raw_input('Senha OpenERP: '))
        self.port = raw_input('Porta do OpenERP: ')
        self.dir_arquivo = raw_input('Diretorio completo da planilha: ')
        self.conn, self.uid = self.new_connection()
        # self.endereco_tipo_id = self.conn.execute(
        #     self.database, self.uid, self.password, 'adm.endereco.tipo', 'search',
        #     [('name', '=', 'Normal')], 0, 1, 'id')[0]
        self.endereco_tipo_cobranca_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.endereco.tipo', 'search',
            [('name', '=', 'Cobranca')], 0, 1, 'id')[0]
        self.telefone_tipo_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.telefone.tipo', 'search',
            [('name', '=', 'Comercial')], 0, 1, 'id')[0]
        self.telefone_tipo_celular_id = self.conn.execute(
            self.database, self.uid, self.password, 'adm.telefone.tipo', 'search',
            [('name', '=', 'Celular')], 0, 1, 'id')[0]
        # self.telefone_tipo_fax_id = self.conn.execute(
        #     self.database, self.uid, self.password, 'adm.telefone.tipo', 'search',
        #     [('name', '=', 'Fax')], 0, 1)[0]

    def new_connection(self):
        """
        O metodo estabalece a conexao com o Odoo.
        :return: objeto da conexao e id do usuario logado.
        :rtype: object
        """
        if not self.password:
            self.password = None
        try:
            sock_common = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                                '/xmlrpc/common')
            socket.setdefaulttimeout(5)
            uid = sock_common.login(self.database, self.user, self.password)
            sock = xmlrpclib.ServerProxy('http://' + self.host + ':' + str(self.port) +
                                         '/xmlrpc/object')
            return sock, uid
        except:
            print 'Nao foi possivel conectar ao OpenERP!'
            exec exit()

    def converts_xlsx_to_csv(self):
        res = False
        wb = xlrd.open_workbook(self.dir_arquivo)
        sh = wb.sheet_by_index(0)
        open_file = open(arquivo_csv, 'wb')
        wr = csv.writer(open_file, quoting=csv.QUOTE_ALL, delimiter='?')
        for rownum in range(1, sh.nrows):
            dados = sh.row_values(rownum)
            try:
                wr.writerow(dados)
                res = True
            except:
                for dado in dados:
                    if isinstance(dado, (int, float)) or not dado:
                        continue
                    dados[dados.index(dado)] = remove_acentuacao(dado)
                wr.writerow(dados)
                res = True
        open_file.close()
        return res

    def create_pessoa(self, pessoa_dados=[]):
        """
        O metodo cria uma nova pessoa.
        :return: id da tabela adm_pessoa
        :rtype: int
        """

        # A ordem abaixo se refere a posicao na lista python:
        # CPF_CNPJ [2] / Codigo [1] / Fantasia [1] / R.Social [0] / Endereco [3] / Numero [4] /
        # Bairro [5] / E-mail [12] / Complemento [13] / Tipo pessoa [15]
        # Cidade [6] / CEP [8] / Estado [7] / Telefone [9] [10] / Celular [11] /
        # IE [14] / Data nascimento [21] / RG [61]
        razao_social = pessoa_dados[0]
        nome_fantasia = pessoa_dados[1]
        cpf_cnpj = pessoa_dados[2]
        endereco = pessoa_dados[3]
        numero = pessoa_dados[4]
        bairro = pessoa_dados[5]
        cidade = pessoa_dados[6]
        estado = pessoa_dados[7]
        cep = pessoa_dados[8]
        telefone1 = pessoa_dados[9]
        telefone2 = pessoa_dados[10]
        celular = pessoa_dados[11]
        email = pessoa_dados[12]
        complemento = pessoa_dados[13] or ''
        ie = pessoa_dados[14]
        tipo_pessoa = pessoa_dados[15]

        vals_endereco_cobranca = dict()
        args_endereco_cobranca = []
        vals_telefone1 = dict()
        vals_telefone2 = dict()
        vals_celular = {}

        print razao_social
        cpf_cnpj = filter(lambda cc: cc.isdigit(), cpf_cnpj)
        name = remove_acentuacao(razao_social, 'ASCII')
        name = remove_caracter(name, numeros=False, outros=['/'])
        nome_fantasia = remove_acentuacao(nome_fantasia, 'ASCII')
        nome_fantasia = remove_caracter(nome_fantasia, numeros=False, outros=['/'])
        if len(filter(lambda nf: nf, name.split(' '))) == 1:
            name += ' ' + nome_fantasia
        pessoa_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                       'search', [('cpf_cnpj', '=', cpf_cnpj or False),
                                                  ('name', '=', name)], 0, 1) or False
        pessoa_id = pessoa_ids and pessoa_ids[0]
        if cpf_cnpj and not pessoa_id:
            pessoa_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                           'search', [('cpf_cnpj', '=', cpf_cnpj or False)], 0, 1) \
                         or False
            pessoa_id = pessoa_ids and pessoa_ids[0]

        list_endereco = endereco.split(' ')
        descricao = remove_acentuacao(' '.join(list_endereco[1:]), 'ASCII')
        logradouro_id = False
        logradouro_ids = self.conn.execute(
            self.database, self.uid, self.password, 'adm.logradouro', 'search',
            [('titulo', '=', list_endereco[0]),
             ('descricao', '=', descricao)])
        if logradouro_ids:
            logradouro_id = logradouro_ids[0]
        logradouro_id = logradouro_id or self.conn.execute(
            self.database, self.uid, self.password, 'adm.logradouro', 'create',
            {'titulo': list_endereco[0], 'descricao': descricao})
        vals_endereco_cobranca.update({'logradouro_id': logradouro_id,
                                       'complemento': complemento})
        args_endereco_cobranca.append(('logradouro_id', '=', logradouro_id))
        numero = numero.replace('.0', '')
        if numero.isdigit():
            numero = str(int(numero.replace('.', '')))
        vals_endereco_cobranca.update({'numero': numero})
        args_endereco_cobranca.append(('numero', '=', numero))
        bairro = remove_acentuacao(bairro, 'ASCII')
        if bairro:
            bairro_ids = self.conn.execute(
                self.database, self.uid, self.password, 'adm.bairro', 'search',
                [('name', '=', bairro)], 0, 1)
            bairro_id = bairro_ids[0] if bairro_ids else self.conn.execute(
                self.database, self.uid, self.password, 'adm.bairro', 'create', {'name': bairro})
            vals_endereco_cobranca.update({'bairro_id': bairro_id})
            args_endereco_cobranca.append(('bairro_id', '=', bairro_id))
        uf_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.uf',
                                   'search', [('sigla', '=', estado.upper())],
                                   0, 1)
        cidade = remove_acentuacao(cidade, 'ASCII')
        args_cidade = [('uf_id', '=', uf_ids[0] if uf_ids else False), ('name', 'ilike', cidade)]
        cidade_ids = self.conn.execute(self.database, self.uid, self.password, 'adm.cidade',
                                       'search', args_cidade, 0, 1)
        cidade_id = False
        if uf_ids and not cidade_ids:
            vals_cidade = {'uf_id': uf_ids[0], 'name': cidade}
            cidade_id = self.conn.execute(
                self.database, self.uid, self.password, 'adm.cidade', 'create', vals_cidade)
        elif uf_ids and cidade_ids:
            cidade_id = cidade_ids[0]
        if cidade_id:
            pais_id = self.conn.execute(
                self.database, self.uid, self.password, 'adm.uf', 'read',
                uf_ids[0], ['pais_id']).get('pais_id')[0]
            vals_endereco_cobranca.update({'cidade_id': cidade_id, 'uf_id': uf_ids[0],
                                           'pais_id': pais_id})
            args_endereco_cobranca.append(('cidade_id', '=', cidade_id))
        if cep and self.endereco_tipo_cobranca_id:
            vals_endereco_cobranca.update({'cep': cep,
                                           'endereco_tipo_id': self.endereco_tipo_cobranca_id})
            args_endereco_cobranca.append(('cep', '=', cep.replace('.', '').replace('-', '')))

        # TELEFONES
        if filter(lambda cc: cc.isdigit(), telefone1) and self.telefone_tipo_id:
            telefone1 = telefone1.split(')')
            vals_telefone1.update({'telefone_tipo_id': self.telefone_tipo_id,
                                   'numero': telefone1[1], 'ddd': telefone1[0][1:]})
        if filter(lambda cc: cc.isdigit(), telefone2) and self.telefone_tipo_id:
            telefone2 = telefone2.split(')')
            vals_telefone2.update({'telefone_tipo_id': self.telefone_tipo_id,
                                   'numero': telefone2[1], 'ddd': telefone2[0][1:]})
        if filter(lambda cc: cc.isdigit(), celular) and self.telefone_tipo_celular_id:
            celular = celular.split(')')
            vals_celular.update({'telefone_tipo_id': self.telefone_tipo_celular_id,
                                 'numero': celular[1], 'ddd': celular[0][1:]})

        if not pessoa_id:
            vals = {
                'name': name, 'fantasia_apelido': nome_fantasia,
                'cpf_cnpj': cpf_cnpj or False, 'pessoa_tipo': tipo_pessoa,
                'admpes_jur_ie': ie.replace('\r\n', '').replace('.0', ''),
            }

            try:
                pessoa_id = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
                                              'create', vals)
            except Exception as e:
                # if 'CPF/CNPJ Invalido' in str(e):
                #     self.arquivo_problema_cpf_cnpj.write(
                #         str(pessoa_dados[1]) + ' - ' + str(cpf_cnpj) + ' - ' + name + '\r\n')
                #     return pessoa_id
                # else:
                #     raise e
                raise str(e)

        if pessoa_id and vals_telefone1:
            vals_telefone1.update({'pessoa_id': pessoa_id})
            if not self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                     'search', [('pessoa_id', '=', pessoa_id),
                                                ('numero', '=', telefone1[1].replace('-', ''))]):
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                  'create', vals_telefone1)
        if pessoa_id and vals_telefone2:
            vals_telefone2.update({'pessoa_id': pessoa_id})
            if not self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                     'search', [('pessoa_id', '=', pessoa_id),
                                                ('numero', '=', telefone2[1].replace('-', ''))]):
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                  'create', vals_telefone2)
        if pessoa_id and vals_celular:
            vals_celular.update({'pessoa_id': pessoa_id})
            if not self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                     'search', [('pessoa_id', '=', pessoa_id),
                                                ('numero', '=', celular[1].replace('-', ''))]):
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.telefone',
                                  'create', vals_celular)

        # codigo = self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa',
        #                            'read', pessoa_id, ['codigo']).get('codigo')
        pessoa_endereco_id = False
        if args_endereco_cobranca:
            args_endereco_cobranca.append(('pessoa_id', '=', pessoa_id))
            pessoa_endereco_ids = self.conn.execute(
                self.database, self.uid, self.password, 'adm.pessoa.endereco', 'search',
                args_endereco_cobranca)
            pessoa_endereco_id = pessoa_endereco_ids and pessoa_endereco_ids[0]
        if pessoa_id and vals_endereco_cobranca and vals_endereco_cobranca.get('pais_id') and not \
                pessoa_endereco_id:
            vals_endereco_cobranca.update({'pessoa_id': pessoa_id})
            try:
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.endereco',
                                  'create', vals_endereco_cobranca)
            except Exception as e:
                # if 'CEP invalido' in str(e):
                #     self.arquivo_problema_cep.write(str(codigo) + ' - ' + name + '\r\n')
                # else:
                #     raise e
                raise str(e)
        if pessoa_id and email:
            email_tipo_id = self.conn.execute(self.database, self.uid, self.password,
                                              'adm.email.tipo', 'search',
                                              [('name', '=', 'Comum')])[0]
            email = email.replace(' ', '')
            if not self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.email',
                                     'search', [('pessoa_id', '=', pessoa_id),
                                                ('name', '=', email)]):
                vals_email = {'pessoa_id': pessoa_id, 'email_tipo_id': email_tipo_id,
                              'name': email}
                self.conn.execute(self.database, self.uid, self.password, 'adm.pessoa.email',
                                  'create', vals_email)
        return pessoa_id

    def create_financeiro_cliente(self, pessoa_id):
        """
        O metodo cria um novo cliente.
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :return: id da tabela financeiro_cliente
        :rtype: int
        """
        if not self.conn.execute(self.database, self.uid, self.password,
                                 'financeiro.cliente', 'search', [('pessoa_id', '=', pessoa_id)]):
            self.conn.execute(self.database, self.uid, self.password, 'financeiro.cliente',
                              'create', {'pessoa_id': pessoa_id})
        return True


if __name__ == "__main__":
    i_cli = ImportaClienteDados()
    if not i_cli.converts_xlsx_to_csv():
        raise Exception('Nao foi possivel converter os dados da planilha!')
    arquivo = open(arquivo_csv, 'r')
    arquivo_dados = arquivo.readlines()
    for dado in arquivo_dados:
        dado = dado.replace('"', '').replace('\r\n', '')
        dado = dado.split('?')
        pessoa_id = i_cli.create_pessoa(dado)
        if not pessoa_id:
            continue
        i_cli.create_financeiro_cliente(pessoa_id)
    arquivo.close()
    # i_cli.arquivo_problema_cep.close()
    # i_cli.arquivo_problema_cpf_cnpj.close()
    print 'Clientes importados com sucesso!'
