# -*- coding: utf-8 -*-

# Este script cria o campo xml_proc no supervisor para as notas que estao faturadas e
# tenham valor no campo xml_gerado.


import psycopg2.extras
from decimal import Decimal
from nfe.nfe_400.manual_600.nfe_400 import AutXML, DetPag
from nfe.nfce.consrecinfce import ProcNFCe
from nfe.nfce.nfce import Det
from nfe.nfce.webservices import ESTADO_WS
from nfe.pysped.xml_sped.certificado import Certificado
from nfe.nfce.webservices_flag import WS_CONSULTA_QRCODE, WS_CONSULTA_CHAVE
from nfe.nfce import qr_code
from nfe.nfce.nfce import NFCe
from datetime import datetime
from nfe.nfc_e import nfce as nfc_e
import socket
import pytz
import re

HOMOLOGACAO = 2
TWOPLACES = Decimal(10) ** -2


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class get_browse(object):
    def __init__(self, d):
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [get_browse(x) if isinstance(x, dict) else x for x in b])
            else:
                setattr(self, a, get_browse(b) if isinstance(b, dict) else b)


class openerp(object):

    def gerar_nfce(self, cr, uid, brw_config, brw_capa, ambiente, context):
        """
        Gera o XML da nota fiscal eletronica (saida)
        """
        nfe = NFCe()
        if not brw_capa.supcap_xml_gerado:
            raise Exception('Nota sem xml!')

        nfe.xml = brw_capa.supcap_xml_gerado
        self._fisnfc_set_nfe_infnfe(cr, uid, brw_config, brw_capa, nfe, context)
        nfe.chave = nfe.infNFe.Id.valor[3:]
        nfe.infNFe.ide.cNF.valor = nfe.chave[35:43]
        # nfe.monta_chave()
        # nfe.infNFe.ide.cDV.valor = nfe.infNFe.Id.valor[-1]
        self.fisnfc_set_nfe_infnfesupl(brw_config, brw_capa, nfe, ambiente)
        return nfe

    def suputi_converte_tz(self, data, tz_origem=None, tz_destino=None):
        """
        Função para converter data de um timezone para outro
        @param data: data no formato datetime para ser convertida
        @param tz_origem: Timezone em que a data se encontra. Se None, vai buscar timezone do usuário
        @param tz_destino: Timezne em que a data será convertida. Se None, vai buscar timezone do usuário
        @return: data no formato datetime convertida para o novo timezone
        """

        obj_origem = pytz.timezone(tz_origem)
        obj_destino = pytz.timezone(tz_destino)
        x = obj_origem.localize(data, is_dst=False)
        return x.astimezone(obj_destino)

    @staticmethod
    def suputi_remover_acentuacao(text, cod='utf-8'):
        """
        Funcao Wrapper para remover acentuacao
        :param text:texto onde sera feita a remocao da acentuacao
        :param cod:Tipo de codificacao
        return:string
        """
        from unicodedata import normalize, category

        try:
            unicode_string = str(text)
        except UnicodeEncodeError:
            return normalize('NFKD', text).encode('ASCII', 'ignore')

        return unicode(filter(lambda c: category(c) != 'Mn',
                              normalize('NFKD', unicode_string.decode(cod))))

    def fisnfc_set_nfe_infnfesupl(self, brw_config, brw_capa, nfe, ambiente):
        """
        Atribui os valores do QR-Code e da URL de consulta por chave da NFC-e.
        """
        # //NFe.infNFeSupl - Informacoes complementares da NFC-e
        uf = brw_config.supcfc_estado
        nfe.chave or nfe.gera_nova_chave()
        versao = '4.00'
        nfe.infNFeSupl.qrCode.valor = self._qrcode_v2(brw_config, brw_capa, nfe, uf, versao, ambiente)
        nfe.infNFeSupl.urlChave.valor = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_CHAVE][
            'url']

    def _qrcode_v2(self, brw_config, brw_capa, nfce, uf, versao, ambiente):
        """
        Constroi texto do QR-Code versao 2.0 da nota fiscal de consumidor.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param nfce: objeto nota fiscal eletronica - NFe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type nfce: NFCe
        :type context: dict
        :return: texto com o QR-Code a ser impresso no DANFE NFC-e.
        :rtype: str
        """
        nfce.chave or nfce.gera_nova_chave()
        url_qrcode = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_QRCODE]['url'] or u''
        csc = brw_config.supcfc_csc or u''
        csc_id = 1 if csc else 2
        if ambiente == HOMOLOGACAO:
            csc_id = 1
        csc = [csc_id, csc]

        # DigestValue em funcao da contingencia OFFLINE
        if nfce.chave[34] in ('5', '9'):
            # Assina a NFC-e para recuperar o DigestValue
            certificado = Certificado()
            certificado.certificado = brw_config.supcfc_certificado_conteudo
            certificado.chave = brw_config.supcfc_chave_conteudo
            certificado.assina_xmlnfe(nfce)
            result = qr_code.qrcode_v2_offline(ambiente, url_qrcode, nfce.chave,
                                               nfce.Signature.DigestValue, csc,
                                               datetime.strptime(str(brw_capa.supcap_data_cupom), '%Y-%m-%d').day,
                                               brw_capa.supcap_total_cupom)
        else:
            result = qr_code.qrcode_v2_online(ambiente, url_qrcode, nfce.chave, csc)

        return result

    def _fisnfc_set_nfe_infnfe(self, cr, uid, brw_config, brw_capa, nfe, context):
        # Identificacao da nota
        self._fisnfc_set_nfe_infnfe_ide(cr, uid, brw_config, brw_capa, nfe)

        # Dados do emitente
        self._fisnfc_set_nfe_infnfe_emit(cr, uid, brw_capa, brw_config, nfe)

        # Dados do destinatario
        self._fisnfc_set_nfe_infnfe_dest(cr, uid, brw_capa, nfe)

        # Autorizados download XML
        self._fisnfc_set_nfe_infnfe_autxml(brw_config, nfe)

        # Dados de pagamento
        self._fisnfc_set_nfe_infnfe_pag(cr, brw_capa, nfe)

        # todo: verificar a tranportadora
        # Dados da transportadora
        self._fisnfc_set_nfe_infnfe_transp(nfe)

        # Itens da NF
        self._fisnfc_set_nfe_infnfe_prod(cr, uid, brw_config, brw_capa, nfe, context)

        # Total produtos
        self._fisnfc_set_nfe_infnfe_total_icmstot(brw_capa, nfe)

    def _fisnfc_set_nfe_infnfe_ide(self, cr, uid, brw_config, brw_capa, nfe):
        # //NFe.infNFe.Ide - Identificacao da NF-e
        # nfe.infNFe.ide.cUF.valor = UF_CODIGO.get(brw_config.supcfc_estado) or False
        # nfe.infNFe.ide.cNF.valor = str(brw_capa.id)[:8].zfill(8)
        nfe.infNFe.ide.natOp.valor = 'Venda para consumidor final'
        # nfe.infNFe.ide.mod.valor = 65
        # nfe.infNFe.ide.serie.valor = brw_capa.supcap_caixa
        # nfe.infNFe.ide.nNF.valor = brw_capa.supcap_nro_cupom
        nfe.infNFe.ide.procEmi.valor = 0  # 0=Emissao de NF-e com aplicativo do contribuinte;
        data = datetime.strptime(str(brw_capa.supcap_data_fim)[:19],
                                 '%Y-%m-%d %H:%M:%S')
        data = self.suputi_converte_tz(data, tz_origem='UTC', tz_destino='America/Sao_Paulo')
        # nfe.infNFe.ide.dhEmi.valor = data
        nfe.infNFe.ide.indPres.valor = '1' or '4'  # 4 = em operacao com entrega a domicilio
        nfe.infNFe.ide.tpNF.valor = 1  # 1 = Saida
        nfe.infNFe.ide.idDest.valor = 1  # 1 = Venda no estado
        nfe.infNFe.ide.cMunFG.valor = brw_config.supcfc_cidade_ibge
        nfe.infNFe.ide.tpImp.valor = '4'  # 4 = DANFE NFC-e
        nfe.infNFe.ide.verProc.valor = nfe.arquivo_esquema.split('.')[0]
        nfe.infNFe.ide.tpEmis.valor = 9  # 1 = Emissao normal, 5 e 9 = contingencias
        nfe.infNFe.ide.dhCont.valor = data
        nfe.infNFe.ide.xJust.valor = 'Sem conexao com a SEFAZ'
        nfe.infNFe.ide.tpAmb.valor = brw_config.supcfc_ambiente == 'P' and 1 or 2
        nfe.infNFe.ide.indFinal.valor = u'1'  # 1 = Consumidor final
        nfe.infNFe.ide.finNFe.valor = 1  # 1 = NF-e normal

    def _fisnfc_set_nfe_infnfe_emit(self, cr, uid, brw_capa, brw_config, nfe):
        # //NFe.infNFe.emit - Dados do emitente
        msg = []
        if not brw_config.supcfc_cnpj:
            msg.append('CNPJ')
        if not brw_config.supcfc_cliente:
            msg.append('Nome')

        if msg:
            msg = 'Dados da empresa incompletos:\n%s' % ',\n'.join(msg)
            print msg
            raise Exception(msg)

        cnpj = brw_config.supcfc_cnpj or u''
        nfe.infNFe.emit.CNPJ.valor = re.sub('[^0-9]', u'', cnpj[-14:])
        nfe.infNFe.emit.xNome.valor = self.suputi_remover_acentuacao(
            (brw_config.supcfc_cliente or u'').replace(u'\u2013', '-'))
        nfe.infNFe.emit.IE.valor = re.sub('[^0-9A-Z]', u'',
                                          brw_config.supcfc_inscricao_estadual or u'')
        nfe.infNFe.emit.xFant.valor = self.suputi_remover_acentuacao(
            (brw_config.supcfc_fantasia or u''))
        if brw_config.supcfc_inscricao_municipal:
            nfe.infNFe.emit.IM.valor = brw_config.supcfc_inscricao_municipal or u''

        nfe.infNFe.emit.enderEmit.fone.valor = \
            re.sub('[^0-9]', u'', str(brw_config.supcfc_telefone) or u'')
        nfe.infNFe.emit.CRT.valor = brw_config.supcfc_regime_tributario == 'S' and 1 or 3
        self._fisnfc_set_nfe_infnfe_emit_enderemit(cr, uid, brw_config, nfe)

    def _fisnfc_set_nfe_infnfe_emit_enderemit(self, cr, uid, brw_config, nfe):
        # //NFe.infNFe.emit.enderEmit - Endereco do emitente
        endereco = self.suputi_remover_acentuacao(brw_config.supcfc_endereco or '')
        complemento = self.suputi_remover_acentuacao(brw_config.supcfc_complemento or '')
        bairro = self.suputi_remover_acentuacao(brw_config.supcfc_bairro or u'Bairro')
        cidade = self.suputi_remover_acentuacao(brw_config.supcfc_cidade or '')
        nfe.infNFe.emit.enderEmit.xLgr.valor = endereco
        nfe.infNFe.emit.enderEmit.nro.valor = u'%s' % brw_config.supcfc_numero
        nfe.infNFe.emit.enderEmit.xCpl.valor = complemento
        nfe.infNFe.emit.enderEmit.xBairro.valor = bairro
        nfe.infNFe.emit.enderEmit.xMun.valor = cidade
        nfe.infNFe.emit.enderEmit.cMun.valor = brw_config.supcfc_cidade_ibge or 0
        nfe.infNFe.emit.enderEmit.UF.valor = brw_config.supcfc_estado or u''
        nfe.infNFe.emit.enderEmit.cPais.valor = int('01058'[-4:]) or 0
        nfe.infNFe.emit.enderEmit.xPais.valor = u'Brasil'
        nfe.infNFe.emit.enderEmit.CEP.valor = re.sub('[^0-9]', u'',
                                                     brw_config.supcfc_cep or u'')

    def _fisnfc_set_nfe_infnfe_dest(self, cr, uid, brw_capa, nfe):
        # //NFe.infNFe.dest - Dados do destinatário
        brw_cliente = False
        if brw_capa.supcap_cliente:
            cr.execute("""SELECT * 
                          FROM supervisor_cad_clientes 
                          WHERE supcli_cnpj_cpf = '%s'""" % brw_capa.supcap_cliente)
            dados = dictfetchall(cr)
            if dados:
                brw_cliente = get_browse(dados[0])

        if nfe.infNFe.ide.tpAmb.valor == 2:
            nfe.infNFe.dest.xNome.valor = u'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR ' \
                                          u'FISCAL'
        else:
            # Se nao houver destinatario (NFC-e) assume 'CONSUMIDOR NAO IDENTIFICADO'
            nome_consumidor = brw_cliente and brw_cliente.name or 'CONSUMIDOR NAO IDENTIFICADO'
            nfe.infNFe.dest.xNome.valor = nome_consumidor

        nfe.infNFe.dest.email.valor = brw_cliente and brw_cliente.supcli_email or u''
        nfe.infNFe.dest.indIEDest.valor = u'9'  # 9 = Nao contribuinte
        self._fisnfc_set_nfe_infnfe_dest_enderdest(brw_cliente, nfe)
        cnpj_cpf = brw_cliente and brw_cliente.supcli_cnpj_cpf
        if brw_cliente and brw_cliente.supcli_tipo_pessoa == u'J':
            nfe.infNFe.dest.CNPJ.valor = re.sub('[^0-9]', u'', cnpj_cpf[-14:] or u'')

        elif cnpj_cpf:
            nfe.infNFe.dest.CPF.valor = re.sub('[^0-9]', u'', cnpj_cpf[-11:] or u'') or u''

        elif not cnpj_cpf: # limpa o cliente se o mesmo for apagado do xml
            if nfe.infNFe.dest.CNPJ.valor:
                nfe.infNFe.dest.CNPJ.valor = ''
            if nfe.infNFe.dest.CPF.valor:
                nfe.infNFe.dest.CPF.valor = ''

    def _fisnfc_set_nfe_infnfe_dest_enderdest(self, brw_cliente, nfe):
        # //NFe.infNFe.dest.enderDest - Endereco do destinatario
        if not brw_cliente or not brw_cliente.supcli_endereco \
                or not brw_cliente.supcli_endereco_numero \
                or not brw_cliente.supcli_cidade \
                or not brw_cliente.supcli_cod_municipio or not int(brw_cliente.supcli_cod_municipio) \
                or not brw_cliente.supcli_estado:
            return False

        nfe.infNFe.dest.enderDest.xLgr.valor = brw_cliente and brw_cliente.supcli_endereco or u''
        nfe.infNFe.dest.enderDest.nro.valor = brw_cliente and brw_cliente.supcli_endereco_numero or u''
        nfe.infNFe.dest.enderDest.xBairro.valor = brw_cliente and brw_cliente.supcli_bairro or u'Bairro'
        nfe.infNFe.dest.enderDest.cMun.valor = brw_cliente and brw_cliente.supcli_cod_municipio or u''
        nfe.infNFe.dest.enderDest.xMun.valor = brw_cliente and brw_cliente.supcli_cidade or u''
        nfe.infNFe.dest.enderDest.UF.valor = brw_cliente and brw_cliente.supcli_estado or u''
        nfe.infNFe.dest.enderDest.xCpl.valor = u''
        nfe.infNFe.dest.enderDest.CEP.valor = re.sub('[^0-9]', u'',
                                                     brw_cliente and brw_cliente.supcli_cep
                                                     or u'')
        telefone = brw_cliente and brw_cliente.supcli_telefone or u''
        nfe.infNFe.dest.enderDest.fone.valor = re.sub('[^0-9]', u'', telefone)
        nfe.infNFe.dest.enderDest.cPais.valor = int('01058'[-4:]) or 0
        nfe.infNFe.dest.enderDest.xPais.valor = u'Brasil'
        return True

    def _fisnfc_set_nfe_infnfe_autxml(self, brw_config, nfe):
        nfe.infNFe.autXML = []
        # //NFe.infNFe.autXML - Pessoas autorizadas para o download do XML da NF-e
        if nfe.infNFe.emit.enderEmit.UF.valor == 'BA':  # Contemplado apenas pela Bahia
            if (nfe.infNFe.dest.CPF.valor or nfe.infNFe.dest.CNPJ.valor) != u'13937073000156':
                autxml = AutXML()
                autxml.CNPJ.valor = '13937073000156'  # CNPJ da SEFAZ Bahia
                nfe.infNFe.autXML.append(autxml)

        # CNPJ da Sig para consultar notas dos clientes.
        if nfe.infNFe.ide.tpAmb.valor == 1: # apenas em producao
            autxml = AutXML()
            autxml.CNPJ.valor = '01526558000178'
            nfe.infNFe.autXML.append(autxml)

        cnpj = brw_config.supcfc_contador_cnpj or ''
        cpf = brw_config.supcfc_contador_cpf or ''
        if cnpj and cnpj != nfe.infNFe.dest.CNPJ.valor:
            autxml = AutXML()
            autxml.CNPJ.valor = cnpj
            nfe.infNFe.autXML.append(autxml)

        elif cpf and cpf != nfe.infNFe.dest.CPF.valor:
            autxml = AutXML()
            autxml.CPF.valor = cpf
            nfe.infNFe.autXML.append(autxml)

    def _fisnfc_set_nfe_infnfe_pag(self, cr, brw_capa, nfe):
        # //NFe.infNFe.pag - Dados de pagamento
        # {'01': 'Dinheiro',
        #  '02': 'Cheque',
        #  '03': 'Cartão de Crédito',
        #  '04': 'Cartão de Débito',
        #  '05': 'Crédito Loja',
        #  '10': 'Vale Alimentação',
        #  '11': 'Vale Refeição',
        #  '12': 'Vale Presente',
        #  '13': 'Vale Combustível',
        #  '99': 'Outros'}
        cr.execute("""
                    SELECT ccf.supfin_avista_aprazo     AS avista,
                           ccf.supfin_cartao_tipo       AS tipo_cartao,
                           ccf.supfin_tipo_finalizadora AS tipo_fin,
                           ccf.supfin_usa_tef           AS usa_tef,
                           cmf.supfec_valor             AS valor,
                           cmf.supfec_troco             AS troco,
                           cmf.supfec_cnpj_cartao       AS cnpj_fec,
                           cmf.supfec_aut_cartao        AS aut_cartao,
                           coc.supopc_cnpj              AS cnpj_op
                    FROM supervisor_mov_fechamentos            AS cmf
                    INNER JOIN supervisor_cad_finalizadora     AS ccf ON ccf.supfin_codigo = cmf.supfec_finalizadora
                    LEFT  JOIN supervisor_cad_operadora_cartao AS coc ON coc.id = ccf.supfin_operadora_id
                    WHERE cmf.supfec_capa_id = %s """ % brw_capa.id)
        dados_pag = dictfetchall(cr)
        troco = 0
        nfe.infNFe.pag.detPag = []
        for condicao in dados_pag:
            troco += (condicao.get('troco') or 0)
            detpag = DetPag()

            avista = 0
            if condicao.get('avista') == 'P':  # a prazo
                avista = 1

            detpag.indPag.valor = str(avista)  # 0=A vista ,1=A Prazo ,2=Outros
            tipo_pagamento = u'01'  # Dinheiro
            if condicao.get('tipo_fin') == 'H':
                tipo_pagamento = u'02'  # cheque
            elif condicao.get('tipo_fin') in ('C', 'T'):  # cartao e ticket
                if condicao.get('tipo_cartao') == 'C':
                    tipo_pagamento = u'03'  # credito
                else:
                    tipo_pagamento = u'04'  # debto

            elif condicao.get('tipo_fin') == 'O':
                tipo_pagamento = u'99'  # outros

            detpag.tPag.valor = tipo_pagamento
            detpag.vPag.valor = '%.2f' % condicao.get('valor')
            if detpag.tPag.valor in (u'03', u'04'):  # Cartao credito/debito
                cnpj = condicao.get('cnpj_fec') or condicao.get('cnpj_op') or u''
                if cnpj:
                    cnpj = cnpj.rjust(14, '0')

                detpag.card.tpIntegra.valor = condicao.get('usa_tef') and 1 or 2  # 1=TEF, 2=POS
                detpag.card.CNPJ.valor = cnpj
                detpag.card.cAut.valor = condicao.get('aut_cartao') or u'123456'

            nfe.infNFe.pag.detPag.append(detpag)

        if troco:
            nfe.infNFe.pag.vTroco.valor = Decimal(troco).quantize(TWOPLACES)

    def _fisnfc_set_nfe_infnfe_transp(self, nfe):
        # //NFe.infNfE.transp - Dados da transportadora
        load = '_classic_write'

        # O DANFE da versao 4.0 ainda nao contempla todas as opcoes de frete
        # frete = {'0': u'0', '1': u'1', '2': u'2', '3': u'0', '4': u'1'}.get(dados_nf.get('frete'))
        # todo: quando tiver frete, tratar o frete
        nfe.infNFe.transp.modFrete.valor = u'9'
        return True

    def _fisnfc_set_nfe_infnfe_prod(self, cr, uid, brw_config, brw_capa, nfe, context):
        # //NFe.infNFe.prod - Itens da nota fiscal
        nfe.infNFe.det = []
        tot_ncm_vr_tributo_fed = tot_ncm_vr_tributo_est = tot_ncm_vr_tributo_mun = 0
        ncm_chave = u''
        self.total_aproveitamento_credito = 0
        cr.execute("""
                        SELECT cmc.*, 
                               supcor_codigo_retaguarda     AS codigo_interno,
                               supcor_codigo_anp            AS codigo_anp,
                               supcor_pglp                  AS pglp,
                               supcor_pgnn                  AS pgnn,
                               supcor_pgni                  AS pgni,
                               supcor_vpart                  AS vpart,
                               supcor_producao              AS producao,
                               supcor_unidade               AS unidade,
                               supcor_origem_mercadoria     AS origem_mercadoria,
                               supcor_aliquota_ibpt_fed     AS supcor_aliquota_ibpt_fed,
                               supcor_aliquota_ibpt_est     AS supcor_aliquota_ibpt_est,
                               supcor_aliquota_ibpt_mun     AS supcor_aliquota_ibpt_mun,
                               supcor_getin                 AS supcor_getin,
                               supcor_codigo_trib_icms      AS codigo_icms,
                               supcor_codigo_cst_icms       AS codigo_cst,
                               supcor_tipo_tributacao_icms  AS tipo_trib,
                               supcor_codigo_csosn_icms     AS csosn,
                               supcor_base_icms             AS base_icms,
                               supcor_aliquota_icms         AS aliquota_icms,
                               supcor_codigo_beneficio_icms AS cod_beneficio,
                               supcor_motivo_beneficio_icms AS mot_beneficio,
                               supcor_aliquota_fcp_icms     AS aliquota_fcp,
                               supcor_codigo_trib_pis       AS codigo_pis,
                               supcor_aliquota_pis          AS aliquota_pis,
                               supcor_codigo_cst_pis        AS codigo_cst_pis,
                               supcor_base_pis              AS base_pis,
                               supcor_codigo_trib_cofins    AS codigo_cofins,
                               supcor_aliquota_cofins       AS aliquota_cofins,
                               supcor_codigo_cst_cofins     AS codigo_cst_cofins,
                               supcor_base_cofins           AS base_cofins

                        FROM supervisor_mov_corpos                AS cmc
                        WHERE cmc.supcor_capa_id = {capa_id}
                          AND cmc.supcor_venda_cancelamento = 'V'
                        ORDER BY cmc.supcor_sequencia""".format(capa_id=brw_capa.id))
        dados_itens = dictfetchall(cr)
        numero_item = 1
        msg_trib = []
        for item in dados_itens:
            if item.get('supcor_item') and item.get('supcor_getin'):
                codigo_barra = item.get('supcor_item')
                # Na nota 4.0 o primeiro digito do codigo de barras nao pode ser 0 quando tiver
                # 14 digitos, o primeiro digito deve ser de 1 a 9 que indicara a quantidade na
                # embalagem
                if len(codigo_barra) == 14:
                    if codigo_barra[0] == '0':
                        codigo_barra = codigo_barra[1:]

                p_cean = codigo_barra
                if int(p_cean) < 10000000:
                    p_cean = u'SEM GTIN'

            else:
                p_cean = u'SEM GTIN'

            ncm_vr_tributo_fed = 0
            if item.get('supcor_aliquota_ibpt_fed') > 0:
                ncm_vr_tributo_fed = Decimal(item.get('supcor_quantidade')) * Decimal(item.get(
                    'supcor_preco_unitario'))
                ncm_vr_tributo_fed *= Decimal(item.get('supcor_aliquota_ibpt_fed') or 0)
                ncm_vr_tributo_fed /= Decimal(100)

            ncm_vr_tributo_est = Decimal(item.get('supcor_quantidade')) * Decimal(item.get(
                'supcor_preco_unitario'))
            ncm_vr_tributo_est *= Decimal(item.get('supcor_aliquota_ibpt_est') or 0)
            ncm_vr_tributo_est /= Decimal(100)
            ncm_vr_tributo_mun = Decimal(item.get('supcor_quantidade')) * Decimal(item.get(
                'supcor_preco_unitario'))
            ncm_vr_tributo_mun *= Decimal(item.get('supcor_aliquota_ibpt_mun') or 0)
            ncm_vr_tributo_mun /= Decimal(100)
            if not ncm_chave:
                ncm_chave = item.get('fisnc_ibpt_chave') or u''

            tot_ncm_vr_tributo_fed += Decimal(ncm_vr_tributo_fed)
            tot_ncm_vr_tributo_est += Decimal(ncm_vr_tributo_est)
            tot_ncm_vr_tributo_mun += Decimal(ncm_vr_tributo_mun)
            p_ucom = item.get('unidade')
            p_qcom = '%.3f' % item.get('supcor_quantidade')
            p_vuncom = '%f' % item.get('supcor_preco_unitario')
            p_vuncom = p_vuncom.rstrip('0')  # Desconsidera zeros aa direita
            _valor = item.get('supcor_preco_total')
            p_vprod = '%.2f' % _valor
            p_ceantrib = p_cean
            det = Det()
            det.nItem.valor = numero_item
            numero_item += 1
            det.prod.cProd.valor = item.get('codigo_interno') or u''
            det.prod.cEAN.valor = p_cean or u''

            # Se NFC-e em ambiente de homologacao, a descricao do primeiro item deve ser
            # obrigatoriamente NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL
            if item == dados_itens[0] and nfe.infNFe.ide.tpAmb.valor == 2:  # Homologacao
                det.prod.xProd.valor = u'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM ' \
                                       u'VALOR FISCAL'
            else:
                nome_item = (item.get('supcor_nome_item') or u'').replace('&', '&amp;')
                det.prod.xProd.valor = nome_item

            # Tratamento para combustiveis. Verifica se o produto tem CFOP para combustiveis e
            # faz a insercao das tags
            det.prod.NCM.valor = re.sub('[^0-9]', u'', item.get('supcor_ncm') or '') or u''
            # todo: verificar nFCI
            # det.prod.nFCI.valor = produto.get('estpro_fci') or u''

            # CFOPS validos 5101; 5102; 5103; 5104; 5115; 5401; 5403; 5405; 5653; 5656; 5667 e 5933
            det.prod.CFOP.valor = item.get('supcor_codigo_cfop')
            if not item.get('tipo_trib'):
                msg_trib.append('Sequencia: %s - %s ICMS: %s' % (item.get('supcor_sequencia'),
                                                                 item.get('supcor_item'),
                                                                 item.get('codigo_icms'))
                                )
                continue

            det.prod.uCom.valor = p_ucom or u''
            det.prod.qCom.valor = p_qcom or 0
            det.prod.vUnCom.valor = p_vuncom or 0
            if item.get('supcor_codigo_cfop') == '5656':
                det.prod.comb.cProdANP.valor = item.get('codigo_anp') or 0
                det.prod.comb.descANP.valor = (item.get('supcor_nome_item') or u'').replace('&', '&amp;')
                det.prod.comb.UFCons.valor = brw_config.supcfc_estado
                if item.get('codigo_anp') == 210203001:
                    # apenas para codigo anp 210203001, a unidade tributavel deve ser 'kg',
                    # se nao ocorre a rejeicao '854'
                    p_ucom = 'kg'
                    det.prod.comb.pGLP.valor = Decimal(item.get('pglp') or 0)
                    det.prod.comb.pGNn.valor = Decimal(item.get('pgnn') or 0)
                    det.prod.comb.pGNi.valor = Decimal(item.get('pgni') or 0)
                    det.prod.comb.vPart.valor = Decimal(item.get('vpart') or 0)

            det.prod.vDesc.valor = '%.2f' % (item.get('supcor_desconto_item') or 0)
            # det.prod.vFrete.valor = '%.2f' % item.get('vr_frete_rateado') or 0
            # det.prod.vSeg.valor = '%.2f' % item.get('vr_seguro_rateado') or 0

            det.prod.vOutro.valor = '%.2f' % item.get('supcor_acrescimo_item') or 0
            det.prod.vProd.valor = p_vprod or 0
            det.prod.cEANTrib.valor = p_ceantrib or u''
            det.prod.uTrib.valor = p_ucom or u''
            det.prod.qTrib.valor = p_qcom or 0
            det.prod.vUnTrib.valor = p_vuncom or 0
            det.prod.indTot.valor = 1  # Valor do item entra no valor total da nota, 1-sim, 0-nao

            # //NFe.infNFe.imposto - Encargos e tributacoes
            det.imposto.vTotTrib.valor = '%.2f' % (item.get('supcor_vr_imposto_fed', 0) +
                                                   item.get('supcor_vr_imposto_est', 0) +
                                                   item.get('supcor_vr_imposto_mun', 0))

            if nfe.infNFe.emit.CRT.valor == 1:
                self.set_regime_tributario_simples_nacional(cr, uid, brw_config, det, item)
            else:
                self.set_regime_tributario_outros(cr, uid, det, item, brw_capa)

            nfe.infNFe.det.append(det)

        if msg_trib:
            raise Exception('Tributacao de ICMS nao encontrada ou sem Tipo!\n '
                            'Nota: %s - %s: \n%s' % (brw_capa.supcap_nro_cupom,
                                                     brw_capa.supcap_caixa, '\n'.join(msg_trib)))

        # Aproveitamento de ICMS para Simples Nacional
        if brw_config.supcfc_valor_faturamento > 0:
            if nfe.infNFe.emit.CRT.valor == 1:
                nfe.infNFe.infAdic.infCpl.valor += \
                    (u'\n Documento emitido por ME ou EPP optante pelo Simples Nacional. - NAO '
                     u'GERA DIREITO DE CREDITO FISCAL DE IPI. - Permite aproveitamento '
                     u'de credito ICMS no valor de R$ %.2f correspondente a aliq. %.2f%% nos '
                     u'termos ART.23 LC123/06') % (
                        self.total_aproveitamento_credito or 0,
                        self._fisnfc_get_aliquota_sn(cr, uid, brw_config))

        total_nf = Decimal(brw_capa.supcap_total_cupom)
        valor_percent_aprox_fed = tot_ncm_vr_tributo_fed / total_nf * Decimal(100)
        valor_percent_aprox_est = tot_ncm_vr_tributo_est / total_nf * Decimal(100)
        valor_percent_aprox_mun = tot_ncm_vr_tributo_mun / total_nf * Decimal(100)
        nfe.infNFe.infAdic.infCpl.valor += \
            (u' \n Trib.Aprox.: R$ %.2f (%.2f%%) federal R$ %.2f (%.2f%%) estadual R$ %.2f '
             u'(%.2f%%) municipal. Fonte IBPT %s') % (
                tot_ncm_vr_tributo_fed, valor_percent_aprox_fed, tot_ncm_vr_tributo_est,
                valor_percent_aprox_est, tot_ncm_vr_tributo_mun, valor_percent_aprox_mun,
                ncm_chave)

        # todo: verificar como fazer esta parte
        str_resumo = u''
        if str_resumo:
            nfe.infNFe.infAdic.infCpl.valor += u'\n Resumo: ' + str_resumo

        total_icms_uf_dest = sum(map(lambda m: float(m.imposto.ICMSUFDest.vICMSUFDest.valor),
                                     nfe.infNFe.det))
        total_fcp = sum(map(lambda m: float(m.imposto.ICMS.vFCP.valor), nfe.infNFe.det))
        if total_icms_uf_dest:
            nfe.infNFe.infAdic.infCpl.valor += (
                                                   u' \n Parcela do diferencial de aliquota devido '
                                                   u'ao estado destino: R$%.2f\n Valor do fundo de '
                                                   u'combate a pobreza: R$%.2f') % (
                                                   total_icms_uf_dest, total_fcp)

    def _fisnfc_set_nfe_infnfe_total_icmstot(self, brw_capa, nfe, context=None):
        # NFe.infNFe.total.ICMSTot - Total do grupo ICMS - Produtos
        context = context and context.copy() or {}
        nfe.infNFe.total.ICMSTot.vBC.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vBC.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMS.valor = '%.2f' % (sum(map(
            lambda m: float(m.imposto.ICMS.vICMS.valor), nfe.infNFe.det)) or 0)
        nfe.infNFe.total.ICMSTot.vICMSDeson.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSDeson.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFCP.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vFCP.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vBCST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vBCST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFCPST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vFCPST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vProd.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vProd.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFrete.valor = '%.2f' % 0
        nfe.infNFe.total.ICMSTot.vSeg.valor = '%.2f' % 0
        nfe.infNFe.total.ICMSTot.vDesc.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vDesc.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vII.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.II.vII.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vIPI.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.IPI.vIPI.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vPIS.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.PIS.vPIS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vCOFINS.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.COFINS.vCOFINS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vOutro.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vOutro.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vNF.valor = '%.2f' % brw_capa.supcap_total_cupom
        nfe.infNFe.total.ICMSTot.vTotTrib.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.vTotTrib.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSUFDest.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMSUFDest.vICMSUFDest.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSUFRemet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMSUFDest.vICMSUFRemet.valor), nfe.infNFe.det))

    def set_beneficio_fiscal(self, det, item, valor_desonerado):
        # CSTs: 20, 30, 40, 41, 50, 51, 70, 90
        pass
        # vr_liquido = Decimal(item.get('supcor_preco_total') - item.get('supcor_desconto_item') +
        #                      item.get('supcor_acrescimo_item'))
        # base_icms = Decimal(vr_liquido * Decimal(item.get('base_icms')) / Decimal(100.0)).quantize(TWOPLACES)
        # vr_icms = Decimal(base_icms * Decimal(18) / Decimal(100.0)).quantize(
        #     TWOPLACES)
        # valor_desonerado = vr_icms
        # det.imposto.ICMS.vICMSDeson.obrigatorio = True
        # det.imposto.ICMS.motDesICMS.obrigatorio = True
        # det.imposto.ICMS.vICMSDeson.valor = '%.2f' % valor_desonerado
        # det.imposto.ICMS.motDesICMS.valor = int(item.get('mot_beneficio') or 0)

    def set_regime_tributario_outros_icms(self, cr, uid, det, item, brw_capa):
        codigo_cst = item.get('codigo_cst')
        det.imposto.ICMS.CST.valor = codigo_cst  # deve ser diferente de 50, 51
        det.imposto.ICMS.orig.valor = int(item.get('origem_mercadoria') or '0')
        vr_encargo = Decimal(item.get('supcor_vr_icms') or 0)
        vr_base_icms = Decimal(item.get('supcor_base_icms'))
        det.imposto.ICMS.pICMS.valor = '%.2f' % item.get('aliquota_icms') or 0
        det.imposto.ICMS.vICMS.valor = '%.2f' % vr_encargo
        brw_cliente = False
        if brw_capa.supcap_cliente:
            cr.execute("""SELECT * 
                          FROM supervisor_cad_clientes 
                          WHERE supcli_cnpj_cpf = '%s'""" % brw_capa.supcap_cliente)
            dados = dictfetchall(cr)
            if dados:
                brw_cliente = get_browse(dados[0])
        # Em operacoes interestaduais (idDest=2) para consumidor final (indFinal=1) nao
        # contribuinte (indIEDest=9), o valor do FCP deve ser informado no campo vFCPUFDest

        det.imposto.ICMS.pFCP.valor = '%.4f' % 0
        det.imposto.ICMS.vFCP.valor = '%.2f' % 0
        if codigo_cst == '00':  # Tributado integralmente
            det.imposto.ICMS.modBC.valor = u'3'
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms

        elif codigo_cst == '20':  # Com reducao de base de calculo
            det.imposto.ICMS.modBC.valor = u'3'
            det.imposto.ICMS.pRedBC.valor = '%.4f' % (Decimal(100) - Decimal(item.get('base_icms')))
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '30':
            aliq_fcp = Decimal(item.get('aliquota_fcp'))
            vr_fcp = 0
            if brw_cliente and brw_cliente.supcli_tipo_pessoa == u'F':
                vr_fcp = vr_base_icms * aliq_fcp / Decimal(100)

            det.imposto.ICMS.pICMS.valor = '%.2f' % 0
            det.imposto.ICMS.vICMS.valor = '%.2f' % 0
            det.imposto.ICMSUFDest.pFCPUFDest.valor = '%.4f' % 0
            det.imposto.ICMSUFDest.vFCPUFDest.valor = '%.2f' % 0
            det.imposto.ICMS.modBCST.valor = '4'
            det.imposto.ICMS.vBCST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pICMSST.valor = '%.2f' % (item.get('aliquota_icms') or 0)
            det.imposto.ICMS.vICMSST.valor = '%.2f' % vr_encargo
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pFCPST.valor = '%.4f' % aliq_fcp
            det.imposto.ICMS.vFCPST.valor = '%.2f' % vr_fcp
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst in ('40','41','50'):  # 40=Isento, 41=Nao tributado, 50=Com suspensao
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '51': # Com difereimento
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '60':  # Cobrado anteriormente com substituicao tributaria
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
            det.imposto.ICMS.vICMSSTRet.valor = '%.2f' % vr_encargo

        elif codigo_cst == '70': # Com redução da BC e cobrança do ICMS por substituição tributária
            aliq_fcp = Decimal(item.get('aliquota_fcp'))
            vr_fcp = 0
            if brw_cliente and brw_cliente.supcli_tipo_pessoa == u'F':
                vr_fcp = vr_base_icms * aliq_fcp / Decimal(100)

            det.imposto.ICMS.modBC.valor = '3'
            det.imposto.ICMS.pRedBC.valor = '%.4f' % (Decimal(100) - Decimal(item.get('base_icms')))
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pFCPST.valor = '%.4f' % aliq_fcp
            det.imposto.ICMS.vFCPST.valor = '%.2f' % vr_fcp
            det.imposto.ICMS.modBCST.valor = '4'
            det.imposto.ICMS.vBCST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vICMSST.valor = '%.2f' % vr_encargo
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '90':  # Outras tributacoes)
            det.imposto.ICMS.modBC.valor = '3'
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            det.prod.CEST.valor = item.get('supcor_cest') or ''

        # Isentos
        if item.get('tipo_trib').strip() == u'I':
            det.imposto.ICMS.vBC.valor = '%.2f' % 0

    def set_regime_tributario_outros_pis(self, det, item):
        codigo_cst = item.get('codigo_cst_pis')
        codigo_cst = '99' if codigo_cst == '05' else codigo_cst
        det.imposto.PIS.CST.valor = codigo_cst
        vr_base_pis = Decimal(item.get('supcor_base_pis') or 0)
        aliq_pis = Decimal(item.get('aliquota_pis') or 0)
        vr_pis = Decimal(item.get('supcor_vr_pis') or 0)
        if codigo_cst in ('01', '02'):
            det.imposto.PIS.vBC.valor = '%.2f' % vr_base_pis
            det.imposto.PIS.pPIS.valor = '%.4f' % aliq_pis
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis
        elif codigo_cst == '03':
            det.imposto.PIS.qBCProd.valor = '%.4f' % item.get('supcor_quantidade')
            det.imposto.PIS.vAliqProd.valor = '%.4f' % (vr_base_pis * (aliq_pis / Decimal(100)))
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis

        elif codigo_cst in (
        '49', '50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63',
        '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99'):
            det.imposto.PIS.vBC.valor = '%.2f' % vr_base_pis
            det.imposto.PIS.pPIS.valor = '%.4f' % aliq_pis
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis
            det.imposto.PIS.qBCProd.valor = '%.4f' % item.get('supcor_quantidade')
            det.imposto.PIS.vAliqProd.valor = '%.4f' % (vr_base_pis * (aliq_pis / Decimal(100)))
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis

    def set_regime_tributario_outros_cofins(self, det, item):
        codigo_cst = item.get('codigo_cst_cofins')
        codigo_cst = '99' if codigo_cst == '05' else codigo_cst
        det.imposto.COFINS.CST.valor = codigo_cst
        vr_base_cofins = Decimal(item.get('supcor_base_cofins') or 0)
        aliq_cofins = Decimal(item.get('aliquota_cofins') or 0)
        vr_cofins = Decimal(item.get('supcor_vr_cofins') or 0)
        if codigo_cst in ('01', '02'):
            det.imposto.COFINS.vBC.valor = '%.2f' % vr_base_cofins
            det.imposto.COFINS.pCOFINS.valor = '%.4f' % aliq_cofins
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % vr_cofins
        elif codigo_cst == '03':
            det.imposto.COFINS.qBCProd.valor = '%.4f' % item.get('supcor_quantidade')
            det.imposto.COFINS.vAliqProd.valor = '%.4f' % \
                                                 (vr_base_cofins * (aliq_cofins / Decimal(100)))
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % vr_cofins
        elif codigo_cst in ('04', '06', '07', '08', '09'):
            pass
        elif codigo_cst in (
        '49', '50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63',
        '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99'):
            det.imposto.COFINS.vBC.valor = '%.2f' % vr_base_cofins
            det.imposto.COFINS.pCOFINS.valor = '%.4f' % aliq_cofins
            det.imposto.COFINS.qBCProd.valor = '%.2f' % item.get('supcor_quantidade')
            det.imposto.COFINS.vAliqProd.valor = '%.4f' % \
                                                 (vr_base_cofins * (aliq_cofins / Decimal(100)))
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % vr_cofins

    def set_regime_tributario_outros(self, cr, uid, det, item, brw_capa):
        vals  = {}
        vr_liquido = Decimal(item.get('supcor_preco_total')) - Decimal(item.get('supcor_desconto_item')) + \
                             Decimal(item.get('supcor_acrescimo_item'))
        # recalcula os valores das tributacoes caso tenha trocado o cadastro das mesmas.
        if item.get('codigo_icms') != item.get('supcor_codigo_trib_icms'):
            base_icms = Decimal(vr_liquido * Decimal(item.get('base_icms')) / Decimal(100)).quantize(TWOPLACES)
            vr_icms = Decimal(base_icms * Decimal(item.get('aliquota_icms'))  / Decimal(100)).quantize(TWOPLACES)
            vals.update({'supcor_vr_icms': vr_icms,
                         'supcor_base_icms': base_icms})

        if item.get('codigo_pis') != item.get('supcor_codigo_trib_pis'):
            base_pis = Decimal(vr_liquido * Decimal(item.get('base_pis')) / Decimal(100)).quantize(
                TWOPLACES)
            vr_pis = Decimal(base_pis * Decimal(item.get('aliquota_pis')) / Decimal(100)).quantize(
                TWOPLACES)
            vals.update({'supcor_vr_pis': vr_pis,
                         'supcor_base_pis': base_pis})

        if item.get('codigo_cofins') != item.get('supcor_codigo_trib_cofins'):
            base_cofins = Decimal(vr_liquido * Decimal(item.get('base_cofins')) / Decimal(100)).quantize(
                TWOPLACES)
            vr_cofins = Decimal(base_cofins * Decimal(item.get('aliquota_cofins'))/ Decimal(100)).quantize(
                TWOPLACES)
            vals.update({'supcor_vr_cofins': vr_cofins,
                         'supcor_base_cofins': base_cofins})

        if vals:
            values = []
            for valor in vals.items():
                values.append('%s = %s' % valor)

            print ",\n".join(values)
            cr.execute("""UPDATE supervisor_mov_corpos 
                          SET %s
                          WHERE id = %s""" % (',\n'.join(values), item.get('id')))

            item.update(vals)

        self.set_regime_tributario_outros_icms(cr, uid, det, item, brw_capa)
        self.set_regime_tributario_outros_pis(det, item)
        self.set_regime_tributario_outros_cofins(det, item)

    def set_regime_tributario_simples_nacional_icms(self, cr, uid, brw_config, det, item):
        """
        Atribui os valores de ICMS para regime tributario Simples Nacional.
        :param det: item de movimentacao
        :param item: dados do item
        :type det: Det (objeto ´nfe.infNFe.det´)
        :type item: dict
        :return: n/a
        :rtype: n/a
        """
        csosn = item.get('csosn')
        # todo: ver como fica essa parte de teste de csosn
        codigo_cst = item.get('codigo_cst')
        det.imposto.ICMS.orig.valor = item.get('origem_mercadoria')
        det.imposto.ICMS.CSOSN.valor = csosn
        aliquota = self._fisnfc_get_aliquota_sn(cr, uid, brw_config)
        vr_cred_icms_sn = Decimal(item.get('supcor_base_icms')) * aliquota / Decimal(100)
        if csosn == '101':
            det.imposto.ICMS.pCredSN.valor = '%.2f' % aliquota
            det.imposto.ICMS.vCredICMSSN.valor = '%.2f' % vr_cred_icms_sn
        elif csosn == '500':
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % item.get('supcor_base_icms')
        # elif csosn == '900':
        #     det.imposto.ICMS.vCredICMSSN.valor = 0

        if codigo_cst == '60':
            det.prod.CEST.valor = item.get('supcor_cest') or ''

        # Para tipo tributado soma o aproveitamento de credito
        if item.get('suptnf_tipo_tributacao') == u'T':
            # Aproveitamento de credito
            self.total_aproveitamento_credito += \
                (Decimal(item.get('supcor_base_icms')) * aliquota / Decimal(100))

    def set_regime_tributario_simples_nacional(self, cr, uid, brw_config, det, item):
        """
        Atribui valores para o regime Simples Nacional
        :param det: item de movimentacao
        :param item: dados do item
        :type det: Det (objeto ´nfe.infNFe.det´)
        :type item: dict
        :return: n/a
        :rtype: n/a
        """
        det.imposto.ICMS.regime_tributario = 1  # Simples Nacional
        self.set_regime_tributario_simples_nacional_icms(cr, uid, brw_config, det, item)
        det.imposto.PIS.CST.valor = u'99'  # Outras operacoes
        det.imposto.COFINS.CST.valor = u'99'  # Outras operacoes

    def _fisnfc_get_aliquota_sn(self, cr, uid, brw_config):
        """
        Recupera o valor da aliquota do Simples Nacional.
        """
        aliquota_sn = 0
        receita_anual = brw_config.supcfc_valor_faturamento

        cr.execute("""SELECT * 
                      FROM supervisor_tributacao_simples_nacional
                      ORDER BY suptsn_valor_faturamento""")
        for trib in dictfetchall(cr):
            if receita_anual <= trib.get('suptsn_valor_faturamento'):
                aliquota_sn = trib.get('suptsn_aliq_icms') or 0
                break

        return aliquota_sn

    def supcap_verifica_ambiente(self, cr, uid, brw_capa, brw_config, context=None):
        context = context and context.copy() or {}
        if brw_capa.supcap_ambiente == brw_config.supcfc_ambiente:
            return True

        amb_nota = {'H': ('Homologacao'),
                    'P': ('Producao')}.get(brw_capa.supcap_ambiente)
        amb_conf = {'H': ('Homologacao'),
                    'P': ('Producao')}.get(brw_config.supcfc_ambiente)
        msg = ('Ambiente da nota diferente da configuracao:\n'
               'Ambiente Nota: %s\nAmbiente Configuracao: %s' % (amb_nota, amb_conf))
        raise Exception(msg)

    def supcap_verifica_status_nfce_sefaz(self, cr, uid, brw_config, brw_capa, context=None):
        context = context and context.copy() or {}
        self.supcap_verifica_ambiente(cr, uid, brw_capa, brw_config, context)
        chave_acesso = brw_capa.supcap_chave_acesso
        ambiente = brw_config.supcfc_ambiente == 'P' and 1 or 2
        try:
            proc_nfce = nfc_e(brw_config.supcfc_certificado_conteudo,
                              brw_config.supcfc_chave_conteudo,
                              ambiente,
                              brw_config.supcfc_estado)
            result, __ = proc_nfce.consultar_nota(chave_acesso)
        except socket.gaierror:
            raise Exception('Falha ao comunicar com o servidor da SEFAZ. Verifique se '
                            'existe conexao com a Internet.')

        except Exception as erro:
            raise Exception(erro.message) or erro

        except IOError:
            raise Exception('Falha ao comunicar com o servidor da SEFAZ. Tente '
                                               'novamente mais tarde.')
        return result, __

    def supcap_precessar_lote(self, cr, uid, brw_config, brw_capa, context=None):
        context = context and context.copy() or {}
        vals = {}
        ambiente = brw_config.supcfc_ambiente == 'P' and 1 or 2  # 1=Producao, 2=Homologacao
        nfce = self.gerar_nfce(cr, uid, brw_config, brw_capa, ambiente, context)
        processo, __ = self.supcap_verifica_status_nfce_sefaz(cr, uid, brw_config, brw_capa, context)
        if processo.resposta.cStat.valor == u'100':  # Faturada
            protocolo = processo.resposta.protNFe.infProt.nProt.valor
            vals['supcap_protocolo'] = protocolo
            procnfce = ProcNFCe()
            certificado = Certificado()
            certificado.certificado = brw_config.supcfc_certificado_conteudo
            certificado.chave = brw_config.supcfc_chave_conteudo
            certificado.assina_xmlnfe(nfce)
            procnfce.NFe.xml = nfce.xml
            procnfce.protNFe.xml = processo.resposta.protNFe.infProt.xml
        else:
            raise Exception('\nErro ao consultar nota "%s": \n%s - %s' % (brw_capa.supcap_chave_acesso,
                                                                          processo.resposta.cStat.valor,
                                                                          processo.resposta.xMotivo.valor))
        vals['supcap_xml_proc'] = procnfce.xml
        valores = []
        for i, j in vals.items():
            valores.append("%s = '%s'" % (i, j))

        if not valores:
            return ''

        cr.execute("""UPDATE supervisor_mov_capa
                      SET {valores}
                      WHERE id = {id};
                      COMMIT;""".format(valores=',\n'.join(valores),
                                        id=brw_capa.id))
        return vals.get('supcap_xml_proc')


print 'Iforme os seguintes dados'
sup_host = raw_input('Ip do SUPERVISOR: ')
sup_usuario = raw_input('Usuario do postgres SUPERVISOR: ')
sup_nome = raw_input('Nome do Banco SUPERVISOR: ')
sup_senha = raw_input('Senha do Banco SUPERVISOR: ')

cont = 0
msg = []
try:
    print '>>>>>> Alterando nota......\n'
    conn = psycopg2.connect(database=sup_nome, user=sup_usuario, host=sup_host,
                            password=sup_senha, connect_timeout=20)
    _cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    try:
        dict_sup = {}
        _cr.execute("""SELECT *
                       FROM supervisor_mov_capa
                       WHERE supcap_xml_proc IS NULL
                         AND supcap_xml_gerado IS NOT NULL
                         AND supcap_state = '3'
                         AND supcap_tipo = 'V'
                         """)
        dados_nf = dictfetchall(_cr)
        if not dados_nf:
            raise Exception("Não encontrada nenhuma Nota sem XML")

        _cr.execute("SELECT * FROM supervisor_cfg_configuracao")
        brw_config = get_browse(dictfetchall(_cr)[0])
        for i in dados_nf:
            brw_capa = get_browse(i)
            xml = openerp().supcap_precessar_lote(_cr, 1, brw_config, brw_capa, {})
            if not xml:
                msg.append('Nota: %s Série: %s' % (brw_capa.supcap_nro_cupom,
                                                   brw_capa.supcap_caixa))
                continue

            cont += 1

        print 'Finalizado, %s notas atualizadas' % cont
        if msg:
            print 'Notas que não conseguiu gerar xml:\n%s' % '\n'.join(msg)

    except Exception as e:
        print 'Erro ao atualizar notas: %s' % e

except Exception as e:
    print 'Erro ao conectar com a Base de dados:%s' % e
