# -*- coding: utf-8 -*-

# Este script recupera o protocolo de cancelamento das NFC-e's no ERP


from nfe.nfc_e import nfce as nfc_e
import psycopg2.extras


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class get_browse(object):
    def __init__(self, d):
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [get_browse(x) if isinstance(x, dict) else x for x in b])
            else:
                setattr(self, a, get_browse(b) if isinstance(b, dict) else b)

def _calcula_dv(valor):
    soma = 0
    m = 2
    for i in range(len(valor) - 1, -1, -1):
        c = valor[i]
        soma += int(c) * m
        m += 1
        if m > 9:
            m = 2

    digito = 11 - (soma % 11)
    if digito > 9:
        digito = 0

    return digito

print 'Iforme os seguintes dados'
erp_host = raw_input('Ip do ERP: ')
erp_usuario = raw_input('Usuario do postgres ERP: ')
erp_nome = raw_input('Nome do Banco ERP: ')
erp_senha = raw_input('Senha do Banco ERP: ')

data_ini = raw_input('Data inicial(DD-MM-AAAA): ').split('-')
data_fim = raw_input('Data final(DD-MM-AAAA):').split('-')

data_ini.reverse()
data_fim.reverse()

cont = 0
try:
    print '>>>>>> Analizando notas......\n'
    conn = psycopg2.connect(database=erp_nome, user=erp_usuario, host=erp_host,
                            password=erp_senha, connect_timeout=20)
    cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    try:
        data_ini = '-'.join(data_ini)
        data_fim = '-'.join(data_fim)
        cr.execute("""SELECT fp.company_id,
                             fp.certificado_conteudo, 
                             fp.chave_conteudo, 
                             fp.fispar_nfce_tentativa_envio, 
                             fp.fispar_nfce_conexao_timeout,
                             ap.uf         
                      FROM fiscal_parametro AS fp
                      LEFT JOIN res_company AS rc ON rc.id = fp.company_id
                      LEFT JOIN adm_pessoa  AS ap ON ap.id = rc.rescon_pessoa_id
                   """)
        parametros = {}
        for i in dictfetchall(cr):
            par = get_browse(i)
            parametros[par.company_id] = par

        cr.execute("""SELECT fnc.id,
                             fnc.nfe_chave_acesso,
                             fnc.numero_nf,
                             fnc.serie_nf,
                             fnc.state,
                             modelo.ambiente,
                             fnc.company_id,
                             fnc.nfe_xml_cancelamento,
                             fnc.nfe_protocolo_cancelamento
                      FROM fiscal_nf_capa AS fnc
                      INNER JOIN fiscal_modelo_nf_series AS modelo ON modelo.id = fnc.modelo_nf_series_id
                      WHERE fnc.data_emissao >= '%s 00:00:00'
                        AND fnc.data_emissao <= '%s 23:59:59'
                        AND fnc.nfe_chave_acesso IS NOT NULL
                        AND fnc.fisnfc_consumidor = True
                        AND fnc.state = 'C'
                        AND fnc.nfe_protocolo_cancelamento is null
                    """ % (data_ini, data_fim))
        for i in dictfetchall(cr):
            brw_nf = get_browse(i)
            print 'Verificando nota %s - %s' % (brw_nf.numero_nf, brw_nf.serie_nf)
            protocolo_canc = ''
            if brw_nf.nfe_xml_cancelamento:
                protocolo_canc = brw_nf.nfe_xml_cancelamento.split('<nProt>')[-1].split('</nProt>')[0]

            if protocolo_canc:
                cr.execute("""UPDATE fiscal_nf_capa SET nfe_protocolo_cancelamento = '%s' WHERE id = %s;
                              COMMIT;""" % (protocolo_canc, brw_nf.id))
                print 'Nota %s - %s com novo protocolo de cancelamento --> %s' % (brw_nf.numero_nf, brw_nf.serie_nf,
                                                                                  protocolo_canc)
                continue

            parametro = parametros.get(brw_nf.company_id)
            if not parametro.uf:
                raise Exception('Erro: a Empresa %s nao possui UF cadastrado no endereco da pessoa!' %
                                parametro.company_id)
            nf = nfc_e(parametro.certificado_conteudo,
                       parametro.chave_conteudo,
                       brw_nf.ambiente,
                       parametro.uf,
                       False)
            nf.process.tentativas = parametro.fispar_nfce_tentativa_envio or 5
            nf.process.timeout = parametro.fispar_nfce_conexao_timeout or 10
            processo, __ = nf.consultar_nota(brw_nf.nfe_chave_acesso)
            'Status na SEFAZ: %s - %s' % (processo.resposta.cStat.valor, processo.resposta.xMotivo.valor)
            if processo.resposta.cStat.valor in (u'100', u'150'):
                if brw_nf.state != '3':  # Faturada
                    print 'Nota %s - %s Faturada na SEFAZ e com state "%s" no ERP' % (brw_nf.numero_nf,
                                                                                      brw_nf.serie_nf,
                                                                                      brw_nf.state)
            elif processo.resposta.cStat.valor == u'101':
                protocolo_canc = processo.resposta.xml.split('<nProt>')[-1].split('</nProt>')[0]
                cr.execute("""UPDATE fiscal_nf_capa SET nfe_protocolo_cancelamento = '%s' WHERE id = %s;
                                              COMMIT;""" % (protocolo_canc, brw_nf.id))
                print 'Nota %s - %s com novo protocolo de cancelamento --> %s' % (brw_nf.numero_nf, brw_nf.serie_nf,
                                                                                  protocolo_canc)
                continue
                if brw_nf.state != 'C':  # Cancelada
                    print 'Nota %s - %s Cancelada na SEFAZ e com state "%s" no ERP' % (brw_nf.numero_nf,
                                                                                       brw_nf.serie_nf,
                                                                                       brw_nf.state)

            elif processo.resposta.cStat.valor in (u'137', u'217'):
                if brw_nf.state != 'I':
                    print 'Nota %s - %s Nao encontrada na SEFAZ e com state "%s" no ERP' % (brw_nf.numero_nf,
                                                                                            brw_nf.serie_nf,
                                                                                            brw_nf.state)
            else:
                cstat = processo.resposta.cStat.valor
                print 'Erro ao consultar Nota %s - %s: Rejeicao %s' % (brw_nf.numero_nf, brw_nf.serie_nf, cstat)

        print 'Finalizado!'

    except Exception as e:
        print 'Erro ao atualizar notas: %s' % e

except Exception as e:
    print 'Erro ao conectar com a Base de dados:%s' % e
