# -*- coding: utf-8 -*-

# Este script Assina o xml gerado das notas canceladas que não possuem assinatura


from nfe.nfc_e import nfce as nfc_e
import psycopg2.extras


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class get_browse(object):
    def __init__(self, d):
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [get_browse(x) if isinstance(x, dict) else x for x in b])
            else:
                setattr(self, a, get_browse(b) if isinstance(b, dict) else b)


def pesquisa_notas_sem_assinatura(cr):
    cr.execute("""SELECT fp.*, 
                         ap.uf
                  FROM fiscal_parametro AS fp
                  LEFT JOIN res_company AS rc ON rc.id = fp.company_id
                  LEFT JOIN adm_pessoa  AS ap ON ap.id = rc.rescon_pessoa_id""")
    dados_par = dictfetchall(cr)
    for par in dados_par:
        par = get_browse(par)
        # PEGANDO DIRETO DO BANCO PARA PEGAR NOTAS DE TODAS AS EMPRESA
        cond_amb = 'AND fnc.fisnfc_nfe_status_configuracao_homologacao IS NOT NULL'
        if par.fispar_nfe_ambiente == 1:
            cond_amb = 'AND fnc.fisnfc_nfe_status_configuracao_producao IS NOT NULL'

        cr.execute("""SELECT fnc.id,
                             fnc.nfe_lote,
                             fnc.state,
                             fnc.numero_nf,
                             fnc.nfe_xml_gerado,
                             fnc.serie_nf,
                             fnc.nfe_chave_acesso,
                             (CASE WHEN fisnfc_nfe_status_configuracao_producao IS NOT NULL
                                   THEN 1
                                   ELSE 2
                              END) AS ambiente
                      FROM fiscal_nf_capa                AS fnc
                      INNER JOIN fiscal_modelo_nf_series AS modelo ON modelo.id = fnc.modelo_nf_series_id
                      WHERE fnc.data_emissao >= '{data_ini} 00:00:00'
                        AND fnc.data_emissao <= '{data_fim} 23:59:59'
                        AND fnc.nfe_chave_acesso IS NOT NULL
                        AND fnc.state = 'C'
                        AND fnc.fisnfc_consumidor = True
                        AND fnc.company_id = {company_id}
                        {cond_amb}
                   """.format(data_ini=data_ini,
                              data_fim=data_fim,
                              company_id=par.company_id,
                              cond_amb=cond_amb))
        for nf in dictfetchall(cr):
            assinar_xml_sem_assinatura(cr, nf, par)

    return True


def assinar_xml_sem_assinatura(cr, dados_nf, parametro):
    brw_nf = get_browse(dados_nf)
    from nfe.nfce.consrecinfce import ProcNFCe
    xml = brw_nf.nfe_xml_gerado or ''
    if not xml:
        return

    try:
        nfce = ProcNFCe()
        nfce.NFe.xml = xml
        if not nfce.NFe.Signature.SignatureValue:
            nf = nfc_e(parametro.certificado_conteudo,
                       parametro.chave_conteudo,
                       brw_nf.ambiente,
                       parametro.uf,
                       False)
            print 'Assinando nota Nota %s - %s ...' % (brw_nf.numero_nf, brw_nf.serie_nf)
            nf.process.certificado.assina_xmlnfe(nfce.NFe)
            xml = nfce.NFe.xml
            cr.execute("""UPDATE fiscal_nf_capa 
                          SET nfe_xml_gerado = '%s' 
                          WHERE id = %s;
                          COMMIT;""" % (xml, brw_nf.id))
            print 'Nota %s - %s: Xml Gerado assinado!' % (brw_nf.numero_nf, brw_nf.serie_nf)

        print '\n\n'

    except Exception as err:
        print 'ERRO - Assina XML %s >> %s' % (brw_nf.nfe_chave_acesso, err)


print 'Iforme os seguintes dados'
erp_host = raw_input('Ip do ERP: ')
erp_usuario = raw_input('Usuario do postgres ERP: ')
erp_nome = raw_input('Nome do Banco ERP: ')
erp_senha = raw_input('Senha do Banco ERP: ')

data_ini = raw_input('Data inicial(DD-MM-AAAA): ').split('-')
data_fim = raw_input('Data final(DD-MM-AAAA):').split('-')

data_ini.reverse()
data_fim.reverse()

cont = 0
msg = []
try:
    print '>>>>>> Analizando notas......\n'
    conn = psycopg2.connect(database=erp_nome, user=erp_usuario, host=erp_host,
                            password=erp_senha, connect_timeout=20)
    cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    try:
        data_ini = '-'.join(data_ini)
        data_fim = '-'.join(data_fim)
        pesquisa_notas_sem_assinatura(cr)
        print '\n\nFinalizado!'
    except Exception as e:
        print 'Erro ao atualizar notas: %s' % e

except Exception as e:
    print 'Erro ao conectar com a Base de dados:%s' % e
