# -*- coding: utf-8 -*-

# Este script recupera o xml de cancelamento que foi gavado apenas com protocolo de cancelamento das NFCEs
# colocando o xml de cancelamento completo

from nfe.nfc_e import nfce as nfc_e
from nfe.nfe_400.manual_600.cancnfe_evento import EnvEvento, RetEnvEvento, ProcEventoNFe
from nfe.nfe_400.processador import ProcessoNFe
from nfe.nfe_400.webservices import UF_CODIGO
from datetime import datetime
import psycopg2.extras
import pytz


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class get_browse(object):
    def __init__(self, d):
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [get_browse(x) if isinstance(x, dict) else x for x in b])
            else:
                setattr(self, a, get_browse(b) if isinstance(b, dict) else b)

def script_gerar_xml_cancelamento(consulta, brw_capa, proc_nfce, protocolo):
    """Refaz o xml de cancelamento, quando a sefaz aceita o cancelamento mas na hora do retorno da erro"""
    ambiente = brw_capa.ambiente
    cr.execute("""SELECT ap.id,
                         ap.uf
                  FROM adm_pessoa       AS ap
                  LEFT JOIN res_company AS rc ON rc.rescon_pessoa_id = ap.id
                  WHERE rc.id = %s
               """ % brw_capa.company_id)
    brw_pessoa = get_browse(dictfetchall(cr)[0])
    chave = brw_capa.nfe_chave_acesso
    envio = EnvEvento()
    resposta = RetEnvEvento()
    processo = ProcessoNFe(2, envio, resposta)
    envio.infEvento.tpAmb.valor = ambiente
    envio.infEvento.chNFe.valor = chave
    envio.infEvento.nProt.valor = protocolo
    envio.infEvento.xJust.valor = 'Nota fiscal emitida em contingencia'
    envio.infEvento.cOrgao.valor = UF_CODIGO[brw_pessoa.uf]
#    dt = datetime.utcnow()
    dt = brw_capa.data_emissao
    dt = dt.replace(tzinfo=pytz.utc)
    data_hora_tz = datetime.astimezone(dt, pytz.timezone("Brazil/East"))
    data_evento = datetime.strftime(data_hora_tz, u'%Y-%m-%dT%H:%M:%S') + str(data_hora_tz)[-6:]
    envio.infEvento.dhEvento.valor = data_evento
    envio.infEvento.CNPJ.valor = chave[6:20]
    proc_nfce.process.certificado.assina_xmlnfe(envio)
    envio.validar()
    # seta os dados que nao tem
    resposta.retEvento.infEvento.cStat.valor = u'135'
    resposta.retEvento.infEvento.tpAmb.valor = ambiente
    resposta.retEvento.infEvento.verAplic.valor = consulta.resposta.protNFe.infProt.verAplic.valor
    resposta.retEvento.infEvento.cOrgao.valor = consulta.resposta.retCancNFe.infCanc.cOrgao.valor
    resposta.retEvento.infEvento.xMotivo.valor = consulta.resposta.retCancNFe.infCanc.xMotivo.valor
    resposta.retEvento.infEvento.dhRegEvento.valor = str(consulta.resposta.retCancNFe.infCanc.dhRegEvento.valor).replace(' ', 'T')
    protocolo_canc = consulta.resposta.retCancNFe.infCanc.nProt.valor
    resposta.retEvento.infEvento.nProt.valor = protocolo_canc
    resposta.retEvento.infEvento.chNFe.valor = chave
    resposta.retEvento.infEvento.Id.valor = 'ID%s' % protocolo_canc
    processo_cancelamento_nfe = ProcEventoNFe()
    processo_cancelamento_nfe.envEvento = envio
    processo_cancelamento_nfe.retEnvEvento = resposta
    processo_cancelamento_nfe.validar()
    processo.processo_cancelamento_nfe = processo_cancelamento_nfe
    return processo.processo_cancelamento_nfe.xml




print 'Iforme os seguintes dados'
erp_host = raw_input('Ip do ERP: ')
erp_usuario = raw_input('Usuario do postgres ERP: ')
erp_nome = raw_input('Nome do Banco ERP: ')
erp_senha = raw_input('Senha do Banco ERP: ')
# erp_host = '192.168.1.166'
# erp_usuario = 'openerp'
# erp_nome = '21L_Robinho_31-08-2022'
# erp_senha = 'SIG_0p3n3rp'

data_ini = raw_input('Data inicial(DD-MM-AAAA): ').split('-')
data_fim = raw_input('Data final(DD-MM-AAAA):').split('-')


data_ini.reverse()
data_fim.reverse()

cont = 0
try:
    print '>>>>>> Analizando notas......\n'
    conn = psycopg2.connect(database=erp_nome, user=erp_usuario, host=erp_host,
                            password=erp_senha, connect_timeout=20)
    cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    try:
        data_ini = '-'.join(data_ini)
        data_fim = '-'.join(data_fim)
        cr.execute("""SELECT fp.company_id,
                             fp.certificado_conteudo, 
                             fp.chave_conteudo, 
                             fp.fispar_nfce_tentativa_envio, 
                             fp.fispar_nfce_conexao_timeout,
                             ap.uf         
                      FROM fiscal_parametro AS fp
                      LEFT JOIN res_company AS rc ON rc.id = fp.company_id
                      LEFT JOIN adm_pessoa  AS ap ON ap.id = rc.rescon_pessoa_id
                   """)
        parametros = {}
        for i in dictfetchall(cr):
            par = get_browse(i)
            parametros[par.company_id] = par

        cr.execute("""SELECT f.id, f.numero_nf, f.serie_nf, f.company_id, f.nfe_chave_acesso, f.data_emissao,
                             (CASE WHEN fisnfc_nfe_status_configuracao_producao IS NOT NULL
                                   THEN 1
                                   ELSE 2
                              END) AS ambiente
                      FROM fiscal_nf_capa f
                      INNER JOIN fiscal_modelo_nf_series AS modelo ON modelo.id = f.modelo_nf_series_id
                      WHERE f.data_emissao AT TIME ZONE 'UTC' >= '%s' 
                        AND f.data_emissao AT TIME ZONE 'UTC' <= '%s'
                        AND f.state = 'C'
                        AND f.nfe_xml_cancelamento not ilike '%%procEventoNFe%%'
                        AND f.fisnfc_consumidor = True              
                   """ % (data_ini, data_fim))

        for i in dictfetchall(cr):
            brw_nf = get_browse(i)
            parametro = parametros.get(brw_nf.company_id)
            #------- Verificando status na sefaz ---------
            parametro = parametros.get(brw_nf.company_id)
            if not parametro.uf:
                raise Exception('Erro: a Empresa %s nao possui UF cadastrado no endereco da pessoa!' %
                                parametro.company_id)
            nf = nfc_e(parametro.certificado_conteudo,
                       parametro.chave_conteudo,
                       brw_nf.ambiente,
                       parametro.uf,
                       False)
            nf.process.tentativas = parametro.fispar_nfce_tentativa_envio or 5
            nf.process.timeout = parametro.fispar_nfce_conexao_timeout or 10
            processo, __ = nf.consultar_nota(brw_nf.nfe_chave_acesso)
            if processo.resposta.cStat.valor != u'101':
                continue

            print '\n>>>>>> Alterando Nota: %s  Serie: %s......' % (brw_nf.numero_nf, brw_nf.serie_nf)
            protocolo = processo.resposta.protNFe.infProt.nProt.valor
            processo.resposta.protNFe.Signature.URI = ''  # tira a assinatura do xml de autorizacao
            xml_proc = processo.resposta.protNFe.infProt.xml
            xml_canc = script_gerar_xml_cancelamento(processo, brw_nf, nf, protocolo)
            cr.execute("""UPDATE fiscal_nf_capa
                          SET nfe_xml_cancelamento = '%s'
                          WHERE id = %s;
                          COMMIT;
                       """ % (xml_canc, brw_nf.id))
            print 'Novo xml de cancelamento:\n', xml_canc

        print 'Finalizado!'

    except Exception as e:
        print 'Erro ao atualizar notas: %s' % e

except Exception as e:
    print 'Erro ao conectar com a Base de dados:%s' % e
