#!/usr/bin/python
# -*- encoding: utf-8 -*-

__author__ = 'Dener Rodrigues'

import getpass
import psycopg2
import psycopg2.extras


class AcertaRateioItens():
    def __init__(self):
        self._connection = self.new_connection()
        if self._connection:
            self._cr = self._connection.cursor()

    @staticmethod
    def new_connection():
        """
        Realiza uma nova conexão no banco de dados.
        """
        try:
            database = raw_input('Nome da base: ')
            host = raw_input('Host: ')
            user = raw_input('Usuario: ')
            password = getpass.getpass('Senha: ')
            port = raw_input('Porta (5432): ')
            connection = psycopg2.connect(database=database, user=user, password=password,
                                          host=host, port=port, connect_timeout=3)
            return connection
        except Exception as err:
            print "Erro ao conectar no banco de dados ... \n\n%s" % str(err)
            raise

    def _exec_sql(self, sql=None, ret=False):
        """
        Executa uma SQL.

        :returns:
        Se 'ret' recebe 'True'
            Retorna uma lista de dicionários com o resultado da SQL
        Se 'ret' recebe 'False'
            Retorna 'True' se a SQL for executada
                Em caso de falha, retorna 'False'
        """
        try:
            self._cr.execute(sql)
            self._connection.commit()
            res = True
            if ret:
                desc = self._cr.description
                fetchall = self._cr.fetchall()
                res = [dict(zip([col[0] for col in desc], row)) for row in fetchall]
        except Exception as err:
            self._connection.rollback()
            res = False
            print str(err)
        return res

    def get_coupons(self):
        """
        Retorna os dados dos cupons que possuem diferença de
        acréscimo/desconto entre a capa do cupom e a somatória dos itens.

        :returns:
        id, caixa, nro_cupom, alteracao, desc_acres_capa, acrescimo_item, desconto_item, data_cupom
        """
        ini = raw_input('Informe a Data Inicio (ano-mes-dia): ')
        fin = raw_input('Informe a Data Fim (ano-mes-dia): ')
        sql = """
                    SELECT *
                    FROM   (SELECT mov_capa.id                           AS id,
                                   mov_capa.supcap_caixa                 AS caixa,
                                   mov_capa.supcap_nro_cupom             AS nro_cupom,
                                   mov_capa.supcap_alteracao             AS alteracao,
                                   mov_capa.supcap_valor_desc_acres      AS desc_acres_capa,
                                   SUM(mov_corpos.supcor_acrescimo_item) AS acrescimo_item_sum,
                                   SUM(mov_corpos.supcor_desconto_item)  AS desconto_item_sum,
                                   mov_capa.supcap_data_cupom            AS data_cupom
                            FROM   supervisor_mov_capa mov_capa
                                   inner join supervisor_mov_corpos mov_corpos
                                           ON mov_capa.id = mov_corpos.supcor_capa_id
                            WHERE  mov_capa.supcap_valor_desc_acres > 0
                            GROUP  BY mov_capa.id
                            ORDER  BY mov_capa.supcap_data_cupom) AS tmp
                    WHERE  ( acrescimo_item_sum != desc_acres_capa
                             AND acrescimo_item_sum > 0
                              OR desconto_item_sum != desc_acres_capa
                                 AND desconto_item_sum > 0 )
        """
        if ini and fin:
            sql += " AND data_cupom >= '%s' AND data_cupom <= '%s' " % (ini, fin)
        print sql
        coupons = self._exec_sql(sql, True)
        return coupons

    def get_item_maior_valor(self, capa_id):
        """
        Retorna o item de maior valor do cupom.
        """
        sql = """
                SELECT id,
                       supcor_acrescimo_item,
                       supcor_desconto_item,
                       supcor_preco_unitario
                FROM   supervisor_mov_corpos
                WHERE  supcor_capa_id = %d
        """ % capa_id
        itens = self._exec_sql(sql, True)
        item_maior_valor_id = None
        if isinstance(itens, list):
            maior_valor = itens[0]['supcor_preco_unitario']
            item_maior_valor_id = itens[0]['id']
            if len(itens) > 1:
                for i in itens:
                    if i['supcor_preco_unitario'] > maior_valor:
                        item_maior_valor_id = i['id']
        return item_maior_valor_id

    def set_item_maior_valor(self, item_id, value, alteracao):
        """
        Altera o valor do maior item do cupom com o valor da diferença de
        acréscimo/desconto entre a capa do cupom e a somatória dos itens.
        """
        col = None
        if alteracao == 'A':
            col = 'supcor_acrescimo_item'
        elif alteracao == 'D':
            col = 'supcor_desconto_item'
        sql = """UPDATE supervisor_mov_corpos
                 SET """ + col + """ = """ + col + """  + %s
                 WHERE  id = %d """ % (value, item_id)
        print sql + '\n'
        return self._exec_sql(sql, False)

    def set_coupons(self):
        cupons = self.get_coupons()
        if isinstance(cupons, list):
            for coo in cupons:
                dif = 0
                alteracao = coo['alteracao']
                acrescimo_desconto_capa = coo['desc_acres_capa']
                capa_id = coo['id']
                if alteracao == 'A':
                    acrescimo_item_sum = coo['acrescimo_item_sum']
                    dif = acrescimo_desconto_capa - acrescimo_item_sum
                elif alteracao == 'D':
                    desconto_item_sum = coo['desconto_item_sum']
                    dif = acrescimo_desconto_capa - desconto_item_sum
                item_id = self.get_item_maior_valor(capa_id=capa_id)
                if item_id and dif:
                    self.set_item_maior_valor(item_id=item_id, value=dif, alteracao=alteracao)


if __name__ == "__main__":
    try:
        acerta_rateio = AcertaRateioItens()
        acerta_rateio.set_coupons()
    except Exception as err:
        print str(err)
