#!/usr/bin/python
# -*- encoding: utf-8 -*-

__author__ = 'Dener Rodrigues'

import getpass
import psycopg2
import psycopg2.extras


class CadastroModBandeira():
    def __init__(self):
        self.connection = self.get_connection()
        self.cursor = self.connection.cursor()

    @staticmethod
    def get_connection():
        """
        Realiza uma nova conexão no banco de dados.
        """
        try:
            database = raw_input('Nome da base: ')
            host = raw_input('Host: ')
            user = raw_input('Usuario: ')
            password = getpass.getpass('Senha: ')
            port = raw_input('Porta: ')
            connection = psycopg2.connect(database=database, user=user, password=password,
                                          host=host, port=port, connect_timeout=3)
            return connection
        except Exception as err:
            raise Exception("Erro ao conectar no banco de dados ... \n\n%s" % str(err))

    def exec_sql(self, sql=None, ret=False):
        """
        Executa uma SQL.

        :rtype : list
        :returns:
        Se 'ret' recebe 'True'
            Retorna uma lista de dicionários com o resultado da SQL
        Se 'ret' recebe 'False'
            Retorna 'True' se a SQL for executada
                Em caso de falha, retorna uma lista vazia
        """
        res = []
        print sql
        try:
            self.cursor.execute(sql)
            self.connection.commit()
            res = True
            if ret:
                desc = self.cursor.description
                fetchall = self.cursor.fetchall()
                res = [dict(zip([col[0] for col in desc], row)) for row in fetchall]
        except Exception as err:
            self.connection.rollback()
            print str(err)
        return res

    def cadastrar_mod_bandeira(self):
        """
        79.945-Cadastro no campo Mod Bandeira
        """

        dict_bandeira_cred = {'00000': U"OUTRO, NAO IDENTIF.", '00001': U"VISA",
                              '00002': U"MASTERCARD",
                              '00003': U"DINERS", '00004': U"AMERICAN EXPRESS", '00005': U"SOLLO",
                              '00006': U"SIDECARD (REDECARD)", '00007': U"PRIVATE LABEL (REDECARD)",
                              '00008': U"REDESHOP", '00009': U"PAO ACUCAR", '00010': U"FININVEST",
                              '00011': U"JCB", '00012': U"HIPERCARD", '00013': U"AURA",
                              '00014': U"LOSANGO", '00015': U"SOROCRED", '10014': U"D1SCOVERY",
                              '00030': U"CABAL", '00031': U"ELO", '00033': U"POLICARD",
                              '00035': U"BANESCARD", '10013': U"ELO", '10015': U"GOOD CARD",
                              '10016': U"POLICARD", '10017': U"CARD SYSTEM", '10020': U"BANESCARD",
                              '10018': U"BONUS CBA"}

        dict_bandeira_deb = {'00000': u"DEBITO_GENERICO", '00001': u"VOUCHER",
                             '00003': u"GIFT PREPAGO",
                             '20001': u"MAESTRO", '20002': u"ELECTRON", '20003': u"CABALDEBIT",
                             '10012': u"CABAL", '10001': u"TICKET", '10024': u"TICKET REFEICAO",
                             '10025': u"TICKET ALIMENTACAO", '10026': u"TICKET PARCEIRO",
                             '10027': u"TICKET CULTURA", '10054': u"TICKET COMBUSTIVEL",
                             '10002': u"VISAVALE", '10003': u"SODEXHO",
                             '10028': u"SODEXHO REFEICAO",
                             '10029': u"SODEXHO ALIMENTACAO", '10030': u"SODEXHO GIFT",
                             '10031': u"SODEXHO PREMIUM", '10032': u"SODEXHO CULTURA",
                             '10033': u"SODEXHO COMBUSTIVEL", '10004': u"NUTRICASH",
                             '10053': u"NUTRICASH CULTURA", '10005': u"GREENCARD",
                             '10006': u"PLANVALE",
                             '10051': u"PLANVALE CULTURA", '10007': u"BANQUET",
                             '10008': u"VEROCHEQUE",
                             '10009': u"SAPORE", '10010': u"BNBCLUBE", '10011': u"VALECARD",
                             '10055': u"VALECARD CULTURA", '10019': u"ALELO",
                             '10021': u"ALELO REFEICAO",
                             '10022': u"ALELO ALIMENTACAO", '10023': u"ALELO CULTURA",
                             '20013': u"ELO",
                             '20034': u"POLICARD"}

        list_bandeiras = [dict_bandeira_cred, dict_bandeira_deb]

        for bandeiras in list_bandeiras:
            tipo = 'C' if bandeiras == list_bandeiras[0] else 'D'
            for bandeira in bandeiras:
                name = dict_bandeira_cred[bandeira] if tipo == 'C' else dict_bandeira_deb[bandeira]
                if not name:
                    name = dict_bandeira_cred['00000'] if tipo == 'C' else dict_bandeira_deb[
                        bandeira]
                descricao_completa = bandeira.rjust(5, '0') + ' - ' + name
                sql_insert = """
                INSERT INTO supervisor_cad_bandeiras
                            (supban_codigo, name, supban_descricao_completa, supban_tipo)
                VALUES      (%s, '%s', '%s','%s')
                RETURNING   id ;
                """ % (bandeira, name, descricao_completa, tipo)
                self.exec_sql(sql_insert, True)

        sql = """
        SELECT id,
               supfin_cartao_tipo,
               supfin_mod_bandeira,
               supfin_mod_bandeira_deb
        FROM   supervisor_cad_finalizadora
        WHERE  supfin_usa_tef IS TRUE ;
        """

        finalizadoras = self.exec_sql(sql, True)

        bandeiras_ids = 0
        for finalizadora in finalizadoras:
            finalizadora_id = finalizadora.get('id')
            tipo = finalizadora.get('supfin_cartao_tipo')
            mod_bandeira_cred = finalizadora.get('supfin_mod_bandeira')
            mod_bandeira_deb = finalizadora.get('supfin_mod_bandeira_deb')

            codigo_bandeira = mod_bandeira_cred if tipo == 'C' else mod_bandeira_deb

            if codigo_bandeira:
                sql_bandeira = """
                SELECT id
                FROM   supervisor_cad_bandeiras
                WHERE  supban_codigo = %s
                       AND supban_tipo = '%s' ;
                """ % (codigo_bandeira, tipo)

                bandeira_id = self.exec_sql(sql_bandeira, True)

                if bandeira_id and isinstance(bandeira_id, list):
                    if tipo == 'C':
                        mod_bandeira_cred = bandeira_id[0].get('id')
                        mod_bandeira_deb = 'Null'
                    else:
                        mod_bandeira_deb = bandeira_id[0].get('id')
                        mod_bandeira_cred = 'Null'

                    sql_update = """
                    UPDATE supervisor_cad_finalizadora
                    SET    supfin_bandeira_cre = %s,
                           supfin_bandeira_deb = %s
                    WHERE  id = %s ;
                    """ % (mod_bandeira_cred, mod_bandeira_deb, finalizadora_id)

                    if self.exec_sql(sql_update):
                        bandeiras_ids += 1

        return True

if __name__ == "__main__":
    try:
        cadastroModBandeira = CadastroModBandeira()
        cadastroModBandeira.cadastrar_mod_bandeira()
    except Exception as err:
        raise err
