#!/usr/bin/python
# -*- encoding: utf-8 -*-


import psycopg2
import getpass


class AcertaDiferencaoC420(object):
    def __init__(self, host, database, user, password, port, data_inicio, data_final):
        self.host = host
        self.database = database
        self.user = user
        self.password = password
        self.port = port
        self.data_inicio = data_inicio
        self.data_final = data_final
        self.con = psycopg2.connect(database=self.database, port=self.port, host=self.host,
                                    user=self.user, password=self.password, connect_timeout=3)
        self.cr = self.con.cursor()
        self.diferenca = []
        self.grade_indisponivel = []
        self.movimento_indisponivel = []

    @staticmethod
    def dictfetchall(cr):
        """Retorna todas as linhas de um cursor como um dicionario"""
        desc = cr.description
        return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]

    def get_diferenca(self):
        sql = """
        SELECT
               (
                      SELECT caixa
                      FROM   ecf_ecf
                      WHERE  id = ecf_id) AS caixa,
                        *,
               vlr_c420 - vlr_c490 AS diferenca
        FROM   (
                        SELECT   ecf_id,
                                 data,
                                 aliquota,
                                 tipo,
                                 Sum(vlr_c420) AS vlr_c420,
                                 Sum(vlr_c490) AS vlr_c490
                        FROM     (
                                        SELECT ecf_id,
                                               data,
                                               aliquota,
                                               tipo,
                                               vlr_acum_tot AS vlr_c420,
                                               0            AS vlr_c490
                                        FROM   (
                                                      SELECT Cast('C420' AS TEXT) AS reg,
                                                             ''                   AS cod_tot_par,
                                                             base_valor           AS vlr_acum_tot,
                                                             tipo,
                                                             aliquota,
                                                             imposto,
                                                             capa_id,
                                                             (
                                                                    SELECT data
                                                                    FROM   ecf_mapa_resumo_capa
                                                                    WHERE  id = capa_id) AS data,
                                                             (
                                                                    SELECT ecf
                                                                    FROM   ecf_mapa_resumo_capa
                                                                    WHERE  id = capa_id) AS ecf_id
                                                      FROM   ecf_mapa_resumo_itens
                                                      WHERE  capa_id IN
                                                             (
                                                                    SELECT id
                                                                    FROM   ecf_mapa_resumo_capa)
                                                      AND    base_valor > 0) AS c420
                                        WHERE  data BETWEEN '{data_inicio}' AND    '{data_final}'
                                        UNION ALL
                                        SELECT   ecf_id,
                                                 date_cupom AS data,
                                                 aliq_icms  AS aliquota,
                                                 CASE
                                                          WHEN tipo = 'U' THEN 'F'
                                                          ELSE tipo
                                                 END         AS tipo,
                                                 0           AS vlr_c420,
                                                 Sum(vl_opr) AS vlr_c490
                                        FROM     (
                                                            SELECT     Cast('C490' AS TEXT) AS reg,
                                                                       cst.codigo           AS cst_icms,
                                                                       (
                                                                              SELECT codigo
                                                                              FROM   fiscal_cfop cfop
                                                                              WHERE  cfop.id = movimento_itens.cfop_id) AS cfop,
                                                                       item_tributacao.aliquota                         AS aliq_icms,
                                                                       Sum(movimento_itens.vr_total_item_rateado)       AS vl_opr,
                                                                       CASE
                                                                                  WHEN cst.codigo NOT IN ( '30',
                                                                                                          '40',
                                                                                                          '41',
                                                                                                          '50',
                                                                                                          '60' ) THEN Sum( vr_base_calculo)
                                                                                  ELSE 0.00
                                                                       END AS vl_bc_icms,
                                                                       CASE
                                                                                  WHEN cst.codigo NOT IN ( '30',
                                                                                                          '40',
                                                                                                          '41',
                                                                                                          '50',
                                                                                                          '60' ) THEN Sum( vr_encargo)
                                                                                  ELSE 0.00
                                                                       END AS vl_icms,
                                                                       ''  AS cod_obs,
                                                                       cupons_capa.ecf_id,
                                                                       cupons_capa.date_cupom,
                                                                       cst.tipo
                                                            FROM       fiscal_nf_item_tributacao item_tributacao
                                                            INNER JOIN fiscal_cst cst
                                                            ON         item_tributacao.cst_id = cst.id
                                                            INNER JOIN estoque_movimento_itens movimento_itens
                                                            ON         movimento_itens.id = item_tributacao.movimento_itens_id
                                                            INNER JOIN estoque_movimento_capa movimento_capa
                                                            ON         movimento_capa.id = movimento_itens.capa_id
                                                            INNER JOIN ecf_cupons_capa cupons_capa
                                                            ON         cupons_capa.id = movimento_itens.cupom_id
                                                            AND        movimento_itens.nf_id IS NULL
                                                            WHERE      cupons_capa.date_cupom BETWEEN '{data_inicio}' AND '{data_final}'
                                                            AND        cupons_capa.tipo_cupom = 'V'
                                                            AND        movimento_itens.item_cancelado = false
                                                            AND        item_tributacao.encargo_id =
                                                                       (
                                                                              SELECT id
                                                                              FROM   fiscal_encargo
                                                                              WHERE  sigla ilike'ICMS')
                                                            GROUP BY   cst.codigo,
                                                                       movimento_itens.cfop_id,
                                                                       item_tributacao.aliquota,
                                                                       cupons_capa.ecf_id,
                                                                       cupons_capa.date_cupom,
                                                                       cst.tipo
                                                            UNION ALL
                                                            SELECT     cast('C490' AS text) AS reg,
                                                                       cst.codigo           AS cst_icms,
                                                                       (
                                                                              SELECT codigo
                                                                              FROM   fiscal_cfop cfop
                                                                              WHERE  cfop.id = movimento_itens.cfop_id) AS cfop,
                                                                       item_tributacao.aliquota                         AS aliq_icms,
                                                                       sum(-movimento_itens.vr_total_item_rateado)      AS vl_opr,
                                                                       CASE
                                                                                  WHEN cst.codigo NOT IN ( '30',
                                                                                                          '40',
                                                                                                          '41',
                                                                                                          '50',
                                                                                                          '60' ) THEN sum( -vr_base_calculo)
                                                                                  ELSE 0.00
                                                                       END AS vl_bc_icms,
                                                                       CASE
                                                                                  WHEN cst.codigo NOT IN ( '30',
                                                                                                          '40',
                                                                                                          '41',
                                                                                                          '50',
                                                                                                          '60' ) THEN sum( -vr_encargo)
                                                                                  ELSE 0.00
                                                                       END AS vl_icms,
                                                                       ''  AS cod_obs,
                                                                       cupons_capa.ecf_id,
                                                                       cupons_capa.date_cupom,
                                                                       cst.tipo
                                                            FROM       fiscal_nf_item_tributacao item_tributacao
                                                            INNER JOIN fiscal_cst cst
                                                            ON         item_tributacao.cst_id = cst.id
                                                            INNER JOIN estoque_movimento_itens movimento_itens
                                                            ON         movimento_itens.id = item_tributacao.movimento_itens_id
                                                            INNER JOIN estoque_movimento_capa movimento_capa
                                                            ON         movimento_capa.id = movimento_itens.capa_id
                                                            INNER JOIN ecf_cupons_capa cupons_capa
                                                            ON         cupons_capa.id = movimento_itens.cupom_id
                                                            AND        movimento_itens.nf_id IS NULL
                                                            WHERE      cupons_capa.date_cupom BETWEEN '{data_inicio}' AND '{data_final}'
                                                            AND        cupons_capa.tipo_cupom = 'C'
                                                            AND        movimento_itens.item_cancelado = true
                                                            AND        item_tributacao.encargo_id =
                                                                       (
                                                                              SELECT id
                                                                              FROM   fiscal_encargo
                                                                              WHERE  sigla ilike'ICMS')
                                                            GROUP BY   cst.codigo,
                                                                       movimento_itens.cfop_id,
                                                                       item_tributacao.aliquota,
                                                                       cupons_capa.ecf_id,
                                                                       cupons_capa.date_cupom,
                                                                       cst.tipo ) AS tmp
                                        GROUP BY reg,
                                                 cst_icms,
                                                 cfop,
                                                 aliq_icms,
                                                 ecf_id,
                                                 date_cupom,
                                                 tipo) AS totalizador
                        GROUP BY ecf_id,
                                 data,
                                 aliquota,
                                 tipo) AS tmp
        WHERE  vlr_c420 <> vlr_c490
        """.format(data_inicio=self.data_inicio, data_final=self.data_final)
        self.cr.execute(sql)
        self.diferenca = self.dictfetchall(self.cr)

    def get_produto_compativel(self, ecf_id, tipo, aliquota, data):
        if self.grade_indisponivel:
            grade_indisponivel = str(self.grade_indisponivel).replace('[', '(').replace(']', ')')
            sql_grade_indisponivel = """
            AND grade_id not in {grade_indisponivel}
            """.format(grade_indisponivel=grade_indisponivel)
        else:
            sql_grade_indisponivel = ""
        sql = """
                SELECT   produto_id,
                         grade_id,
                         Sum(qtd)     AS qtd,
                         Sum(vl_item) AS vl_item
                FROM     (
                                    SELECT     produto.id                           AS produto_id,
                                               grade.id                             AS grade_id,
                                               Sum(mov_itens.quantidade)            AS qtd,
                                               Sum(mov_itens.vr_total_item_rateado) AS vl_item
                                    FROM       estoque_movimento_itens mov_itens
                                    INNER JOIN estoque_produto_embalagem produto_embalagem
                                    ON         mov_itens.produto_embalagem_id = produto_embalagem.id
                                    INNER JOIN estoque_embalagem embalagem
                                    ON         produto_embalagem.embalagem_id = embalagem.id
                                    INNER JOIN fiscal_nf_item_tributacao item_tributacao
                                    ON         item_tributacao.movimento_itens_id = mov_itens.id
                                    INNER JOIN estoque_produto produto
                                    ON         mov_itens.produto_id = produto.id
                                    INNER JOIN estoque_produto_grade grade
                                    ON         mov_itens.grade_id = grade.id
                                    INNER JOIN ecf_cupons_capa cupons_capa
                                    ON         mov_itens.cupom_id = cupons_capa.id
                                    WHERE      cupons_capa.ecf_id = {ecf_id}
                                    AND        item_tributacao.cst_id IN
                                               (
                                                      SELECT id
                                                      FROM   fiscal_cst
                                                      WHERE  tipo = '{tipo}')
                                    AND        item_tributacao.aliquota = {aliquota}
                                    AND        cupons_capa.state = 'V'
                                    AND        cupons_capa.date_cupom = '{data}'
                                    AND        cupons_capa.tipo_cupom = 'V'
                                    AND        mov_itens.item_cancelado = false
                                    AND        item_tributacao.encargo_id =
                                               (
                                                      SELECT id
                                                      FROM   fiscal_encargo
                                                      WHERE  sigla ilike'ICMS')
                                    AND        mov_itens.nf_id IS NULL
                                    GROUP BY   mov_itens.produto_id,
                                               produto.id,
                                               grade.id,
                                               cupons_capa.tipo_cupom,
                                               mov_itens.item_cancelado
                                    UNION ALL
                                    --Cancelamento
                                    SELECT     produto.id                            AS produto_id,
                                               grade.id                              AS grade_id,
                                               sum(-mov_itens.quantidade)            AS qtd,
                                               sum(-mov_itens.vr_total_item_rateado) AS vl_item
                                    FROM       estoque_movimento_itens mov_itens
                                    INNER JOIN estoque_produto_embalagem produto_embalagem
                                    ON         mov_itens.produto_embalagem_id = produto_embalagem.id
                                    INNER JOIN estoque_embalagem embalagem
                                    ON         produto_embalagem.embalagem_id = embalagem.id
                                    INNER JOIN fiscal_nf_item_tributacao item_tributacao
                                    ON         item_tributacao.movimento_itens_id = mov_itens.id
                                    INNER JOIN estoque_produto produto
                                    ON         mov_itens.produto_id = produto.id
                                    INNER JOIN estoque_produto_grade grade
                                    ON         mov_itens.grade_id = grade.id
                                    INNER JOIN ecf_cupons_capa cupons_capa
                                    ON         mov_itens.cupom_id = cupons_capa.id
                                    WHERE      cupons_capa.ecf_id = {ecf_id}
                                    AND        item_tributacao.cst_id IN
                                               (
                                                      SELECT id
                                                      FROM   fiscal_cst
                                                      WHERE  tipo = '{tipo}')
                                    AND        item_tributacao.aliquota = {aliquota}
                                    AND        cupons_capa.state = 'V'
                                    AND        cupons_capa.date_cupom = '{data}'
                                    AND        cupons_capa.tipo_cupom = 'C'
                                    AND        mov_itens.item_cancelado = true
                                    AND        item_tributacao.encargo_id =
                                               (
                                                      SELECT id
                                                      FROM   fiscal_encargo
                                                      WHERE  sigla ilike'ICMS')
                                    AND        mov_itens.nf_id IS NULL
                                    GROUP BY   mov_itens.produto_id,
                                               produto.id,
                                               grade.id,
                                               cupons_capa.tipo_cupom,
                                               mov_itens.item_cancelado ) AS tmp
                WHERE    qtd > 0
                AND      vl_item > 0
                {sql_grade_indisponivel}
                GROUP BY produto_id,
                         grade_id
                ORDER BY produto_id,
                         grade_id limit 1
        """.format(ecf_id=ecf_id, tipo=tipo, aliquota=aliquota, data=data,
                   sql_grade_indisponivel=sql_grade_indisponivel)
        self.cr.execute(sql)
        return self.dictfetchall(self.cr)

    def get_movimento_compativel(self, produto_id, grade_id, ecf_id, data, aliquota,
                                 diferenca=False):

        if self.movimento_indisponivel:
            mov_indisponivel = str(self.movimento_indisponivel).replace('[', '(').replace(']', ')')
            sql_mov_indisponivel = """
            AND emi.id NOT IN {movimento_indisponivel}
            """.format(movimento_indisponivel=mov_indisponivel)
        else:
            sql_mov_indisponivel = ''
        if diferenca:
            sql_diferenca = "AND emi.vr_total_item_rateado > {diferenca}".\
                format(diferenca=diferenca)
        else:
            sql_diferenca = ""
        sql = """
            SELECT emi.id AS mov_id,
                   emi.vr_total_item_rateado,
                   fisnit.vr_base_calculo,
                   fisnit.vr_encargo
            FROM   estoque_movimento_itens emi
                   INNER JOIN ecf_cupons_capa ecc
                           ON ecc.id = emi.cupom_id
                              AND emi.nf_id IS NULL
                   INNER JOIN fiscal_nf_item_tributacao fisnit
                           ON fisnit.movimento_itens_id = emi.id
                              AND fisnit.encargo_id = (SELECT id
                                                       FROM   fiscal_encargo
                                                       WHERE  sigla = 'ICMS')
            WHERE  produto_id = {produto_id}
                   AND grade_id = {grade_id}
                   AND ecc.ecf_id = {ecf_id}
                   AND ecc.date_cupom = '{data}'
                   AND state = 'V'
                   AND tipo_cupom = 'V'
                   AND aliquota = {aliquota}
                   {sql_diferenca}
                   {sql_mov_indisponivel}
        """.format(produto_id=produto_id, grade_id=grade_id, ecf_id=ecf_id, data=data,
                   aliquota=aliquota, sql_diferenca=sql_diferenca,
                   sql_mov_indisponivel=sql_mov_indisponivel)
        self.cr.execute(sql)
        return self.dictfetchall(self.cr)

    def acerta_movimento_estoque(self, mov_id, vr_total_item_rateado_antigo,
                                 vr_total_item_rateado_novo):
        msg = ""
        try:
            sql_mov_itens = """
                UPDATE estoque_movimento_itens
                SET    vr_total_item_rateado = {vr_total_item_rateado}
                WHERE  id = {mov_id}
                """.format(mov_id=mov_id, vr_total_item_rateado=vr_total_item_rateado_novo)
            self.cr.execute(sql_mov_itens)
            self.con.commit()
            msg = "Movimento-Estoque: {mov_id}\tValor-antigo: {vr_total_item_rateado_antigo}\t" \
                  "Valor-novo: {vr_total_item_rateado_novo}\tAcertou: {acertou}".\
                format(mov_id=mov_id, vr_total_item_rateado_antigo=vr_total_item_rateado_antigo,
                       vr_total_item_rateado_novo=vr_total_item_rateado_novo, acertou='OK')
        except:
            self.con.rollback()
            msg = "Movimento-Estoque: {mov_id}\tValor-antigo: {vr_total_item_rateado_antigo}\t" \
                  "Valor-novo: {vr_total_item_rateado_novo}\tAcertou: {acertou}". \
                format(mov_id=mov_id, vr_total_item_rateado_antigo=vr_total_item_rateado_antigo,
                       vr_total_item_rateado_novo=vr_total_item_rateado_novo, acertou='FALHOU')
        finally:
            print msg

    def acerta_movimento_fisnit(self, mov_id, vr_base_calculo, vr_encargo):
        msg = ""
        sql = """
                UPDATE fiscal_nf_item_tributacao
                SET    vr_base_calculo = {vr_base_calculo},
                       vr_encargo = {vr_encargo}
                WHERE  movimento_itens_id = {mov_id}
                       AND encargo_id = (SELECT id
                                         FROM   fiscal_encargo
                                         WHERE  sigla = 'ICMS')
            """.format(mov_id=mov_id, vr_base_calculo=vr_base_calculo,
                       vr_encargo=vr_encargo)
        try:
            self.cr.execute(sql)
            self.con.commit()
            msg = "Valor-base-calculo: {vr_base_calculo}\tValor-encargo: {vr_encargo}\t" \
                  "Acertou: {acertou}".format(vr_base_calculo=vr_base_calculo, vr_encargo=vr_encargo
                                              , acertou='OK')
        except:
            self.con.rollback()
            msg = "Valor-base-calculo: {vr_base_calculo}\tValor-encargo: {vr_encargo}\t" \
                  "Acertou: {acertou}".format(vr_base_calculo=vr_base_calculo, vr_encargo=vr_encargo
                                              , acertou='FALHOU')
        finally:
            print msg

if __name__ == '__main__':
    host = raw_input('IP do banco: ')
    database = raw_input('Nome do banco: ')
    user = raw_input('Usuario do banco: ')
    password = getpass.getpass('Senha do banco: ')
    port = raw_input('Porta do banco: ')
    data_inicio = raw_input('Data inicial: ')
    data_final = raw_input('Data final: ')
    acerta = AcertaDiferencaoC420(host, database, user, password, port, data_inicio, data_final)
    acerta.get_diferenca()
    existe_diferenca = True
    while existe_diferenca:
        for i in acerta.diferenca:
            if i.get('tipo') == 'F':
                tipo = 'U'
            else:
                tipo = i.get('tipo')
            produto_compativel = acerta.get_produto_compativel(i.get('ecf_id'), tipo, i.get('aliquota'),
                                                               i.get('data'))
            checa_diferenca = False
            produto_id = False
            grade_id = False
            if i.get('vlr_c490') > i.get('vlr_c420'):
                checa_diferenca = i.get('diferenca')
            for produto in produto_compativel:
                produto_id = produto.get('produto_id')
                grade_id = produto.get('grade_id')
            mov_compativel = acerta.get_movimento_compativel(produto_id, grade_id, i.get('ecf_id'),
                                                             i.get('data'), i.get('aliquota'),
                                                             checa_diferenca)
            if not mov_compativel:
                acerta.grade_indisponivel.append(grade_id)
                continue
            for mov in mov_compativel:
                mov_id = mov.get('mov_id')
                vr_total_item_rateado = mov.get('vr_total_item_rateado')
                vr_base_calculo = mov.get('vr_base_calculo')
                vr_encargo = mov.get('vr_encargo')
                if checa_diferenca:
                    if vr_total_item_rateado <= i.get('diferenca'):
                        acerta.movimento_indisponivel.append(mov_id)
                        continue
                    vr_total_item_rateado_novo = vr_total_item_rateado - i.get('diferenca')
                    vr_base_calculo_novo = vr_base_calculo - i.get('diferenca')
                else:
                    vr_total_item_rateado_novo = vr_total_item_rateado + i.get('diferenca')
                    vr_base_calculo_novo = vr_base_calculo + i.get('diferenca')
                vr_encargo_novo = round((vr_base_calculo_novo * i.get('aliquota')) / 100, 2) or 0
                acerta.acerta_movimento_estoque(mov_id, vr_total_item_rateado,
                                                vr_total_item_rateado_novo)
                acerta.acerta_movimento_fisnit(mov_id, vr_base_calculo_novo, vr_encargo_novo)
            acerta.diferenca.remove(i)
        if not acerta.diferenca:
            existe_diferenca = False
