#!/usr/bin/python
# -*- encoding: utf-8 -*-


__description__ = "Script python para corrigir preco total da venda/cancelamento que difere do " \
                  "valor total do cupom"

__author__ = 'Dener Rodrigues'


import getpass
import psycopg2
import psycopg2.extras


class Connection(object):
    def __init__(self):
        self.database = raw_input('Nome da base: ')
        self.host = raw_input('Host: ') or 'localhost'
        self.user = raw_input('Usuario: ') or 'openerp'
        self.password = getpass.getpass('Senha: ')
        self.port = raw_input('Porta: ') or 5432

    def get_connection(self):
        """
        Realiza uma nova conexão no banco de dados.
        :rtype: psycopg2._psycopg.connection
        """

        try:
            connection = psycopg2.connect(database=self.database, host=self.host, user=self.user,
                                          password=self.password, port=self.port, connect_timeout=3)
            return connection
        except Exception as err:
            raise("Erro ao conectar no banco de dados ... \n\n%s" % str(err))


class AcertaCuponsDivergentes(object):
    def __init__(self, cursor):
        self.cursor = cursor
        self.divergencia = 'O preco total da venda/cancelamento difere do valor total do cupom'

    def dictfetchall(self):
        """
        Retorna todas as linhas de um cursor como um dicionario
        """
        return [dict(zip([col[0] for col in self.cursor.description], row)) for row in self.cursor.fetchall()]

    def get_cupons_divergentes(self):

        print 'Procurando cupons onde %s ...' % self.divergencia.lower()

        sql = """
            SELECT capa_id,
                   nro_cupom,
                   somatoria_itens

            FROM   (SELECT capa.id                           AS capa_id,
                           capa.supcap_nro_cupom             AS nro_cupom,
                           capa.supcap_total_cupom           AS total_cupom,
                           SUM(corpo.supcor_preco_total
                             + corpo.supcor_acrescimo_item
                             - corpo.supcor_desconto_item)   AS somatoria_itens

                    FROM   supervisor_mov_capa capa
                           INNER JOIN supervisor_mov_corpos corpo
                                   ON corpo.supcor_capa_id = capa.id

                    GROUP  BY capa_id) AS tmp

            WHERE  nro_cupom IN (SELECT ecfccp.coo
                                 FROM   ecf_cupons_capa ecfccp
                                        INNER JOIN ecf_cupom_log log
                                                ON log.cupom_id = ecfccp.id
                                 WHERE  log.txt_erro LIKE '%%%s%%')
        """ % self.divergencia

        self.cursor.execute(sql)
        cupons_divergentes = self.dictfetchall()

        print '%s cupons encontrados!' % len(cupons_divergentes)

        return cupons_divergentes

    def atualiza_total_cupons(self):
        cupons_divergentes = self.get_cupons_divergentes()
        capa_ids = map(lambda x: x.get('capa_id'), cupons_divergentes)

        for cupom in cupons_divergentes:
            sql = """
                UPDATE supervisor_mov_capa
                SET    supcap_total_cupom = %s
                WHERE  id = %s
            """ % (cupom.get('somatoria_itens'), cupom.get('capa_id'))

            print 'Atualizando cupom %s ...' % cupom.get('nro_cupom')

            try:
                self.cursor.execute(sql)
            except Exception, e:
                self.cursor.connection.commit()
                print sql
                raise('Erro ao atualizar cupom %s: %s' % (cupom.get('nro_cupom'), str(e)))

        self.cursor.connection.commit()

        return capa_ids

    def deleta_cupons_divergentes(self):

        sql = """
            DELETE FROM ecf_cupons_capa
            WHERE       id IN (SELECT cupom_id
                               FROM   ecf_cupom_log
                               WHERE  txt_erro LIKE '%%%s%%')
        """ % self.divergencia

        self.cursor.execute(sql)
        self.cursor.connection.commit()

        return True

    def atualiza_cupom_servidor(self, capa_ids):
        sql = """
            UPDATE supervisor_mov_cupom_servidor
            SET    supcse_enviado = FALSE
            WHERE  supcse_capa_id IN %s
        """ % str(tuple(capa_ids)).replace(',)', ')')

        self.cursor.execute(sql)
        self.cursor.connection.commit()

        return True

    def reimportar_cupons(self, capa_ids):
        if not capa_ids:
            return False

        print 'Reimportando cupons ...'

        try:
            self.deleta_cupons_divergentes()
            self.atualiza_cupom_servidor(capa_ids)
        except Exception, e:
            raise('Erro ao reimportar cupons: %s' % str(e))

        return True


if __name__ == "__main__":
    print
    print __description__
    print __author__
    print

    obj_acerta_cupons = AcertaCuponsDivergentes(Connection().get_connection().cursor())
    obj_acerta_cupons.reimportar_cupons(obj_acerta_cupons.atualiza_total_cupons())

    print 'Fim de execucao!'
