#!/usr/bin/python
# -*- encoding: utf-8 -*-
import psycopg2
from getpass import getpass
from openerp.osv.expression import fields


def dictfetchall(cr):
    """Retorna todas as linhas de um cursor como um dicionario"""
    desc = cr.description
    return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]


def get_digerenca(cr):
    sql = """
            SELECT fisnit.id AS fisnit_id,
                   emi.vr_total_item_rateado,
                   fisnit.vr_base_calculo,
                   fisnit.aliquota,
                   fe.sigla
            FROM   estoque_movimento_itens emi
                   INNER JOIN fiscal_nf_item_tributacao fisnit
                           ON fisnit.movimento_itens_id = emi.id
                   INNER JOIN fiscal_encargo fe
                           ON fe.id = fisnit.encargo_id
                   INNER JOIN fiscal_nf_capa fnc
                           ON fnc.id = emi.nf_id AND emi.cupom_id IS NULL
            WHERE  fisnit.vr_base_calculo <> emi.vr_total_item_rateado
                   AND fisnit.vr_base_calculo > 0
                   AND fe.sigla IN ('PIS', 'COFINS')
    """
    cr.execute(sql)
    return dictfetchall(cr)


def acerta_base_calculo(cr, dados):
    sql = """
            UPDATE fiscal_nf_item_tributacao
            SET    vr_base_calculo = {vr_base_calculo},
                   vr_encargo = {vr_encargo}
            WHERE  id = {fisnit_id}"""
    for dado in dados:
        fisnit_id = dado.get('fisnit_id')
        vr_total_item_rateado = dado.get('vr_total_item_rateado')
        vr_base_calculo = dado.get('vr_base_calculo')
        aliquota = dado.get('aliquota')
        if vr_base_calculo > 0.0:
            try:
                vr_base_calculo = vr_total_item_rateado
                vr_encargo = fields.float_round((vr_base_calculo * aliquota) / 100, 2)
                cr.execute(sql.format(fisnit_id=fisnit_id, vr_base_calculo=vr_base_calculo,
                                      vr_encargo=vr_encargo))
                cr.connection.commit()
            except:
                cr.connection.rollback()
                continue

if __name__ == '__main__':
    try:
        host = raw_input('IP do banco: ')
        db = raw_input('Nome do banco: ')
        user = raw_input('Usuario do banco: ')
        password = getpass('Senha do banco: ')
        port = raw_input('Porta do banco: ')
        con = psycopg2.connect(database=db, user=user, password=password, port=port, host=host,
                               connect_timeout=3)
        cr = con.cursor()
        dados_diferenca = get_digerenca(cr)
        acerta_base_calculo(cr, dados_diferenca)
    except:
        raise ("Erro na execucao do script!")
