#!/usr/bin/python
# -*- encoding: utf-8 -*-
import psycopg2
import getpass


class AcertaBase():
    def __init__(self):
        self.database = str(raw_input('Nome do banco: '))
        self.host = str(raw_input('IP do banco: '))
        self.user = str(raw_input('Usuario do banco: '))
        self.password = getpass.getpass('Senha do banco: ')
        self.port = raw_input('Porta do banco: ')
        self.conn = self.new_connection()
        self.cr = self.conn.cursor()
        self.data_inicio = raw_input('Informe a inicial: (Ex: 2014-11-01): ')
        self.data_final = raw_input('Informe a inicial: (Ex: 2014-11-30): ')

    def new_connection(self):
        if not self.password:
            self.password = None
        if not self.port:
            self.port = 5432
        try:
            conn = psycopg2.connect(database=self.database, user=self.user, password=self.password,
                                    host=self.host, port=self.port, connect_timeout=3)
            return conn
        except:
            print 'Nao foi possivel conectar ao banco de dados!'
            exec exit()

    @staticmethod
    def dictfetchall(cr):
        """Retorna todas as linhas de um cursor como um dicionario"""
        desc = cr.description
        return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]

    def get_nf_item_trib_id(self):
        sql = """
                SELECT fn.id                               AS nf_tributacao_id,
                       Trunc(emi.vr_total_item_rateado, 2) AS total_item_rateado,
                       Trunc(fn.vr_base_calculo, 2)        AS base_calculo,
                       Trunc(emi.vr_desconto, 2)           AS desconto
                FROM   fiscal_nf_capa nf_capa
                       INNER JOIN estoque_movimento_itens emi
                               ON emi.nf_id = nf_capa.id
                       INNER JOIN estoque_produto ep
                               ON ep.id = emi.produto_id
                       INNER JOIN fiscal_nf_item_tributacao fn
                               ON fn.movimento_itens_id = emi.id
                       INNER JOIN fiscal_encargo fe
                               ON fe.id = fn.encargo_id
                                  AND fe.sigla = 'ICMS'
                                  AND vr_base_calculo > vr_total_item_rateado
                       WHERE nf_capa.data_saida_entrada >= '%s'
                       AND nf_capa.data_saida_entrada <= '%s'
                ORDER  BY nf_capa.data_saida_entrada,
                          nf_capa.numero_nf ASC
        """ % (self.data_inicio, self.data_final)
        self.cr.execute(sql)
        dados = self.dictfetchall(self.cr)
        return dados

    @staticmethod
    def valida_base_menos_desconto(total_item_rateado, desconto, base_calculo):
        base_liquida = float(base_calculo) - float(desconto)
        base_liquida = round(base_liquida, 2)
        if float(base_liquida) == float(total_item_rateado):
            return base_liquida
        return False

    def set_base_item_tributacao(self, nf_tributacao_id, base_liquida):
        try:
            sql = """
                  UPDATE fiscal_nf_item_tributacao SET vr_base_calculo = %s WHERE id = %d
                  """ % (base_liquida, nf_tributacao_id)
            self.cr.execute(sql)
            self.conn.commit()
            return True
        except:
            self.conn.rollback()
            return False


class Principal():
    def __init__(self):
        pass
    acertabase = AcertaBase()
    dados = acertabase.get_nf_item_trib_id()
    log = open('/tmp/log_acerto_desconto_base.log', 'w')
    for dado in dados:
        nf_tributacao_id = dado.get('nf_tributacao_id')
        base_calculo = dado.get('base_calculo')
        desconto = dado.get('desconto')
        total_item_rateado = dado.get('total_item_rateado')
        base_liquida = acertabase.valida_base_menos_desconto(total_item_rateado, desconto,
                                                             base_calculo)

        if base_liquida:
            acertou = acertabase.set_base_item_tributacao(nf_tributacao_id, base_liquida)
            if acertou:
                msg = "NF TRIBUTACAO ID: %s \t BASE ANTIGA: %s \t BASE NOVA: %s \t OK!\n" % \
                      (nf_tributacao_id, base_calculo, base_liquida)
            else:
                msg = "NF TRIBUTACAO ID: %s \t BASE ANTIGA: %s \t BASE NOVA: %s \t FALHOU!\n" % \
                      (nf_tributacao_id, base_calculo, base_liquida)
            log.writelines(msg)
    log.close()
