#!/usr/bin/python
# -*- encoding: utf-8 -*-
import psycopg2
import getpass


class AcertaBase():
    def __init__(self):
        self.database = str(raw_input('Nome do banco: '))
        self.host = str(raw_input('IP do banco: '))
        self.user = str(raw_input('Usuario do banco: '))
        self.password = getpass.getpass('Senha do banco: ')
        self.port = raw_input('Porta do banco: ')
        self.conn = self.new_connection()
        self.cr = self.conn.cursor()
        self.data_inicio = raw_input('Informe a inicial: (Ex: 2014-11-01): ')
        self.data_final = raw_input('Informe a inicial: (Ex: 2014-11-30): ')
        self.company_id = raw_input('Informe o company_id: ')

    def new_connection(self):
        if not self.password:
            self.password = None
        if not self.port:
            self.port = 5432
        try:
            conn = psycopg2.connect(database=self.database, user=self.user, password=self.password,
                                    host=self.host, port=self.port, connect_timeout=3)
            return conn
        except:
            print 'Nao foi possivel conectar ao banco de dados!'
            exec exit()

    @staticmethod
    def dictfetchall(cr):
        """Retorna todas as linhas de um cursor como um dicionario"""
        desc = cr.description
        return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]

    def get_mov_base_difere_vr_total(self):
        sql = """
                SELECT emi.id as mov_itens_id,
                       fn.id  as tributacao_itens_id,
                       emi.vr_total_item_rateado,
                       emi.vr_acrescimos,
                       emi.vr_desconto,
                       fn.vr_base_calculo + fn.vr_reducao_base AS vr_base_calculo,
                       fn.aliquota
                FROM   estoque_movimento_itens emi
                       INNER JOIN ecf_cupons_capa ecc
                               ON ecc.id = emi.cupom_id
                       INNER JOIN fiscal_nf_item_tributacao fn
                               ON fn.movimento_itens_id = emi.id
                       INNER JOIN fiscal_encargo fe
                               ON fe.id = fn.encargo_id
                WHERE  ecc.date_cupom BETWEEN '%s' AND '%s'
                       AND ecc.company_id = {company_id}
                       AND fe.sigla = 'ICMS'
                       AND emi.vr_total_item_rateado <> fn.vr_base_calculo
        """ % (self.data_inicio, self.data_final)
        self.cr.execute(sql.format(company_id=self.company_id))
        dados = self.dictfetchall(self.cr)
        return dados

    @staticmethod
    def valida_diferenca(vr_total_item, vr_base_calculo, vr_desconto, vr_acrescimo):
        vr_base_liquida = vr_base_calculo + vr_acrescimo - vr_desconto
        if vr_total_item == vr_base_liquida:
            return vr_base_liquida
        else:
            return vr_total_item

    def set_base_calculo(self, base_calculo_liquida, tributacao_item_id):
        sql = """
        UPDATE fiscal_nf_item_tributacao SET vr_base_calculo = %s WHERE id = %s
        """ % (base_calculo_liquida, tributacao_item_id)
        try:
            self.cr.execute(sql)
            self.cr.connection.commit()
            return True
        except:
            self.conn.rollback()
            return False

    def set_vr_encargo(self, vr_encargo, tributacao_item_id):
        sql = """
        UPDATE fiscal_nf_item_tributacao SET vr_encargo = %s WHERE id = %s
        """ % (vr_encargo, tributacao_item_id)
        try:
            self.cr.execute(sql)
            self.cr.connection.commit()
            return True
        except:
            self.conn.rollback()
            return False

    def close_connection(self):
        self.conn.close()


class Principal():
    def __init__(self):
        pass
    acertabase = AcertaBase()
    mov_com_diff = acertabase.get_mov_base_difere_vr_total()
    acertados = 0
    diff_nao_bate = 0
    erro = 0
    for mov in mov_com_diff:
        tributacao_itens_id = mov['tributacao_itens_id']
        vr_total_item_rateado = mov['vr_total_item_rateado']
        vr_base_calculo = mov['vr_base_calculo']
        vr_acrescimo = mov['vr_acrescimos']
        vr_desconto = mov['vr_desconto']
        aliquota = mov['aliquota']
        base_calculo_liquida = acertabase.valida_diferenca(vr_total_item_rateado, vr_base_calculo,
                                                           vr_desconto, vr_acrescimo)
        if base_calculo_liquida:
            acertou_base = acertabase.set_base_calculo(base_calculo_liquida, tributacao_itens_id)
            if aliquota:
                vr_encargo = (base_calculo_liquida * aliquota) / 100
                vr_encargo = float('%.2f' % vr_encargo)
                acertou_encargo = acertabase.set_vr_encargo(vr_encargo, tributacao_itens_id)
            if aliquota:
                vr_encargo = (base_calculo_liquida * aliquota) / 100
                acertou_encargo = acertabase.set_vr_encargo(vr_encargo, tributacao_itens_id)
            if acertou_base:
                acertados += 1
            else:
                erro += 1
        else:
            diff_nao_bate += 1
        print 'TRIBUTACAO ID: %s \t VALOR: %s \tBASE: %s \tDESCONTO: %s \tACRESCIMO: %s' % (
            tributacao_itens_id, vr_total_item_rateado, vr_base_calculo, vr_desconto, vr_acrescimo
        )
    print 'CORRIGIDOS: %s\t\t DIFF NAO E ACRES/DESC: %s \t\tNAO CORRIDOS POR ERRO : %s' % (
        acertados, diff_nao_bate, erro)
    acertabase.close_connection()



