#!/usr/bin/python
# -*- encoding: utf-8 -*-

import psycopg2
from getpass import getpass
from openerp.osv.fields import float_round


def dictfetchall(cr):
    """Retorna todas as linhas de um cursor como um dicionario"""
    desc = cr.description
    return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]


def get_dados(cr, dt_ini, dt_fim):
    sql = """
            SELECT emi.id                      AS emi_id,
                   fisnit.id                   AS fisnit_id,
                   emi.vr_total_item,
                   emi.vr_total_item_rateado,
                   emi.vr_desconto,
                   emi.vr_acrescimos,
                   fisnit.vr_base_calculo,
                   fisnit.aliquota,
                   fisnit.vr_encargo,
                   fe.sigla
            FROM   estoque_movimento_itens emi
                   INNER JOIN fiscal_nf_item_tributacao fisnit
                           ON fisnit.movimento_itens_id = emi.id
                   INNER JOIN fiscal_encargo fe
                           ON fe.id = fisnit.encargo_id
                   INNER JOIN ecf_cupons_capa ecc
                           ON ecc.id = emi.cupom_id
            WHERE  ecc.date_cupom BETWEEN '{dt_ini}' AND '{dt_fim}'
                   AND sigla IN ('ICMS', 'PIS', 'COFINS')
                   AND ( emi.vr_desconto > 0
                          OR emi.vr_acrescimos > 0 )
            ORDER  BY emi.id
    """.format(dt_ini=dt_ini, dt_fim=dt_fim)
    cr.execute(sql)
    return dictfetchall(cr)


def set_dados(cr, fisnit_id, vr_base, vr_encargo):
    sql = """
            UPDATE fiscal_nf_item_tributacao
            SET    vr_base_calculo = {vr_base_calculo},
                   vr_encargo = {vr_encargo}
            WHERE  id = {fisnit_id}
    """.format(vr_base_calculo=vr_base, vr_encargo=vr_encargo, fisnit_id=fisnit_id)
    try:
        cr.execute(sql)
        cr.connection.commit()
    except:
        cr.connection.rollback()
        raise


if __name__ == '__main__':
    host = raw_input('IP do banco: ')
    db = raw_input('Nome do banco: ')
    user = raw_input('Usuario do banco: ')
    password = getpass('Senha do banco: ')
    port = raw_input('Porta do banco: ')
    dt_ini = raw_input('Data inicio: ')
    dt_fim = raw_input('Data final: ')
    try:
        con = psycopg2.connect(database=db, user=user, password=password, port=port, host=host,
                               connect_timeout=3)
        cr = con.cursor()
        dados = get_dados(cr, dt_ini, dt_fim)
        mov_item_ids = map(lambda i: i.get('emi_id'), dados)
        for item_id in mov_item_ids:
            base_icms = 0.00
            vr_pis = 0.0
            vr_cofins = 0.0
            aliquota_cofins = False
            aliquota_pis = False
            fisnit_pis_id = False
            fisnit_cofins_id = False
            for x in filter(lambda i: i.get('emi_id') == item_id, dados):
                if x.get('sigla') == 'ICMS':
                    base_icms = x.get('vr_base_calculo')
                elif x.get('sigla') == 'PIS':
                    aliquota_pis = x.get('aliquota')
                    fisnit_pis_id = x.get('fisnit_id')
                    vr_pis = float_round((base_icms * aliquota_pis) / 100, 2)
                    vr_base_pis = x.get('vr_base_calculo')
                else:
                    aliquota_cofins = x.get('aliquota')
                    fisnit_cofins_id = x.get('fisnit_id')
                    vr_cofins = float_round(base_icms * aliquota_cofins / 100, 2)
                    vr_base_cofins = x.get('vr_base_calculo')
            if vr_base_pis > 0:
                set_dados(cr, fisnit_pis_id, base_icms, vr_pis)
            if vr_base_cofins > 0:
                set_dados(cr, fisnit_cofins_id, base_icms, vr_cofins)
    except:
        raise

