#!/usr/bin/python
# -*- encoding: utf-8 -*-

import psycopg2
from getpass import getpass
from openerp.osv.fields import float_round


def dictfetchall(cr):
    """Retorna todas as linhas de um cursor como um dicionario"""
    desc = cr.description
    return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]


def get_diferenca(cr, dt_ini, dt_fim):
    sql = """
            SELECT ecc.id              AS capa_id,
                   ecc.date_cupom      AS data_cupom,
                   ecc.alteracao       AS alteracao,
                   ecc.tipo_cupom      AS tipo_cupom,
                   ecc.valor_alteracao AS valor_alteracao_capa,
                   Sum(CASE
                         WHEN ecc.alteracao = 'A' THEN eci.acrescimo
                         ELSE eci.desconto
                       END)            AS valor_alteracao_itens
            FROM   ecf_cupons_item eci
                   INNER JOIN ecf_cupons_capa ecc
                           ON ecc.id = eci.capa_id
            WHERE  ecc.date_cupom BETWEEN '{dt_ini}' AND '{dt_fim}'
                   AND ecc.alteracao IN ( 'A', 'D' )
                   AND ecc.tipo_cupom IN ('C', 'V')
            GROUP  BY ecc.id,
                      ecc.alteracao,
                      ecc.valor_alteracao
            HAVING Sum(CASE
                         WHEN ecc.alteracao = 'A' THEN eci.acrescimo
                         ELSE eci.desconto
                       END) <> ecc.valor_alteracao
            ORDER BY ecc.id
    """.format(dt_ini=dt_ini, dt_fim=dt_fim)
    cr.execute(sql)
    return dictfetchall(cr)


def get_dados_rateado(cr, cupom_id):
    sql = """
            SELECT eci.id,
                   eci.preco_total,
                   ecc.total_cupom,
                   Round(Sum(( eci.preco_total * ( ecc.valor_alteracao * 100 ) /
                               ecc.total_cupom ))
                         / 100, 2) AS alteracao_calculado
            FROM   ecf_cupons_item eci
                   INNER JOIN ecf_cupons_capa ecc
                           ON ecc.id = eci.capa_id
            WHERE  eci.capa_id = {capa_id}
            GROUP  BY eci.id,
                      eci.quantidade,
                      eci.preco_total,
                      ecc.total_cupom,
                      ecc.valor_alteracao,
                      ecc.alteracao
            ORDER  BY alteracao_calculado DESC
                """.format(capa_id=cupom_id)
    cr.execute(sql)
    return dictfetchall(cr)


def set_dados(vr_alteracao, alteracao, item_id):
    if alteracao == 'D':
        field = 'desconto'
    else:
        field = 'acrescimo'
    sql = """
            UPDATE ecf_cupons_item
            SET    {field} = {vr_alteracao}
            WHERE  id = {id}
    """.format(field=field, vr_alteracao=vr_alteracao, id=item_id)
    try:
        cr.execute(sql)
        cr.connection.commit()
    except:
        cr.connection.rollback()
        raise


if __name__ == '__main__':
    host = raw_input('IP do banco: ')
    db = raw_input('Nome do banco: ')
    user = raw_input('Usuario do banco: ')
    password = getpass('Senha do banco: ')
    port = raw_input('Porta do banco: ')
    dt_ini = raw_input('Data inicio: ')
    dt_fim = raw_input('Data final: ')
    try:
        con = psycopg2.connect(database=db, user=user, password=password, port=port, host=host,
                               connect_timeout=3)
        cr = con.cursor()
        diferencas = get_diferenca(cr, dt_ini, dt_fim)
        for diferenca in diferencas:
            alteracao = diferenca.get('alteracao')
            valor_alteracao = diferenca.get('valor_alteracao_capa')
            dados_calculados = get_dados_rateado(cr, diferenca.get('capa_id'))
            maior_percentual = False
            vr_alt_maior_percent = False
            for dado in dados_calculados:
                if not maior_percentual:
                    maior_percentual = dado.get('id')
                    vr_alt_maior_percent = dado.get('alteracao_calculado')
                valor_alteracao_item = dado.get('alteracao_calculado')
                set_dados(valor_alteracao_item, alteracao, dado.get('id'))
                valor_alteracao -= valor_alteracao_item

            if valor_alteracao:
                vr_alteracao = vr_alt_maior_percent + valor_alteracao
                if vr_alteracao > 0:
                    set_dados(valor_alteracao + vr_alt_maior_percent, alteracao, maior_percentual)
    except:
        raise
