#!/usr/bin/python
# -*- encoding: utf-8 -*-

import psycopg2
import getpass


class AcertaMovimento():
    """
    Este script foi criado para acertar a movimentacao dos itens de acordo com os totalizadores
    da reducao Z dos caixas. Estas diferencas impactam diretamente nos registros do sped C420,
    C425 e C490.

    Ao executar este script ele ira verificar a diferenca entre os totalizadores da reducao Z
    para os itens na estoque_movimento_itens, caso exista será acrescentado ou decrementado o valor
    da diferenca em um item na estoque_movimento_itens de acordo com a tributacao que contem esta
    diferenca.

    E extremamente importante que ao rodar este script nao exista cupons divergentes e faltantes
    na base de dados, pois uma vez que os valores foram alterados nao tem como serem revertidos.
    """
    def __init__(self):
        self.database = str(raw_input('Nome do banco: '))
        self.host = str(raw_input('IP do banco: '))
        self.user = str(raw_input('Usuario do banco: '))
        self.password = getpass.getpass('Senha do banco: ')
        self.port = raw_input('Porta do banco: ')
        self.conn = self.new_connection()
        self.cr = self.conn.cursor()
        self.data_inicio = str(raw_input('Data inicial no formato aaaa/mm/dd: '))
        self.data_final = str(raw_input('Data final no formato aaaa/mm/dd: '))
        self.company_id = int(raw_input('ID da empresa:    '))

    def new_connection(self):
        if not self.password:
            self.password = None
        if not self.port:
            self.port = 5432
        try:
            conn = psycopg2.connect(database=self.database, user=self.user, password=self.password,
                                    host=self.host, port=self.port, connect_timeout=3)
            return conn
        except:
            print 'Nao foi possivel conectar ao banco de dados!'
            exec exit()

    @staticmethod
    def dictfetchall(cr):
        """Retorna todas as linhas de um cursor como um dicionario"""
        desc = cr.description
        return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]


    def get_ecf_ids(self):
        sql = """
                SELECT ecf_id
                FROM   ecf_cupons_capa
                WHERE  date_cupom >= '%s'
                       AND date_cupom <= '%s'
                       AND company_id = %d
                GROUP  BY ecf_id
                ORDER  BY ecf_id ASC
        """ % (self.data_inicio, self.data_final, self.company_id)
        self.cr.execute(sql)
        ecf_ids = self.cr.fetchall()
        ecf_ids = map(lambda i: i[0], ecf_ids)
        return ecf_ids

    def get_capa_z(self, ecf_ids):
        for ecf_id in ecf_ids:
            sql = """
                    SELECT id,
                           data
                    FROM   ecf_mapa_resumo_capa
                    WHERE  data >= '%s'
                           AND data <= '%s'
                           AND ecf = %d
            """ % (self.data_inicio, self.data_final, ecf_id)
            self.cr.execute(sql)
            dados = self.dictfetchall(self.cr)
            yield dados

    def get_totalizadores_capa(self, capa_id):
        sql = """
                SELECT base_valor            AS vlr_acum_tot,
                       tipo,
                       aliquota,
                       capa_id,
                       (SELECT data
                        FROM   ecf_mapa_resumo_capa
                        WHERE  id = capa_id) AS data,
                       (SELECT ecf
                        FROM ecf_mapa_resumo_capa
                        WHERE id = capa_id) AS ecf_id
                FROM   ecf_mapa_resumo_itens
                WHERE  capa_id = %d
                       AND base_valor > 0
        """ % capa_id
        self.cr.execute(sql)
        dados = self.dictfetchall(self.cr)
        return dados

    def get_totalizadores_itens(self, ecf_id, tipo_tributacao, aliquota, data):
        if tipo_tributacao == 'F':
            tipo_tributacao = 'U'
        sql = """
                SELECT Sum(qtd)     AS qtd,
                       Sum(vl_item) AS vl_item
                FROM   (
                                  SELECT     Sum(mov_itens.quantidade)            AS qtd,
                                             Sum(mov_itens.vr_total_item_rateado) AS vl_item
                                  FROM       estoque_movimento_itens mov_itens
                                  INNER JOIN estoque_produto_embalagem produto_embalagem
                                  ON         mov_itens.produto_embalagem_id = produto_embalagem.id
                                  INNER JOIN estoque_embalagem embalagem
                                  ON         produto_embalagem.embalagem_id = embalagem.id
                                  INNER JOIN fiscal_nf_item_tributacao item_tributacao
                                  ON         item_tributacao.movimento_itens_id = mov_itens.id
                                  INNER JOIN estoque_produto produto
                                  ON         mov_itens.produto_id = produto.id
                                  INNER JOIN estoque_produto_grade grade
                                  ON         mov_itens.grade_id = grade.id
                                  INNER JOIN ecf_cupons_capa cupons_capa
                                  ON         mov_itens.cupom_id = cupons_capa.id
                                  WHERE      cupons_capa.ecf_id = %d
                                  AND        item_tributacao.cst_id IN
                                             (
                                                    SELECT id
                                                    FROM   fiscal_cst
                                                    WHERE  tipo = '%s')
                                  AND        item_tributacao.aliquota = %s
                                  AND        cupons_capa.state = 'V'
                                  AND        cupons_capa.date_cupom = '%s'
                                  AND        cupons_capa.tipo_cupom = 'V'
                                  AND        mov_itens.item_cancelado = false
                                  AND        item_tributacao.encargo_id =
                                             (
                                                    SELECT id
                                                    FROM   fiscal_encargo
                                                    WHERE  sigla ilike'ICMS')
                                  GROUP BY   mov_itens.produto_id,
                                             produto.id,
                                             grade.id,
                                             cupons_capa.tipo_cupom,
                                             mov_itens.item_cancelado
                                  UNION ALL
                                  --Cancelamento
                                  SELECT     sum(-mov_itens.quantidade)            AS qtd,
                                             sum(-mov_itens.vr_total_item_rateado) AS vl_item
                                  FROM       estoque_movimento_itens mov_itens
                                  INNER JOIN estoque_produto_embalagem produto_embalagem
                                  ON         mov_itens.produto_embalagem_id = produto_embalagem.id
                                  INNER JOIN estoque_embalagem embalagem
                                  ON         produto_embalagem.embalagem_id = embalagem.id
                                  INNER JOIN fiscal_nf_item_tributacao item_tributacao
                                  ON         item_tributacao.movimento_itens_id = mov_itens.id
                                  INNER JOIN estoque_produto produto
                                  ON         mov_itens.produto_id = produto.id
                                  INNER JOIN estoque_produto_grade grade
                                  ON         mov_itens.grade_id = grade.id
                                  INNER JOIN ecf_cupons_capa cupons_capa
                                  ON         mov_itens.cupom_id = cupons_capa.id
                                  WHERE      cupons_capa.ecf_id = %d
                                  AND        item_tributacao.cst_id IN
                                             (
                                                    SELECT id
                                                    FROM   fiscal_cst
                                                    WHERE  tipo = '%s')
                                  AND        item_tributacao.aliquota = %s
                                  AND        cupons_capa.state = 'V'
                                  AND        cupons_capa.date_cupom = '%s'
                                  AND        cupons_capa.tipo_cupom = 'C'
                                  AND        mov_itens.item_cancelado = true
                                  AND        item_tributacao.encargo_id =
                                             (
                                                    SELECT id
                                                    FROM   fiscal_encargo
                                                    WHERE  sigla ilike'ICMS')
                                  GROUP BY   mov_itens.produto_id,
                                             produto.id,
                                             grade.id,
                                             cupons_capa.tipo_cupom,
                                             mov_itens.item_cancelado ) AS tmp
        """ % (ecf_id, tipo_tributacao, aliquota, data, ecf_id, tipo_tributacao, aliquota, data)
        self.cr.execute(sql)
        dados = self.dictfetchall(self.cr)
        return dados

    @staticmethod
    def get_diferenca_totalizadores(totalizadores):
        if totalizadores['capa'] > totalizadores['itens']:
            resultado = totalizadores['capa'] - totalizadores['itens']
            diferenca = {'capa': 'maior',
                         'diferenca': resultado}
        elif totalizadores['capa'] < totalizadores['itens']:
            resultado = totalizadores['itens'] - totalizadores['capa']
            diferenca = {'capa': 'menor',
                         'diferenca': resultado}
        else:
            diferenca = False
        return diferenca

    def get_mov_item_id_para_acerto(self, tipo_tributacao, aliquota, data, company_id, ecf_id):
        if tipo_tributacao == 'F':
            tipo_tributacao = 'U'
        sql = """
                SELECT mv_itens.id
                FROM   estoque_movimento_itens mv_itens
                       INNER JOIN ecf_cupons_capa ecc
                               ON ecc.id = mv_itens.cupom_id
                       INNER JOIN fiscal_nf_item_tributacao fn
                               ON fn.movimento_itens_id = mv_itens.id
                       INNER JOIN fiscal_encargo fe
                               ON fe.id = fn.encargo_id
                       INNER JOIN fiscal_tributacao ft
                               ON ft.id = fn.tributacao_id
                       INNER JOIN fiscal_cst fc
                               ON fc.id = fn.cst_id
                WHERE  fe.sigla = 'ICMS'
                       AND fc.tipo = '%s'
                       AND fn.aliquota = %s
                       AND ecc.date_cupom = '%s'
                       AND ecc.tipo_cupom = 'V'
                       AND ecc.state = 'V'
                       AND ecc.company_id = %s
                       AND ecc.ecf_id = %s
                ORDER BY  mv_itens.vr_total_item_rateado DESC
                LIMIT  1
        """ % (tipo_tributacao, aliquota, data, company_id, ecf_id)
        self.cr.execute(sql)
        movimento_item_id = self.cr.fetchall()
        movimento_item_id = map(lambda i: i[0], movimento_item_id)
        return movimento_item_id

    def set_valor_estoque_movimento_itens(self, diferenca, movimento_item_id, operacao,):
        if operacao == 'soma':
            sql = """
                    UPDATE estoque_movimento_itens
                    SET    vr_total_item_rateado = (vr_total_item_rateado + %0.2f)
                    WHERE  id = %d
            """ % (diferenca, movimento_item_id)
            self.cr.execute(sql)
            self.cr.connection.commit()
            sql = """
                    UPDATE estoque_movimento_itens
                    SET    vr_total_item = (vr_total_item + %0.2f)
                    WHERE  id = %d
            """ % (diferenca, movimento_item_id)
            self.cr.execute(sql)
            self.cr.connection.commit()
        else:
            sql = """
                    UPDATE estoque_movimento_itens
                    SET    vr_total_item_rateado = (vr_total_item_rateado - %0.2f)
                    WHERE  id = %d
            """ % (diferenca, movimento_item_id)
            self.cr.execute(sql)
            self.cr.connection.commit()
            sql = """
                    UPDATE estoque_movimento_itens
                    SET    vr_total_item = (vr_total_item - %0.2f)
                    WHERE  id = %d
            """ % (diferenca, movimento_item_id)
            self.cr.execute(sql)
            self.cr.connection.commit()
        return True


    def get_trib_item_id_para_acerto(self, movimento_item_id):
        sql = """
            SELECT fn.id
            FROM   fiscal_nf_item_tributacao fn
                   INNER JOIN fiscal_encargo fe
                           ON fe.id = fn.encargo_id
            WHERE  fn.movimento_itens_id = %d
                   AND fe.sigla = 'ICMS'
        """ % movimento_item_id
        self.cr.execute(sql)
        fiscal_nf_trib_id = self.cr.fetchall()
        fiscal_nf_trib_id = map(lambda i: i[0], fiscal_nf_trib_id)
        return fiscal_nf_trib_id

    def set_valor_fiscal_nf_item_tributacao(self, fis_nf_trib_id, diferenca, aliquota, tipo_tributacao,
                                            operacao):
        if tipo_tributacao == 'F':
            tipo_tributacao = 'U'
        if operacao == 'soma':
            sql = """
                    UPDATE fiscal_nf_item_tributacao
                    SET    vr_base_calculo = vr_base_calculo + %0.2f
                    WHERE id = %d
                  """ % (diferenca, fis_nf_trib_id)
        else:
            sql = """
                    UPDATE fiscal_nf_item_tributacao
                    SET    vr_base_calculo = vr_base_calculo + %0.2f
                    WHERE id = %d
                  """ % (diferenca, fis_nf_trib_id)
        self.cr.execute(sql)
        self.cr.connection.commit()
        if tipo_tributacao == 'T':
            sql = """
                    UPDATE fiscal_nf_item_tributacao
                    SET    vr_encargo = (vr_base_calculo * %0.2f) / 100
                    WHERE id = %d
            """ % (aliquota, fis_nf_trib_id)
            self.cr.execute(sql)
            self.cr.connection.commit()
        return True


class Principal():
    def __init__(self):
        pass

    @staticmethod
    def principal():
        acerto = AcertaMovimento()
        ecf_ids = acerto.get_ecf_ids()
        for i in acerto.get_capa_z(ecf_ids):
            reducao_capa_id = False
            data = False
            if isinstance(i, list):
                reducao_capa_id = i[0].get('id')
                data = i[0].get('data')
            if all([reducao_capa_id, data]):
                totalizadores_capa = acerto.get_totalizadores_capa(reducao_capa_id)
                for x in totalizadores_capa:
                    tipo_totalizador = x.get('tipo')
                    valor_acumulado = x.get('vlr_acum_tot')
                    aliquota = x.get('aliquota')
                    data = x.get('data')
                    ecf_id = x.get('ecf_id')
                    totalizadores_itens = acerto.get_totalizadores_itens(ecf_id, tipo_totalizador,
                                                                         aliquota, data)
                    valor_acumulado_itens = totalizadores_itens[0].get('vl_item')
                    totalizadores = {'capa': valor_acumulado,
                                     'itens': valor_acumulado_itens}
                    diferenca = acerto.get_diferenca_totalizadores(totalizadores)
                    acerto_mv_item = False
                    acerto_nf_item_trib = False
                    vlr_diferenca = 0.0
                    movimento_item_id = False
                    if diferenca:
                        vlr_diferenca = diferenca.get('diferenca')
                        if diferenca.get('capa') == 'maior':
                            operacao = 'soma'
                        else:
                            operacao = 'subtrai'
                        movimento_item_id = acerto.get_mov_item_id_para_acerto(tipo_totalizador,
                                                                               aliquota, data,
                                                                               acerto.company_id,
                                                                               ecf_id)[0]
                        acerto_mv_item = acerto.set_valor_estoque_movimento_itens(vlr_diferenca,
                                                                                  movimento_item_id,
                                                                                  operacao)
                        if acerto_mv_item:
                            fiscal_trib_id = acerto.get_trib_item_id_para_acerto(movimento_item_id)
                            fiscal_trib_id = fiscal_trib_id[0]
                            acerto_nf_item_trib = acerto.set_valor_fiscal_nf_item_tributacao(fiscal_trib_id,
                                                                                             vlr_diferenca, aliquota, tipo_totalizador, operacao)
                    if all([acerto_mv_item, acerto_nf_item_trib]):
                        print "ECF ID: %s\t DATA: %s \t TIPO TOT: %s\t ALIQUOTA: %s \t" \
                              "DIFERENCA: %s\t CAPA: %s \t MOVIMENTO_ITEM_ID: %s" \
                              % (ecf_id, data, tipo_totalizador, aliquota, vlr_diferenca,
                                 str(diferenca.get('capa')).upper(), movimento_item_id)


principal = Principal()
principal.principal()
